'From Squeak 2.0 of May 22, 1998 on 29 May 1998 at 6:16:54 pm'!
"Change Set:		jhmOptionKeyAndDo
Date:			29 May 1998
Author:			John Maloney

Eliminates tests for the option key (which isn't supported
on all platforms) and renames #optionKeyPressed to
#macOptionKeyPressed. Fixes a problem with transforms
when invoking the yellow button menu from Morphic
list views.

Also replaces Object>do: with Object>in: allowing expression
like:

	Float pi in: [:x | x * x]"
!


!Object methodsFor: 'accessing' stamp: 'jm 5/29/1998 14:51'!
in: aBlock
	"Evaluate the given block with the receiver as its argument."

	^ aBlock value: self
! !


!CompositionScanner methodsFor: 'initialize-release' stamp: 'jm 5/29/1998 14:48'!
forParagraph: aParagraph
	"Initialize the receiver for scanning the given paragraph."

	super
		initializeFromParagraph: aParagraph
		clippedBy: aParagraph clippingRectangle.
! !


!ControlManager methodsFor: 'scheduling' stamp: 'jm 5/29/1998 14:19'!
findWindowSatisfying: aBlock
	"Present a menu of window titles, and activate the one that gets chosen"

	| sortAlphabetically controllers listToUse labels index |
	sortAlphabetically _ Sensor shiftPressed.
	controllers _ OrderedCollection new.
	scheduledControllers do: [:controller |
		controller == screenController ifFalse:
			[(aBlock value: controller) ifTrue: [controllers addLast: controller]]].
	controllers size == 0 ifTrue: [^ self].
	listToUse _ sortAlphabetically
		ifTrue: [controllers asSortedCollection: [:a :b | a view label < b view label]]
		ifFalse: [controllers].
	labels _ String streamContents:
		[:strm | 
			listToUse do: [:controller | strm nextPutAll: (controller view label contractTo: 40); cr].
		strm skip: -1  "drop last cr"].
	index _ (PopUpMenu labels: labels) startUp.
	index > 0 ifTrue:
		[self activateController: (controllers at: index)].
! !


!EventRecorder methodsFor: 'as yet unclassified' stamp: 'jm 5/29/1998 14:21'!
testControl: anEvent
	"See if it is a control event for me.  
Control 1 = start recording
Control 2 = stop recording (or playing back)
Control 3 = start playing back"

	anEvent isKeystroke ifFalse: [^ self].
	anEvent controlKeyPressed ifFalse: [^ self].
	anEvent commandKeyPressed ifTrue: [^ self].	"not this"
	anEvent macOptionKeyPressed ifTrue: [^ self].	"not this"
	anEvent shiftPressed ifTrue: [^ self].	"not this"

	anEvent keyCharacter = $1 ifTrue: ["start recording"
		tape ifNil: [tape _ OrderedCollection new].
		state _ #record].
	anEvent keyCharacter = $2 ifTrue: ["stop recording (or playing back)"
		state _ nil].
	anEvent keyCharacter = $3 ifTrue: ["start playing back"
		state _ #play.
		tape ifNotNil: [
			tape do: [:evt | 
				anEvent hand world runStepMethods.
				anEvent hand handleEvent: evt.
				anEvent hand world displayWorld]].
		state _ nil].! !


!HaloMorph methodsFor: 'accessing' stamp: 'jm 5/22/1998 16:28'!
innerTarget

	^ innerTarget
! !


!HandMorph methodsFor: 'event dispatching' stamp: 'jm 5/29/1998 18:08'!
newMouseFocus: aMorphOrNil

	mouseDownMorph _ aMorphOrNil.
	self updateMouseDownTransform.
! !


!InputSensor methodsFor: 'modifier keys' stamp: 'jm 5/29/1998 14:17'!
macOptionKeyPressed
	"Answer whether the option key on the Macintosh keyboard is being held down. Macintosh specific."

	^ self primMouseButtons anyMask: 32! !


!MorphicEvent methodsFor: 'keyboard' stamp: 'jm 5/29/1998 14:16'!
macOptionKeyPressed
	"Answer whether the option key on the Macintosh keyboard was being held down when this event occurred. Macintosh specific."

	^ buttons anyMask: 32! !

!MorphicEvent methodsFor: 'keyboard' stamp: 'jm 5/29/1998 14:20'!
shiftPressed
	"Answer true if the shift key on the keyboard was being held down when this event occurred."

	^ buttons anyMask: 8
! !


!Paragraph methodsFor: 'accessing' stamp: 'jm 5/29/1998 14:48'!
replaceFrom: start to: stop with: aText displaying: displayBoolean
	"Replace the receiver's text starting at position start, stopping at stop, by 
	the characters in aText. It is expected that most requirements for 
	modifications to the receiver will call this code. Certainly all cut's or 
	paste's." 

	| compositionScanner obsoleteLines obsoleteLastLine firstLineIndex lastLineIndex
	startLine stopLine replacementRange visibleRectangle startIndex newLine done
	newStop obsoleteY newY moveRectangle |

	text replaceFrom: start to: stop with: aText.		"Update the text."
	lastLine = 0 ifTrue:
		["if lines have never been set up, measure them and display
		all the lines falling in the visibleRectangle"
		self composeAll.
		displayBoolean ifTrue: [^ self displayLines: (1 to: lastLine)]].

	"save -- things get pretty mashed as we go along"
	obsoleteLines _ lines copy.
	obsoleteLastLine _ lastLine.

	"find the starting and stopping lines"
	firstLineIndex _ startLine _ self lineIndexOfCharacterIndex: start.
	stopLine _ self lineIndexOfCharacterIndex: stop.

	"how many characters being inserted or deleted
		-- negative if aText size is < characterInterval size."
	replacementRange _ aText size - (stop - start + 1).
	"Give ourselves plenty of elbow room."
	compositionRectangle _ compositionRectangle withHeight: (textStyle lineGrid * 9999).
	"build a boundingBox of the actual screen space in question -- we'll need it later"
	visibleRectangle _ (clippingRectangle intersect: compositionRectangle)
							intersect: destinationForm boundingBox.
	compositionScanner _ CompositionScanner new forParagraph: self.		"Initialize a scanner."

	"If the starting line is not also the first line, then measuring must commence from line preceding the one in which characterInterval start appears.  For example, deleting a line with only a carriage return may move characters following the deleted portion of text into the line preceding the deleted line."
	startIndex _ (lines at: firstLineIndex) first.
	startLine > 1
		ifTrue: 	[newLine _ compositionScanner composeLine: startLine - 1
						fromCharacterIndex: (lines at: startLine - 1) first
						inParagraph: self.
				(lines at: startLine - 1) = newLine
					ifFalse:	["start in line preceding the one with the starting character"
							startLine _ startLine - 1.
							self lineAt: startLine put: newLine.
							startIndex _ newLine last + 1]].
	startIndex > text size ifTrue:
		["nil lines after a deletion -- remeasure last line below"
		self trimLinesTo: (firstLineIndex - 1 max: 0).
		text size = 0 ifTrue:
			["entire text deleted -- clear visibleRectangle and return."
			displayBoolean ifTrue: [destinationForm fill: visibleRectangle rule: rule fillColor: self backgroundColor].
			self updateCompositionHeight.
			^self]].

	"Now we really get to it."
	done _ false.
	lastLineIndex _ stopLine.
	[done or: [startIndex > text size]]
		whileFalse: 
		[self lineAt: firstLineIndex put:
			(newLine _ compositionScanner composeLine: firstLineIndex
							fromCharacterIndex: startIndex inParagraph: self).
		[(lastLineIndex > obsoleteLastLine
			or: ["no more old lines to compare with?"
				newLine last <
					(newStop _ (obsoleteLines at: lastLineIndex) last + replacementRange)])
			  	or: [done]]
			whileFalse: 
			[newStop = newLine last
				ifTrue:	["got the match"
						"get source and dest y's for moving the unchanged lines"
						obsoleteY _ self topAtLineIndex: lastLineIndex + 1
									using: obsoleteLines and: obsoleteLastLine.
						newY _ self topAtLineIndex: firstLineIndex + 1.
						stopLine _ firstLineIndex.
						done _ true.
							"Fill in the new line vector with the old unchanged lines.
							Update their starting and stopping indices on the way."
						((lastLineIndex _ lastLineIndex + 1) to: obsoleteLastLine) do:
							[:upDatedIndex | 
							self lineAt: (firstLineIndex _ firstLineIndex + 1) 
								put: ((obsoleteLines at: upDatedIndex)
							  		slide: replacementRange)].
							"trim off obsolete lines, if any"
						self trimLinesTo: firstLineIndex]
				ifFalse:	[lastLineIndex _ lastLineIndex + 1]].
		startIndex _ newLine last + 1.
		firstLineIndex _ firstLineIndex + 1].

	"Now the lines are up to date -- Whew!!.  What remains is to move
	the 'unchanged' lines and display those which have changed."
	displayBoolean   "Not much to do if not displaying"
		ifFalse: [^ self updateCompositionHeight].
	startIndex > text size ifTrue:
		["If at the end of previous lines simply display lines from the line in
		which the first character of the replacement occured through the
		end of the paragraph."
		self updateCompositionHeight.
		self displayLines:
			(startLine to: (stopLine _ firstLineIndex min: lastLine)).
		destinationForm  "Clear out area at the bottom"
			fill: ((visibleRectangle left @ (self topAtLineIndex: lastLine + 1)
						extent: visibleRectangle extent)
					intersect: visibleRectangle)
			rule: rule fillColor: self backgroundColor]
		ifFalse:
		[newY ~= obsoleteY ifTrue:
			["Otherwise first move the unchanged lines within
			the visibleRectangle with a good old bitblt."
			moveRectangle _
				visibleRectangle left @ (obsoleteY max: visibleRectangle top)
					corner: visibleRectangle corner.
			destinationForm copyBits: moveRectangle from: destinationForm
				at: moveRectangle origin + (0 @ (newY-obsoleteY))
				clippingBox: visibleRectangle
				rule: Form over fillColor: nil].

		"Then display the altered lines."
		self displayLines: (startLine to: stopLine).

		newY < obsoleteY
			ifTrue:
			[(self topAtLineIndex: obsoleteLastLine+1 using: obsoleteLines and: obsoleteLastLine) > visibleRectangle bottom
				ifTrue:
				["A deletion may have 'pulled' previously undisplayed lines
				into the visibleRectangle.  If so, display them."
				self displayLines:
					((self lineIndexOfTop: visibleRectangle bottom - (obsoleteY - newY))
						to: (self lineIndexOfTop: visibleRectangle bottom))].
			"Clear out obsolete material at the bottom of the visibleRectangle."
			destinationForm
				fill: ((visibleRectangle left @ ((self bottomAtLineIndex: lastLine) + 1)
						extent: visibleRectangle extent)
					intersect: visibleRectangle)  "How about just corner: ??"
				rule: rule fillColor: self backgroundColor].

		(newY > obsoleteY and: [obsoleteY < visibleRectangle top])
			ifTrue:
				["An insertion may have 'pushed' previously undisplayed lines
				into the visibleRectangle.  If so, display them."
				self displayLines:
					((self lineIndexOfTop: visibleRectangle top)
						to: (self lineIndexOfTop: visibleRectangle top + (newY-obsoleteY)))].

		self updateCompositionHeight]! !

!Paragraph methodsFor: 'composition' stamp: 'jm 5/29/1998 14:48'!
composeAll
	"Compose a collection of characters into a collection of lines."

	| startIndex stopIndex lineIndex maximumRightX compositionScanner |
	lines _ Array new: 32.
	lastLine _ 0.
	maximumRightX _ 0.
	text size = 0
		ifTrue:
			[compositionRectangle _ compositionRectangle withHeight: 0.
			^maximumRightX].
	startIndex _ lineIndex _ 1.
	stopIndex _ text size.
	compositionScanner _ CompositionScanner new forParagraph: self.
	[startIndex > stopIndex] whileFalse: 
		[self lineAt: lineIndex 
				put: (compositionScanner composeLine: lineIndex 
										fromCharacterIndex: startIndex 
										inParagraph: self).
		 maximumRightX _ compositionScanner rightX max: maximumRightX.
		 startIndex _ (lines at: lineIndex) last + 1.
		 lineIndex _ lineIndex + 1].
	self updateCompositionHeight.
	self trimLinesTo: lineIndex - 1.
	^ maximumRightX! !


!PinMorph methodsFor: 'events' stamp: 'jm 5/29/1998 14:33'!
handlesMouseDown: evt

	^ (evt yellowButtonPressed | evt blueButtonPressed) not
! !


!PluggableListMorph methodsFor: 'events' stamp: 'jm 5/29/1998 14:13'!
mouseDown: event onItem: aMorph

	event yellowButtonPressed ifTrue: [
		event shiftPressed
			ifTrue: [^ self shiftedYellowButtonActivity]
			ifFalse: [^ self yellowButtonActivity]].
	model okToChange ifFalse: [^ self].  "No change if model is locked"
	((autoDeselect == nil or: [autoDeselect]) and: [aMorph == selectedMorph])
		ifTrue: [self setSelectedMorph: nil]
		ifFalse: [self setSelectedMorph: aMorph].
! !


!SketchMorph methodsFor: 'other' stamp: 'jm 5/22/1998 16:30'!
addOptionalHandlesTo: aHalo box: box

	| aHandle |
	aHandle _ aHalo addHandleAt: box rightCenter color: Color lightGray.
	aHandle on: #mouseDown send: #editDrawing to: aHalo innerTarget.
! !


!StringHolder methodsFor: 'code pane menu' stamp: 'wod 5/29/1998 16:35'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !


!SystemDictionary methodsFor: 'browsing' stamp: 'jm 5/29/1998 14:30'!
browseMethodsWithString: aString
	"Launch a browser on all methods that contain string literals with aString as a substring. The search is case-sensitive, unless the shift key is pressed, in which case the search is case-insensitive (and consequently somewhat slower)."

	| caseBlind testString suffix |
	(caseBlind _ Sensor controlKeyPressed)
		ifTrue:
			[testString _ aString asLowercase.
			suffix _ ' (case-blind)']
		ifFalse:
			[testString _ aString.
			suffix _ ' (case-sensitive)'].
	self browseAllSelect:
			[:method |
				method  hasLiteralSuchThat: [:lit |
					lit class == String and:
					[lit includesSubstring: testString caseSensitive: caseBlind not]]]
		name:  'Methods with string ''', aString, '''', suffix
		autoSelect: aString.
! !


!TextMorphForEditView methodsFor: 'all' stamp: 'jm 5/29/1998 14:15'!
mouseDown: event

	event yellowButtonPressed ifTrue: [
		event shiftPressed
			ifTrue: [^ editView shiftedYellowButtonActivity]
			ifFalse: [^ editView yellowButtonActivity]].
	^ super mouseDown: event
! !


!TranscriptStream methodsFor: 'all' stamp: 'di 5/29/1998 17:13'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !


Object removeSelector: #do:!
CompositionScanner removeSelector: #in:!
HandMorph removeSelector: #mouseDownRecipient:!
InputSensor removeSelector: #optionKeyPressed!
MorphicEvent removeSelector: #optionKeyPressed!
