'From Squeak 2.4c of May 10, 1999 on 26 July 1999 at 3:06:40 pm'!
"Change Set:		SundryFixes
Date:			16 July 1999
Author:			Dan Ingalls

Fix to WaveEditor showEnvelope suggested by Matt Rosen.
Fix to Semaphore smartInspect suggested by Bob Arning.
Enhancement to fileList supporting background images, submitted by Ian Trudel.
Fix to Parser temporaries submitted by Bob Arning and Leandro Caniglia.
"!


!FileList methodsFor: 'file list menu' stamp: 'di 7/16/1999 17:04'!
itemsForFileEnding: suffix
	| labels lines selectors |
	labels _ OrderedCollection new.
	lines _ OrderedCollection new.
	selectors _ OrderedCollection new.
	(suffix = 'bmp') | (suffix = 'gif') | (suffix = 'jpg') | (suffix = 'form') | (suffix = '*') ifTrue:
		[labels addAll: #('open image in a window' 'read image into ImageImports'
						 'open image as background').
		selectors addAll: #(openImageInWindow importImage openAsBackground)].
	(suffix = 'morph') | (suffix = 'morphs') | (suffix = 'sp') | (suffix = '*') ifTrue:
		[labels add: 'load as morph'.
		selectors add: #openMorphFromFile.
		labels add: 'load as project'.
		selectors add: #openProjectFromFile].
	(suffix = 'bo') | (suffix = '*') ifTrue:[
		labels add: 'load as book'.
		selectors add: #openBookFromFile].
	(suffix = 'mid') | (suffix = '*') ifTrue:
		[labels add: 'play midi file'.
		selectors add: #playMidiFile].
	(suffix = 'movie') | (suffix = '*') ifTrue:
		[labels add: 'open as movie'.
		selectors add: #openAsMovie].
	(suffix = 'st') | (suffix = 'cs') | (suffix = '*') ifTrue:
		[suffix = '*' ifTrue: [lines add: labels size].
		labels addAll: #('fileIn' 'file into new change set' 'browse changes' 'browse code' 'broadcast as update').
		lines add: labels size - 1.
		selectors addAll: #(fileInSelection fileIntoNewChangeSet browseChanges browseFile putUpdate)].
	(suffix = 'swf') | (suffix = '*') ifTrue:[
		labels add:'open as Flash'.
		selectors add: #openAsFlash].
	(suffix = 'ttf') | (suffix = '*') ifTrue:[
		labels add: 'open true type font'.
		selectors add: #openAsTTF].
	(suffix = 'gz') | (suffix = '*') ifTrue:[
		labels addAll: #('view decompressed' 'decompress to file').
		selectors addAll: #(viewGZipContents saveGZipContents)].
	(suffix = '3ds') | (suffix = '*') ifTrue:[
		labels add: 'Open 3DS file'.
		selectors add: #open3DSFile].
	(suffix = 'tape') | (suffix = '*') ifTrue:
		[labels add: 'open for playback'.
		selectors add: #openTapeFromFile].
	(suffix = '*') ifTrue:
		[labels addAll: #('generate HTML').
		lines add: labels size - 1.
		selectors addAll: #(renderFile)].
	^ Array with: labels with: lines with: selectors! !

!FileList methodsFor: 'file list menu' stamp: 'IT 6/14/1999 14:24'!
openAsBackground
"Set an image as a background image. Support Squeak's common file format 
 (GIF, JPG, Form stoteOn: (run coded) and BMP)"
| image |
image _ Form fromFileNamed: self fullName.
World
     ifNil: [ScheduledControllers screenController model form: image. 
             Display restoreAfter: [].]
     ifNotNil: [World color: (InfiniteForm with: image)].! !


!Parser methodsFor: 'expression types' stamp: 'di 7/26/1999 14:50'!
temporaries
	" [ '|' (variable)* '|' ]"
	| vars theActualText |
	(self match: #verticalBar) ifFalse: 
		["no temps"
		tempsMark _ prevMark + prevToken.
		tempsMark > 0 ifTrue:
			[theActualText _ source contents.
			[tempsMark < theActualText size and: [(theActualText at: tempsMark) isSeparator]]
				whileTrue: [tempsMark _ tempsMark + 1]].
			^ #()].
	vars _ OrderedCollection new.
	[hereType == #word]
		whileTrue: [vars addLast: (encoder bindTemp: self advance)].
	(self match: #verticalBar) ifTrue: 
		[tempsMark _ prevMark.
		^ vars].
	^ self expected: 'Vertical bar'! !


!Semaphore methodsFor: 'initialize-release' stamp: 'di 7/16/1999 16:54'!
smartInspect
	"Smart does the wrong thing for semaphores"

	^ self inspect! !


!WaveEditor methodsFor: 'menu' stamp: 'di 7/16/1999 08:30'!
showEnvelope
	"Show an envelope wave constructed by collecting the maximum absolute value of the samples in fixed-size time windows of mSecsPerQuantum."

	| data mSecsPerQuantum samplesPerQuantum result endOfQuantum maxThisQuantum s nSamples |
	data _ graph data.
	mSecsPerQuantum _ 10.
	samplesPerQuantum _ (mSecsPerQuantum / 1000.0) * self samplingRate.
	result _ WriteStream on: (Array new: data size // samplesPerQuantum).
	endOfQuantum _ samplesPerQuantum.
	maxThisQuantum _ 0.
	nSamples _ (data isKindOf: SoundBuffer)
		ifTrue: [data monoSampleCount]
		ifFalse: [data size].
	1 to: nSamples do: [:i |
		i asFloat > endOfQuantum ifTrue: [
			result nextPut: maxThisQuantum.
			maxThisQuantum _ 0.
			endOfQuantum _ endOfQuantum + samplesPerQuantum].
		s _ data at: i.
		s < 0 ifTrue: [s _ 0 - s].
		s > maxThisQuantum ifTrue: [maxThisQuantum _ s]].
	WaveEditor openOn: result contents.
! !


