'From Squeak 2.5 of August 6, 1999 on 27 September 1999 at 10:37:10 am'!
"Change Set:		pptToBook-mjg
Date:			27 September 1999
Author:			Mark Guzdial

Provides two new menu items to BookMorphs which allow you to load PowerPoint presentations into a BookMorph, one slide per page.

Here's how to do it:
- Save your PowerPoint presentation as GIF.  This saves each slide as a
file named Slide1, Slide2, etc. in a new directory under the name you
specify.
- Create a new BookMorph.
- Choose from the BookMorph menu 'Load PPT images from Slide #1'
- You'll get the StandardFileMenu dialog (cool widget, Andrew!!).  Select Slide1

That's it -- all the images will get loaded into your book.  

Caution: you sometimes don't get the result that you might like.  PowerPoint saves out transparent colors where it expects the background color to show through.  If your page color is still the default white, your slides won't look right.  So, I also added a BookMorph menu item 'set pages background color'.  Choose that, pick your color, and all the pages will get their color set appropriately.  Note that at present this uses the old 'Color fromUser' rather than the latest color-picker."!


!BookMorph methodsFor: 'menu' stamp: 'sw 9/27/1999 10:33'!
invokeBookMenu
	"Invoke the book's control panel menu."
	| aMenu hand controlsShowing |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addList:
		#(('sort pages'			sortPages)
		('uncache page sorter'	uncachePageSorter)).
	(self hasProperty: #dontWrapAtEnd)
		ifTrue: [aMenu add: 'wrap after last page' selector: #setWrapPages: argument: true]
		ifFalse: [aMenu add: 'stop at last page' selector: #setWrapPages: argument: false].
	aMenu addList:
		#(('make bookmark'		bookmarkForThisPage)
		('make thumbnail'		thumbnailForThisPage)).
	controlsShowing _ self hasSubmorphWithProperty: #pageControl.
	controlsShowing
		ifTrue:
			[aMenu add: 'hide page controls' action: #hidePageControls.
			aMenu add: 'fewer page controls' action: #fewerPageControls]
		ifFalse:
			[aMenu add: 'show page controls' action: #showPageControls].

	aMenu addLine.
	aMenu add: 'sound effect for all pages' action: #menuPageSoundForAll:.
	aMenu add: 'sound effect this page only' action: #menuPageSoundForThisPage:.
	aMenu add: 'visual effect for all pages' action: #menuPageVisualForAll:.
	aMenu add: 'visual effect this page only' action: #menuPageVisualForThisPage:.

	aMenu addLine.
	(self primaryHand classOfPasteBuffer isKindOf: PasteUpMorph class) ifTrue:
		[aMenu add: 'paste book page'   action: #pasteBookPage].

	aMenu add: 'save as new-page prototype' action: #setNewPagePrototype.
	newPagePrototype ifNotNil: [
		aMenu add: 'clear new-page prototype' action: #clearNewPagePrototype].

	aMenu add: (openToDragNDrop ifTrue: ['close'] ifFalse: ['open']) , ' dragNdrop'
			action: #openCloseDragNDrop.
	aMenu add: 'make all pages this size' action: #makeUniformPageSize.
	aMenu add: 'search for text' action: #textSearch.
	aMenu add: 'send all pages to server' action: #savePagesOnURL.
	aMenu add: 'send this page to server' action: #saveOneOnURL.
	aMenu add: 'reload all from server' action: #reload.
	aMenu add: 'copy page url to clipboard' action: #copyUrl.
	aMenu add: 'keep in one file' action: #keepTogether.

	aMenu addLine.
	aMenu add: 'load PPT images from slide #1' action: #loadImagesIntoBook.
	aMenu add: 'background color for all pages...' action: #setPageColor.

	hand _ self world primaryHand.
	aMenu popUpAt: hand position forHand: hand! !

!BookMorph methodsFor: 'menu' stamp: 'sw 9/27/1999 10:29'!
loadImagesIntoBook
	"PowerPoint stores GIF presentations as individual slides named Slide1, Slide2, etc.
	Load these into the book.  mjg 9/99"

	| directory filenumber form newpage |
	directory _ ((StandardFileMenu oldFileFrom: FileDirectory default) ifNil: [^ nil]) directory.
	directory isNil ifTrue: [^ nil].

	"Start loading 'em up!!"
	filenumber _ 1.
	[directory fileExists: 'Slide',(filenumber asString)] whileTrue:
		[form _ Form fromFileNamed: (directory fullNameFor: 'Slide', (filenumber asString)).
		newpage _ PasteUpMorph new extent: (form extent).
		newpage addMorph: (SketchMorph new form: form).
		self pages addLast: newpage.
		filenumber _ filenumber + 1.].

	"After adding all, delete the first page."
	self goToPage: 1.
	self deletePageBasic! !

!BookMorph methodsFor: 'menu' stamp: 'mjg 9/23/1999 15:36'!
setPageColor
	 "Get a color from the user, then set all the pages to that color"

	| usercolor |
	usercolor _ Color fromUser.
	self pages do: [:page | page color: usercolor]! !


