'From Squeak2.6 of 11 October 1999 [latest update: #1696] on 7 December 1999 at 12:56:58 pm'!
"Change Set:		standardTextFont-sw
Date:			7 December 1999
Author:			Scott Wallace

Adds an item to the Appearance menu that allows the user to choose a standard text font, which will be used for menus, lists, browsers, workspaces, and other tools.

Please do not change the standard text font frequently or lightly!!  Preexisting windows may feel the effect only partially, and buttons in existing windows may become strange, etc., and it is possible that subsequent method submissions in preexisting browsers may result in unwanted style changes being recorded in source in some such cases.  Thus, for best results, change the standard system font in an image that has few, or no, tools windows (browsers, workspaces, change sorters, etc.) present.

After changing the system font, you may find it expedient to call
	Utilities reinstateDefaultFlaps
and
    ScriptingSystem resetStandardPartsBin
to get your new look installed in protypical windows, etc.

A TextStyle can now declare any of its fonts to be its default font, without reordering the font array, in a manner that preserves as much as possible of historical smalltalk font-handling.

Most hard-coded special-casing that gave special status to the first font of a text style is removed -- instead, these references are changed to refer to the style's default font.

Choice of the standard window-title font, for both mvc and morphic, is now lodged in Preferences.windowTitleFont.  This is not affected by the standard text font.

Similarly, choice of the font to be used in various buttons and widgets (including the widgets in the title bars of morphic windows) is now lodged in Preferences.standardButtonFont, which is also not affected by the standard text font.

CAVEAT: the order of methods in this update has been manually rearranged to allow a smooth file-in; simply filing out the resulting change-set will *not* result in an update file with the proper order.

Thanks to Robin Gibson for the code that provides WYSIWYG menus for style/font choice in Morphic."!

Object subclass: #TextStyle
	instanceVariableNames: 'fontArray fontFamilySize lineGrid baseline alignment firstIndent restIndent rightIndent tabsArray marginTabsArray leading defaultFontIndex '
	classVariableNames: ''
	poolDictionaries: 'TextConstants '
	category: 'Graphics-Text'!

!TextStyle methodsFor: 'default font' stamp: 'sw 12/6/1999 12:30'!
defaultFontIndex
	^ defaultFontIndex ifNil: [defaultFontIndex _ 1]! !

!BasicButton methodsFor: 'as yet unclassified' stamp: 'sw 12/7/1999 18:14'!
label: aString

	| oldLabel m |
	(oldLabel _ self findA: StringMorph)
		ifNotNil: [oldLabel delete].
	m _ StringMorph contents: aString font: TextStyle defaultFont.
	self extent: m extent + (borderWidth + 6).
	m position: self center - (m extent // 2).
	self addMorph: m.
	m lock! !


!FileList methodsFor: 'initialization' stamp: 'sw 12/7/1999 17:41'!
optionalButtonRow
	| aRow aButton |
	aRow _ AlignmentMorph newRow.
	aButton _ SimpleButtonMorph new target: self.
	aButton color: Color transparent; borderWidth: 1; borderColor: Color black.
	aRow addTransparentSpacerOfSize: (5@0).
	#(	('Name' 		sortByName)
		('Date'			sortByDate)
		('Size'			sortBySize)
		('Changes'		browseChanges)
		('File-in'		fileInSelection)
		('File-in to New'	fileIntoNewChangeSet)
) do:
			[:pair |
				aButton actionSelector: pair second.
				aButton label: pair first.
				aRow addMorphBack: aButton.
				aRow addTransparentSpacerOfSize: (3 @ 0).
				aButton _ aButton fullCopy.
				(pair second == #sortBySize)
					ifTrue:
						[aRow addTransparentSpacerOfSize: (4@0)]].
	^ aRow! !


!CategoryViewer methodsFor: 'header pane' stamp: 'sw 12/7/1999 18:58'!
addHeaderMorph
	| header aFont aButton wrpr |
	header _ AlignmentMorph newRow color: self color; centering: #center.
	aFont _ Preferences standardButtonFont.
	header addMorph: (aButton _ SimpleButtonMorph new label: 'X' font: aFont).
	aButton target: self;
			color:  Color lightRed;
			actionSelector: #delete;
			setBalloonText: 'Delete this pane'.
	header addTransparentSpacerOfSize: 5@5.

	Preferences useCategoryListsInViewers
		ifFalse:
			[header addUpDownArrowsFor: self.
			(wrpr _ header submorphs last) submorphs second setBalloonText: 'previous category'.	
			wrpr submorphs first  setBalloonText: 'next category'].
	header beSticky.
	self addMorph: header.

	namePane _ RectangleMorph newSticky color: Color brown veryMuchLighter.
	namePane borderWidth: 0.
	aButton _ (StringButtonMorph contents: '-----' font: (StrikeFont familyName: #NewYork size: 12)) color: Color black.
	aButton target: self; arguments: Array new; actionSelector: #chooseCategory.
	aButton actWhen: #buttonDown.
	namePane addMorph: aButton.
	aButton position: namePane position.
	namePane align: namePane topLeft with: (bounds topLeft + (50 @ 0)).
	namePane setBalloonText: 'category (click here to choose a different one)'.

	header addMorphBack: namePane.
	(namePane isKindOf: RectangleMorph) ifTrue:
		[namePane addDropShadow.
		namePane owner color: Color gray].

	self categoryChoice: 'basic'! !

!Celeste class methodsFor: 'instance creation' stamp: 'sw 12/6/1999 12:34'!
buildViewsFor: model
	"Answer a collection of window panes for the Celeste user interface."

	| listViewClass textViewClass listFont views v |
	Smalltalk isMorphic
		ifTrue: [
			listViewClass _ PluggableListMorphByItem.
			textViewClass _ PluggableTextMorph]
		ifFalse: [
			listViewClass _ PluggableListViewByItem.
			textViewClass _ PluggableTextView].

	listFont _ StrikeFont allSubInstances
		detect: [:f | (f name beginsWith: 'CourierFixed') and: [f height = 11]]
		ifNone: [TextStyle defaultFont].

	views _ OrderedCollection new.
	v _ listViewClass
		on: model
		list: #categoryList
		selected: #category
		changeSelected: #setCategory:
		menu: #categoryMenu:
		keystroke: #categoriesKeystroke:.
	views add: v.

	v _ listViewClass
		on: model
		list: #tocEntryList
		selected: #tocEntry
		changeSelected: #setTOCEntry:
		menu: #tocMenu:
		keystroke: #tocKeystroke:.
	v font: listFont.
	views add: v.

	v _ textViewClass new
		on: model
		text: #messageText
		accept: #messageText:
		readSelection: nil
		menu: #messageMenu:shifted:.
	v borderWidth: 1.
	model messageTextView: v.
	views add: v.

	^ views! !


!CharacterScanner methodsFor: 'private' stamp: 'sw 12/7/1999 11:33'!
setFont
	"Set the font and other emphasis."
	emphasisCode _ 0.
	kern _ 0.
	indentationLevel _ 0.
	alignment _ textStyle alignment.
	font _ nil.
	(text attributesAt: lastIndex forStyle: textStyle) do: 
		[:att | att emphasizeScanner: self].
	font == nil ifTrue:
		[self setFont: textStyle defaultFontIndex].
	font _ font emphasized: emphasisCode.

	"Install various parameters from the font."
	spaceWidth _ font widthOf: Space. 
	sourceForm _ font glyphs.  "Should only be needed in DisplayScanner"
	height _ font height.			" ditto "
	xTable _ font xTable.
	stopConditions _ font stopConditions.
	stopConditions at: Space asciiValue + 1 put: #space.
	stopConditions at: Tab asciiValue + 1 put: #tab.
	stopConditions at: CR asciiValue + 1 put: #cr.
	stopConditions at: EndOfRun put: #endOfRun.
	stopConditions at: CrossedX put: #crossedX! !


!ControlManager methodsFor: 'private' stamp: 'sw 12/6/1999 23:40'!
unCacheWindows
	scheduledControllers ifNotNil: [scheduledControllers do:
		[:aController | aController view uncacheBits]]! !


!CustomMenu methodsFor: 'private' stamp: 'sw 12/7/1999 12:56'!
build
	"Turn myself into an invokable ActionMenu."

	| stream |
	stream _ WriteStream on: (String new).
	labels do: [:label | stream nextPutAll: label; cr].
	(labels isEmpty) ifFalse: [stream skip: -1].  "remove final cr"
	super labels: stream contents
		font: TextStyle defaultFont
		lines: dividers! !


!FlapTab methodsFor: 'edge' stamp: 'sw 12/6/1999 18:00'!
setEdge: anEdge
	| changedOrientation |
	changedOrientation _ nil.
	self orientation == #vertical
			ifTrue:
				[(#(top bottom) includes: anEdge) ifTrue:
					[changedOrientation _ #horizontal]]
			ifFalse:
				[(#(top bottom) includes: anEdge) ifFalse:
					[changedOrientation _ #vertical]].
	changedOrientation ifNotNil:
		[^ self inform:
'SORRY -- this sort of switch from horizontal to vertical
or vice-versa is not yet permitted.  For now, to achieve
the desired effect, create a new flap and copy over
the elements you want.'].

	self isCurrentlyTextual ifTrue:
		[changedOrientation ifNotNil:
			[self assumeString: (submorphs first contents copyWithout: Character cr) font:  Preferences standardButtonFont orientation: changedOrientation color: self color]].

	self edgeToAdhereTo: anEdge.
	changedOrientation ifNotNil:
		[self transposeParts].
	referent isInWorld ifTrue: [self positionReferent].
	self adjustPositionVisAVisFlap! !

!FlapTab methodsFor: 'textual tabs' stamp: 'sw 12/6/1999 18:00'!
changeTabText
	| reply |

	reply _ FillInTheBlankMorph request: 'new wording for this tab:' initialAnswer: self existingWording centerAt: self cursorPoint inWorld: self world.
	reply size > 0 ifFalse: [^ self].
	self useStringTab: reply.
	submorphs first delete.
	self assumeString: reply font: Preferences standardButtonFont orientation: (Utilities orientationForEdge: edgeToAdhereTo) color: nil! !

!FlapTab methodsFor: 'textual tabs' stamp: 'sw 12/6/1999 18:01'!
useTextualTab
	| stringToUse colorToUse |
	self preserveDetails.
	colorToUse _ self valueOfProperty: #priorColor ifAbsent: [Color green muchLighter].
	submorphs size > 0 ifTrue: [self removeAllMorphs].
	stringToUse _ self valueOfProperty: #priorWording ifAbsent: ['Unnamed Flap'].
	self assumeString: stringToUse font:  Preferences standardButtonFont orientation: self orientation color: colorToUse! !


!HandMorph methodsFor: 'world menu' stamp: 'sw 12/7/1999 10:31'!
appearanceMenu
	"Build the appearance menu for the world."
	| menu screenCtrl |
	screenCtrl _ ScreenController new.
	menu _ (MenuMorph entitled: 'appearance...') defaultTarget: self.
	menu addStayUpItem.
	menu add: 'window colors...' target: Preferences action: #windowSpecificationPanel.
	menu balloonTextForLastItem: 'Lets you specify colors for standard system windows.'.
	menu add: 'standard text font...' target: Utilities action: #chooseSystemFont.
	menu balloonTextForLastItem: 'Choose the default font to use for text, lists, and menus.'.
	menu add: 'text highlight color...' target: Preferences action: #chooseTextHighlightColor.
	menu balloonTextForLastItem: 'Choose which color should be used for text highlighting in Morphic.'.
	menu add: 'insertion point color...' target: Preferences action: #chooseInsertionPointColor.
	menu balloonTextForLastItem: 'Choose which color to use for the text insertion point in Morphic.'.
	menu addLine.

	menu addUpdating: #menuColorString target: Preferences action: #toggleMenuColorPolicy.
	menu balloonTextForLastItem: 'Governs whether menu colors should be derived from the desktop color.'.
	menu addUpdating: #roundedCornersString target: Preferences action: #toggleRoundedCorners.
	menu balloonTextForLastItem: 'Governs whether morphic windows and menus should have rounded corners.'.

	menu addLine.

	menu add: 'full screen on' target: screenCtrl action: #fullScreenOn.
	menu balloonTextForLastItem: 'puts you in full-screen mode, if not already there.'.
	menu add: 'full screen off' target: screenCtrl action: #fullScreenOff.
	menu balloonTextForLastItem: 'if in full-screen mode, takes you out of it.'.
	menu addLine.

	menu add: 'set display depth...' action: #setDisplayDepth.
	menu balloonTextForLastItem: 'choose how many bits per pixel.'.
	menu add: 'set desktop color...' action: #changeBackgroundColor.
	menu balloonTextForLastItem: 'choose a uniform color to use as desktop background.'.
	menu add: 'use texture background' target: self world action: #setStandardTexture.
	menu balloonTextForLastItem: 'apply a graph-paper-like texture background to the desktop.'.

	^ menu! !


!NewParagraph methodsFor: 'editing' stamp: 'sw 12/7/1999 11:34'!
clickAt: clickPoint for: model controller: editor
	"Give sensitive text a chance to fire.  Display flash: (100@100 extent: 100@100)."
	| startBlock action target |
	action _ false.
	startBlock _ self characterBlockAtPoint: clickPoint.
	(text attributesAt: startBlock stringIndex forStyle: textStyle) 
		do: [:att | att mayActOnClick ifTrue:
				[
	"			range _ text rangeOf: att startingAt: startBlock stringIndex.
				boxes _ self selectionRectsFrom: (self characterBlockForIndex: range first) 
							to: (self characterBlockForIndex: range last).
				box _ boxes detect: [:each | each containsPoint: clickPoint].
					*This doesn't work in morphic*
				Need to replace by a highlighting morph that waits for moueUp.
				Utilities awaitMouseUpIn: (editor transform invertRect: box)
						repeating: []
						ifSucceed: [(att actOnClickFor: model) ifTrue: [action _ true]].
	"
				(target _ model) ifNil: [target _ editor morph].
				(att actOnClickFor: target)
					ifTrue: [Sensor waitNoButton.  "FIX THIS"
							action _ true]]].
	^ action! !


!Paragraph methodsFor: 'selecting' stamp: 'sw 12/7/1999 12:26'!
clickAt: clickPoint for: model controller: aController
	"Give sensitive text a chance to fire.  Display flash: (100@100 extent: 100@100)."
	| startBlock action range box boxes |
	action _ false.
	startBlock _ self characterBlockAtPoint: clickPoint.
	(text attributesAt: startBlock stringIndex forStyle: textStyle) 
		do: [:att | att mayActOnClick ifTrue:
				[range _ text rangeOf: att startingAt: startBlock stringIndex forStyle: textStyle.
				boxes _ self selectionRectsFrom: (self characterBlockForIndex: range first) 
							to: (self characterBlockForIndex: range last).
				box _ boxes detect: [:each | each containsPoint: clickPoint]
							ifNone: [^ action].
				Utilities awaitMouseUpIn: box repeating: []
					ifSucceed: [aController terminateAndInitializeAround:
								[(att actOnClickFor: model) ifTrue: [action _ true]]]]].
	^ action! !


!ListParagraph class methodsFor: 'initialization' stamp: 'sw 12/6/1999 13:03'!
initialize  "ListParagraph initialize"
	"Allow different line spacing for lists"
	ListStyle _ TextStyle default copy gridForFont: TextStyle default defaultFontIndex withLead: 1! !


!ParagraphEditor methodsFor: 'accessing' stamp: 'sw 12/7/1999 11:39'!
zapSelectionWith: aText
	"Deselect, and replace the selection text by aText.
	 Remember the resulting selectionInterval in UndoInterval and otherInterval.
	 Do not set up for undo."

	| start stop |
	self deselect.
	start _ startBlock stringIndex.
	stop _ stopBlock stringIndex.
	(aText isEmpty and: [stop > start]) ifTrue:
		["If deleting, then set emphasisHere from 1st character of the deletion"
		emphasisHere _ (paragraph text attributesAt: start forStyle: paragraph textStyle)
					select: [:att | att mayBeExtended]].
	(start = stop and: [aText size = 0]) ifFalse:
		[paragraph
			replaceFrom: start
			to: stop - 1
			with: aText
			displaying: true.
		self computeIntervalFrom: start to: start + aText size - 1.
		UndoInterval _ otherInterval _ self selectionInterval]! !

!ParagraphEditor methodsFor: 'editing keys' stamp: 'sw 12/7/1999 11:37'!
changeEmphasis: characterStream 
	"Change the emphasis of the current selection or prepare to accept characters with the change in emphasis. Emphasis change amounts to a font change.  Keeps typeahead."
	| keyCode attribute oldAttributes index thisSel colors |		 "control 0..9 -> 0..9"
	keyCode _ ('0123456789-=' indexOf: sensor keyboard ifAbsent: [1]) - 1.
	oldAttributes _ paragraph text attributesAt: startBlock stringIndex forStyle: paragraph textStyle.
	thisSel _ self selection.

	"Decipher keyCodes for Command 0-9..."
	(keyCode between: 1 and: 5) ifTrue:
		[attribute _ TextFontChange fontNumber: keyCode].
	keyCode = 6 ifTrue:
		[colors _ #(black magenta red yellow green blue cyan white).
		index _ (PopUpMenu labelArray: colors , #('Do it' 'Link to comment of class' 'Link to definition of class' 'Link to hierarchy of class' 'Link to method' 'URL' 'Copy hidden info')
							lines: (Array with: colors size)) startUp.
		index = 0 ifTrue: [^ true].
		index <= colors size
		ifTrue:
			[attribute _ TextColor color: (Color perform: (colors at: index))]
		ifFalse:
			[index _ index - colors size.
			index = 1 ifTrue: [attribute _ TextDoIt new.
				thisSel _ attribute analyze: self selection asString].
			index = 2 ifTrue: [attribute _ TextLink new. 
				thisSel _ attribute analyze: self selection asString with: 'Comment'].
			index = 3 ifTrue: [attribute _ TextLink new. 
				thisSel _ attribute analyze: self selection asString with: 'Definition'].
			index = 4 ifTrue: [attribute _ TextLink new. 
				thisSel _ attribute analyze: self selection asString with: 'Hierarchy'].
			index = 5 ifTrue: [attribute _ TextLink new. 
				thisSel _ attribute analyze: self selection asString].
			index = 6 ifTrue: [attribute _ TextURL new. 
				thisSel _ attribute analyze: self selection asString].
			index = 7 ifTrue: ["Copy hidden info"
				self copyHiddenInfo.  ^ true].	"no other action"
		thisSel ifNil: [^ true]].	"Could not figure out what to link to"
		].
	(keyCode between: 7 and: 11) ifTrue:
		[sensor leftShiftDown
		ifTrue:
			[keyCode = 10 ifTrue: [attribute _ TextKern kern: -1].
			keyCode = 11 ifTrue: [attribute _ TextKern kern: 1]]
		ifFalse:
			[attribute _ TextEmphasis perform:
					(#(bold italic narrow underlined struckOut) at: keyCode - 6).
			oldAttributes do:
				[:att | (att dominates: attribute) ifTrue: [attribute turnOff]]]].
	(keyCode = 0) ifTrue:
		[attribute _ TextEmphasis normal].

	beginTypeInBlock ~~ nil
		ifTrue:  "only change emphasisHere while typing"
			[self insertTypeAhead: characterStream.
			emphasisHere _ Text addAttribute: attribute toArray: oldAttributes.
			^ true].
	self replaceSelectionWith: (thisSel asText addAttribute: attribute).
	^ true! !

!ParagraphEditor methodsFor: 'editing keys' stamp: 'sw 12/7/1999 11:38'!
copyHiddenInfo
	"In TextLinks, TextDoits, TextColor, and TextURLs, there is hidden
info.  Copy that to the clipboard.  You can paste it and see what it is.
Usually enclosed in <>."

	| attrList |
	attrList _ paragraph text attributesAt: (startBlock stringIndex +
stopBlock stringIndex)//2 forStyle: paragraph text style.
	attrList do: [:attr |
		(attr isKindOf: TextAction) ifTrue:
			[^ self clipboardTextPut: ('<', attr info, '>') asText]].
	"If none of the above"
	attrList do: [:attr |
		attr class == TextColor ifTrue:
			[^ self clipboardTextPut: attr color printString asText]].
	^ self clipboardTextPut: '[No hidden info]' asText! !

!ParagraphEditor methodsFor: 'editing keys' stamp: 'sw 12/7/1999 11:38'!
setEmphasis: emphasisSymbol
	"Change the emphasis of the current selection."

	| oldAttributes attribute |
	oldAttributes _ paragraph text attributesAt: startBlock stringIndex forStyle: paragraph textStyle.
	(emphasisSymbol == #plain) 
		ifTrue:
			[attribute _ TextEmphasis normal]
		ifFalse:
			[attribute _ TextEmphasis perform: emphasisSymbol.
			oldAttributes do:
				[:att | (att dominates: attribute) ifTrue: [attribute turnOff]]].
	self replaceSelectionWith: (self selection addAttribute: attribute)! !

!ParagraphEditor methodsFor: 'typing support' stamp: 'sw 12/7/1999 11:39'!
setEmphasisHere

	emphasisHere _ (paragraph text attributesAt: (startBlock stringIndex-1 max: 1) forStyle: paragraph textStyle)
					select: [:att | att mayBeExtended]! !


!Pen methodsFor: 'operations' stamp: 'sw 12/7/1999 11:01'!
print: str withFont: font
	"Print the given string in the given font at the current heading"
	| lineStart form charStart rowStart scale wasDown bb pix |
	scale _ sourceForm width.
	wasDown _ penDown.
	lineStart _ location.
	str do:
		[:char |
		char = Character cr ifTrue:
			[self place: lineStart; up; turn: 90; go: font height*scale; turn: -90; down]
		ifFalse:
			[form _ font characterFormAt: char.
			charStart _ location.
wasDown ifTrue: [
			self up; turn: -90; go: font descent*scale; turn: 90; down.
			0 to: form height-1 do:
				[:y |
				rowStart _ location.
				bb _ BitBlt bitPeekerFromForm: form.
				pix _ RunArray newFrom:
					((0 to: form width-1) collect: [:x | bb pixelAt: x@y]).
				pix runs with: pix values do:
					[:run :value |
					value = 0
						ifTrue: [self up; go: run*scale; down]
						ifFalse: [self go: run*scale]].
				self place: rowStart; up; turn: 90; go: scale; turn: -90; down].
].
			self place: charStart; up; go: form width*scale; down].
			].
	wasDown ifFalse: [self up]
"
Display restoreAfter:
[Pen new squareNib: 2; color: Color red; turn: 45;
	print: 'The owl and the pussycat went to sea
in a beautiful pea green boat.' withFont: TextStyle defaultFont]
"! !


!PluggableListMorph methodsFor: 'initialization' stamp: 'sw 12/6/1999 13:06'!
list: listOfStrings 
	| morphList handler h loc index |
	scroller removeAllMorphs.
	list _ listOfStrings ifNil: [Array new].
	list isEmpty ifTrue: [self setScrollDeltas.  ^ self selectedMorph: nil].
	"NOTE: we will want a quick StringMorph init message, possibly even
		combined with event install and positioning"
	font ifNil: [font _ TextStyle defaultFont].
	morphList _ list collect:
		[:item | item isText
			ifTrue: [StringMorph contents: item font: font emphasis: (item emphasisAt: 1)]
			ifFalse: [StringMorph contents: item font: font]].

	"Sensitize first morph and copy handler to all the rest"
	morphList first on: #mouseDown send: #mouseDown:onItem: to: self.
	handler _ morphList first eventHandler.
	morphList do: [:m | m eventHandler: handler].

	"Lay items out vertically and install them in the scroller"
	h _ morphList first height "self listItemHeight".
	loc _ 0@0.
	morphList do: [:m | m bounds: (loc extent: 9999@h).  loc _ loc + (0@h)].
	scroller addAllMorphs: morphList.
	index _ self getCurrentSelectionIndex.
	self selectedMorph: ((index = 0 or: [index > morphList size]) ifTrue: [nil] ifFalse: [morphList at: index]).
	self setScrollDeltas.
	scrollBar setValue: 0.0! !


!PopUpMenu class methodsFor: 'instance creation' stamp: 'sw 12/6/1999 17:55'!
labels: aString lines: anArray
	"Answer an instance of me whose items are in aString, with lines drawn 
	after each item indexed by anArray."

	^ self new
		labels: aString
		font: MenuStyle defaultFont
		lines: anArray! !

!PopUpMenu class methodsFor: 'class initialization' stamp: 'sw 12/6/1999 13:08'!
initialize  "PopUpMenu initialize"
	(MenuStyle _ TextStyle default copy)
		gridForFont: TextStyle default defaultFontIndex withLead: 0;
		centered.
	PopUpMenu allSubInstancesDo: [:m | m rescan]! !


!PostscriptCanvas methodsFor: 'drawing support' stamp: 'sw 12/6/1999 13:10'!
defaultFont
	^ TextStyle defaultFont! !


!PostscriptCharacterScanner methodsFor: 'accessing' stamp: 'sw 12/6/1999 13:10'!
defaultFont
	^ TextStyle defaultFont! !

!PostscriptCharacterScanner methodsFor: 'textstyle support' stamp: 'sw 12/7/1999 12:28'!
setTextStylesForOffset:offset
	" default text style "
	font _ self textStyle defaultFont.
	emphasis _ 0.
	foregroundColor _ Color black.

	" set text styles defined at this point, these methods will set instance vars of myself "
	(paragraph text attributesAt: offset forStyle: paragraph textStyle) do: 
		[:att | att emphasizeScanner: self].

	" post-processing of 'emphasis' "
	self setActualFont: (font emphasized: emphasis)! !


!Preferences class methodsFor: 'pref buttons' stamp: 'sw 12/6/1999 18:01'!
buttonRepresenting: prefSymbol wording: aString color: aColor
	"self currentHand attachMorph: (Preferences buttonRepresenting: #balloonHelpEnabled wording: 'Balloon Help' color: nil)"
	"Return a button that controls the setting of prefSymbol.  It will keep up to date even if the preference value is changed in a different place"
	| outerButton aButton aForm |
	(FlagDictionary includesKey: prefSymbol) ifFalse: [self error: 'Unknown preference: ', prefSymbol printString].
	outerButton _ AlignmentMorph newRow height: 24.
	outerButton color:  (aColor ifNil: [Color r: 0.645 g: 1.0 b: 1.0]).
	outerButton hResizing: #spaceFill; vResizing: #shrinkToFit.
	outerButton addMorph: (aButton _ UpdatingThreePhaseButtonMorph new).
	aButton onImage: (aForm _ ScriptingSystem formAtKey: #ButtonOn);
		pressedImage:	aForm;
		offImage: (ScriptingSystem formAtKey: #ButtonOff);
		actionSelector: #togglePreference:;
		extent: 20 @ 20;
		arguments: (Array with: prefSymbol);
		target: Preferences;
		getSelector: prefSymbol.
	outerButton addTransparentSpacerOfSize: (2 @ 0).
	outerButton addMorphBack: (StringMorph contents: aString font: Preferences standardButtonFont) lock.
	outerButton setBalloonText: (Preferences helpMessageForPreference: prefSymbol).

	^ outerButton! !

!Preferences class methodsFor: 'fonts' stamp: 'sw 12/6/1999 16:15'!
standardButtonFont
	"Answer an attractive font to use for buttons"

	^ StrikeFont familyName: #ComicBold size: 16! !

!Preferences class methodsFor: 'fonts' stamp: 'sw 12/7/1999 11:48'!
windowTitleFont
	"Answer the standard font to use for window titles"
	^ StrikeFont familyName: #NewYork size: 15! !

!Preferences class methodsFor: 'fonts' stamp: 'sw 12/7/1999 11:52'!
windowTitleStyle
	"Answer the standard style to use for window titles"
	^  TextStyle named: #NewYork! !


!QuickPrint methodsFor: 'private' stamp: 'sw 12/6/1999 13:10'!
newOn: aForm box: aRectangle font: aStrikeFont color: textColor
	"Initialize myself."
	font _ aStrikeFont ifNil: [TextStyle defaultFont].
	self setFont.
	destForm _ aForm.
	self colorMap: (Bitmap with: 0      "Assumes 1-bit deep fonts"
						with: ((textColor bitPatternForDepth: destForm depth) at: 1)).
	combinationRule _ Form paint.
	self clipRect: aRectangle.
	sourceY _ 0.
	kern _ 0.
	"sourceX is set when selecting the character from the font strike bitmap"! !


!QuickPrint class methodsFor: 'queries' stamp: 'sw 12/6/1999 13:10'!
defaultFont
	^ TextStyle defaultFont! !


!ScreenController methodsFor: 'menu messages' stamp: 'sw 12/6/1999 17:55'!
chooseSystemFont
	Utilities chooseSystemFont! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 12/7/1999 10:31'!
appearanceMenu 
	"Answer the appearance menu to be put up as a screen submenu"

	^ SelectionMenu labelList:
		#(	'window colors...'
			'standard text font...'
			'full screen on'
			'full screen off'
			'set display depth...'
			'set desktop color...' ) 

		lines: #(2 4)
		selections: #(windowSpecificationPanel chooseSystemFont
fullScreenOn fullScreenOff setDisplayDepth setDesktopColor)
"
ScreenController new appearanceMenu startUp
"! !


!ScriptEditorMorph methodsFor: 'buttons' stamp: 'sw 12/6/1999 18:02'!
addDismissButtonTo: aRowMorph
	aRowMorph addMorphBack:
		((SimpleButtonMorph new label: 'X' font: Preferences standardButtonFont)
			target: self;
			color:  Color lightRed;
			actionSelector: #dismiss;
			setBalloonText: 
'Remove this script
from the screen
(you can open it
again from a Viewer)').
	^ aRowMorph! !

!ScriptEditorMorph methodsFor: 'buttons' stamp: 'sw 12/6/1999 18:02'!
buttonRowForEditor
	| r aString aButtonMorph buttonFont aStatusMorph aStatus |
	buttonFont _ Preferences standardButtonFont.
	r _ AlignmentMorph newRow color: Color transparent; inset: 0.

	r addMorphFront:
		(SimpleButtonMorph new
			label: '!!' font: (StrikeFont familyName: #ComicBold size: 16);
			target: self;
			color: Color yellow;
			borderWidth: 0;
			actWhen: #whilePressed;
			actionSelector: #tryMe;
			balloonTextSelector: #tryMe).
	r addTransparentSpacerOfSize: 6@10.

	aString _ playerScripted externalName, ' ', self scriptTitle.
	r addMorphBack:
		(aButtonMorph _ SimpleButtonMorph new useSquareCorners label: aString font: buttonFont; target: self; setNameTo: 'title').
	aButtonMorph actWhen: #buttonDown; actionSelector: #offerScriptorMenu.
	aButtonMorph borderColor: (Color fromRgbTriplet: #(0.065 0.258 1.0)).
	aButtonMorph color: (self isAnonymous ifTrue: [Color blue muchLighter] ifFalse: [ScriptingSystem uniformTileInteriorColor]).
	aButtonMorph balloonTextSelector: #offerScriptorMenu.
	r addTransparentSpacerOfSize: 6@10.

	aStatus _ self scriptInstantiation status.
	r addMorphBack:
			(aStatusMorph _ SimpleButtonMorph new useSquareCorners label: aStatus font: buttonFont;
				setNameTo: 'trigger';
				target: self;
				actWhen: #buttonDown;
				actionSelector: #chooseTrigger;
				setBalloonText: (ScriptingSystem helpStringFor: #chooseTrigger) maxLineLength: 65;
				balloonTextSelector: #chooseTrigger).

	r addTransparentSpacerOfSize: 10@10.
	r addMorphBack:
		(IconicButton new borderWidth: 0;
			labelGraphic: (ScriptingSystem formAtKey: 'AddTest'); color: Color transparent; 
			actWhen: #buttonDown;
			target: self;
			actionSelector: #addYesNoToHand;
			shedSelvedge;
			balloonTextSelector: #addYesNoToHand).
	r addTransparentSpacerOfSize: 12@10.
	self addDismissButtonTo: r.
	self updateStatusMorph: aStatusMorph.
	^ r! !

!ScriptEditorMorph methodsFor: 'buttons' stamp: 'sw 12/6/1999 18:02'!
updateStatusMorph: statusMorph
	"my status button may need to reflect an externally-induced change in status"
	|  statusSymbol colorSelector |
	statusMorph ifNil: [^ self].
	statusSymbol _ self scriptInstantiation status.
	colorSelector _ ScriptingSystem statusColorSymbolFor: statusSymbol.
	statusMorph recolor: (Color perform: colorSelector) muchLighter.
	statusMorph label: statusSymbol asString font: Preferences standardButtonFont! !


!ScriptEvaluatorMorph methodsFor: 'initialization' stamp: 'sw 12/6/1999 18:03'!
initializeFor: aPhraseTileMorph
	| lm aStatus |
	self playerScripted: aPhraseTileMorph actualObject.
	scriptName _ aPhraseTileMorph userScriptSelector.
	self removeAllMorphs.
	self addMorphFront: (lm _ AlignmentMorph newRow).
	lm color: Color transparent; inset: 0.
	lm vResizing: #shrinkWrap.
	lm addMorphBack: (ScriptingSystem tryButtonFor: aPhraseTileMorph).
	lm addTransparentSpacerOfSize: 6@1.
	lm addMorphBack: aPhraseTileMorph beSticky.
	lm addTransparentSpacerOfSize: 4@1.
	
	aStatus _ (playerScripted scriptInstantiationForSelector: scriptName) status.
	lm addMorphBack:
			(SimpleButtonMorph new label: aStatus font: Preferences standardButtonFont;
				setNameTo: 'trigger';
				target: self;
				actWhen: #buttonDown;
				actionSelector: #chooseTrigger).
	lm addTransparentSpacerOfSize: 4@1.
	self addDismissButtonTo: lm.
	self openToDragNDrop: false.
	lm openToDragNDrop: true! !


!StandardSystemView class methodsFor: 'class initialization' stamp: 'sw 12/6/1999 23:42'!
initialize		"StandardSystemView initialize"
	self doCacheBits.
	self setLabelStyle! !

!StandardSystemView class methodsFor: 'class initialization' stamp: 'sw 12/7/1999 11:54'!
setLabelStyle
	"StandardSystemView setLabelStyle"

	LabelStyle _ Preferences windowTitleStyle copy.
	LabelStyle gridForFont: 2 withLead: 0! !


!StandardViewer methodsFor: 'as yet unclassified' stamp: 'sw 12/6/1999 18:03'!
addHeaderMorphWithBarHeight: anInteger includeDismissButton: aBoolean
	| header aFont aButton aTextMorph nail wrpr costs headWrapper inner |
	header _ AlignmentMorph newRow color: self color muchLighter; centering: #center.
	aFont _ Preferences standardButtonFont.
	aBoolean ifTrue:
		[header addMorph: (aButton _ SimpleButtonMorph new label: 'X' font: aFont).
		aButton target: self;
				color:  Color lightRed;
				actionSelector: #dismiss;
				setBalloonText: 'Delete this entire Viewer'.
		header addTransparentSpacerOfSize: 4@1].

	aButton _ IconicButton new borderWidth: 0;
			labelGraphic: (ScriptingSystem formAtKey: #AddCategoryViewer); color: Color transparent; 
			actWhen: #buttonDown;
			target: self;
			actionSelector: #addCategoryViewer;
			setBalloonText: 'click here to add
another category pane';
			shedSelvedge.
	header addMorphBack: aButton.
	header addTransparentSpacerOfSize: 4@1.

	costs _ scriptedPlayer costumes.
	costs ifNotNil:
	[(costs size > 1 or: [costs size = 1 and: [costs first ~~ scriptedPlayer costume]]) ifTrue:
		[header addUpDownArrowsFor: self.
		(wrpr _ header submorphs last) submorphs second setBalloonText: 'switch to previous costume'.	
		wrpr submorphs first  setBalloonText: 'switch to next costume']].	

	aBoolean
		ifTrue:
			[nail _ ThumbnailMorph new objectToView: scriptedPlayer viewSelector: #costume]
		ifFalse:
			[inner _ ImageMorph new image: (ScriptingSystem formAtKey: 'Menu').
			nail _ RectangleMorph new beTransparent extent: inner extent.
			nail addMorph: inner lock].
	nail on: #mouseDown send: #thumbnailMenuEvt:forMorph: to: scriptedPlayer.
	header addMorphBack: nail.
	nail setBalloonText: 'click here to get a menu
that will allow you to
add an instance variable,,
tear off a tile, etc..'.
	nail borderWidth: 3; borderColor: #raised.
	"nail addDropShadow.
	nail owner color: Color gray."

	header addTransparentSpacerOfSize: 5@5.

"	aButton _ SimpleButtonMorph new target: self; actionSelector: #newEmptyScript; label: 'S' font: (aFont _ StrikeFont familyName: #ComicBold size: 16);  color: Color transparent; borderWidth: 0; actWhen: #buttonDown.
	aButton setBalloonText: 'drag from here to
create a new script
for this object'.	
	header addMorphBack: aButton.

	header addTransparentSpacerOfSize: 8@5.

	aButton _ SimpleButtonMorph new target: scriptedPlayer; actionSelector: #addInstanceVariable; label: 'I' font: aFont;  color: Color transparent; borderWidth: 0; actWhen: #buttonUp.
	aButton setBalloonText: 'click here to add
an instance variable
to this object.'.
	header addMorphBack: aButton.

	header addTransparentSpacerOfSize: 5@5."

	scriptedPlayer costume assureExternalName.
	aTextMorph _ UpdatingStringMorph new
		useStringFormat;
		target:  scriptedPlayer;
		getSelector: #getName;
		putSelector: #setName:;
		setNameTo: 'name';
		font: ScriptingSystem fontForNameEditingInScriptor.
	aTextMorph setProperty: #okToTextEdit toValue: true.
	aTextMorph step.
	header  addMorphBack: aTextMorph.
	aTextMorph setBalloonText: 'Click here to edit the player''s name.'.	

	header beSticky.
	anInteger > 0
		ifTrue:
			[headWrapper _ AlignmentMorph newColumn color: self color.
			headWrapper addTransparentSpacerOfSize: (0 @ anInteger).
			headWrapper addMorphBack: header.
			self addMorph: headWrapper]
		ifFalse:
			[self addMorph: header]! !


!StrikeFont methodsFor: 'character shapes' stamp: 'sw 12/7/1999 11:02'!
characterFormAt: character put: characterForm
	"Copy characterForm over the glyph for the argument, character."
	| ascii leftX rightX widthDif newGlyphs |
	ascii _ character asciiValue.
	leftX _ xTable at: ascii + 1.
	rightX _ xTable at: ascii + 2.
	widthDif _ characterForm width - (rightX - leftX).
	widthDif ~= 0 ifTrue:
		["Make new glyphs with more or less space for this char"
		newGlyphs _ Form extent: (glyphs width + widthDif) @ glyphs height.
		newGlyphs copy: (0@0 corner: leftX@glyphs height)
			from: 0@0 in: glyphs rule: Form over.
		newGlyphs copy: ((rightX+widthDif)@0 corner: newGlyphs width@glyphs height)
			from: rightX@0 in: glyphs rule: Form over.
		glyphs _ newGlyphs.
		"adjust further entries on xTable"
		ascii+2 to: xTable size
			do: [:i | xTable at: i put: (xTable at: i) + widthDif]].
	glyphs copy: (leftX @ 0 extent: characterForm extent)
		from: 0@0 in: characterForm rule: Form over
"
| f |  f _ TextStyle defaultFont.
f characterFormAt: $  put: (Form extent: (f widthOf: $ )+10@f height)
"! !


!StringMorph methodsFor: 'accessing' stamp: 'sw 12/6/1999 13:15'!
fontToUse

	| fontToUse |
	fontToUse _ font == nil
		ifTrue: [TextStyle defaultFont]
		ifFalse: [font].
	(emphasis == nil or: [emphasis = 0])
		ifTrue: [^ fontToUse]
		ifFalse: [^ fontToUse emphasized: emphasis]! !

!StringMorph methodsFor: 'accessing' stamp: 'sw 12/6/1999 13:16'!
setWidth: width

	self extent: width @ (font ifNil: [TextStyle defaultFont]) height! !


!SystemWindow methodsFor: 'initialization' stamp: 'sw 12/6/1999 18:04'!
addCloseBox
	self addMorph: (closeBox _ SimpleButtonMorph new borderWidth: 0;
			label: 'X' font: Preferences standardButtonFont; color: Color transparent;
			actionSelector: #delete; target: self; extent: 16@16)! !

!SystemWindow methodsFor: 'initialization' stamp: 'sw 12/7/1999 11:48'!
initialize
	| aFont |
	super initialize.
	labelString ifNil: [labelString _ 'Untitled Window'].
	isCollapsed _ false.
	activeOnlyOnTop _ true.
	paneMorphs _ Array new.
	paneRects _ Array new.
	borderColor _ #raised.
	borderWidth _ 1.
	color _ Color black.
	aFont _ Preferences standardButtonFont.
	stripes _ Array with: (RectangleMorph newBounds: bounds)  "see extent:"
				with: (RectangleMorph newBounds: bounds).
	self addMorph: (stripes first borderWidth: 1).
	self addMorph: (stripes second borderWidth: 2).
	self addMorph: (label _ StringMorph new contents: labelString;
			font: Preferences windowTitleFont emphasis: 1).
	self setLabelWidgetAllowance.
	self addCloseBox.
	self addMenuControl.
	self addMorph: (collapseBox _ SimpleButtonMorph new borderWidth: 0;
			label: 'O' font: aFont; color: Color transparent;
			actionSelector: #collapseOrExpand; target: self; extent: 16@16).
	Preferences noviceMode ifTrue:
		[closeBox ifNotNil: [closeBox setBalloonText: 'close window'].
		menuBox ifNotNil: [menuBox setBalloonText: 'window menu'].
		collapseBox ifNotNil: [collapseBox setBalloonText: 'collapse/expand window']].
	self on: #mouseEnter send: #spawnReframeHandle: to: self.
	self on: #mouseLeave send: #spawnReframeHandle: to: self.
	label on: #mouseDown send: #relabelEvent: to: self.
	self extent: 300@200.
	mustNotClose _ false.
	updatablePanes _ Array new.! !

!SystemWindow methodsFor: 'label' stamp: 'sw 12/7/1999 11:48'!
widthOfFullLabelText
	^ (QuickPrint newOn: Display box: Display boundingBox font: (Preferences windowTitleFont emphasized: 1)) stringWidth: labelString! !


!FlashPlayerWindow methodsFor: 'as yet unclassified' stamp: 'sw 12/6/1999 18:05'!
initialize
	| aFont |
	super initialize.
	aFont _  Preferences standardButtonFont.
	self addMorph: (startButton _ SimpleButtonMorph new borderWidth: 0;
			label: 'play' font: aFont; color: Color transparent;
			actionSelector: #startPlaying; target: self).
	startButton setBalloonText: 'continue playing'.

	self addMorph: (stopButton _ SimpleButtonMorph new borderWidth: 0;
			label: 'stop' font: aFont; color: Color transparent;
			actionSelector: #stopPlaying; target: self).
	stopButton setBalloonText: 'stop playing'.

	startButton submorphs first color: Color blue.
	stopButton submorphs first color: Color red.

	self adjustBookControls! !


!PartsWindow methodsFor: 'as yet unclassified' stamp: 'sw 12/6/1999 18:06'!
initialize
	| aFont aForm |
	super initialize.
	color _ Color white.
	openForEditing _ false.
	aFont _ Preferences standardButtonFont.
	self addMorph: (prevButton _ SimpleButtonMorph new borderWidth: 0;
			label: '<' font: aFont; color: Color transparent;
			setBalloonText: 'previous page';
			actionSelector: #previousPage; target: self; extent: 16@16).
	self addMorph: (nextButton _ SimpleButtonMorph new borderWidth: 0;
			label: '>' font: aFont; color: Color transparent;
			setBalloonText: 'next page';
			actionSelector: #nextPage; target: self; extent: 16@16).
	menuButton _ ThreePhaseButtonMorph new onImage: (aForm _ ScriptingSystem formAtKey: 'OfferToUnlock'); offImage: (ScriptingSystem formAtKey: 'OfferToLock'); pressedImage: (ScriptingSystem formAtKey: 'OfferToLock'); extent: aForm extent; state: #on.
	menuButton target: self; actionSelector: #toggleStatus; actWhen: #buttonUp.
	menuButton setBalloonText: 'open for editing'.
	self addMorph: menuButton.
"
	self addMorph: (menuButton _ SimpleButtonMorph new borderWidth: 0;
			label: '' font: aFont; color: Color transparent;
			actWhen: #buttonDown;
			actionSelector: #invokePartsWindowMenu; target: self; extent: 16@16)."

	self adjustBookControls! !


!PreDebugWindow methodsFor: 'as yet unclassified' stamp: 'sw 12/6/1999 18:06'!
initialize
	| aFont proceedLabel debugLabel aWidth |
	super initialize.
	true "Preferences optionalMorphicButtons" ifFalse:
		[(aWidth _ self widthOfFullLabelText) > 280 ifTrue: [^ self].   "No proceed/debug buttons if title too long"
		aWidth > 210
			ifTrue: "Abbreviated buttons if title pretty long"
				[proceedLabel _ 'p'.
				debugLabel _ 'd']
			ifFalse: "Full buttons if title short enough"
				[proceedLabel _ 'proceed'.
				debugLabel _ 'debug'].
		aFont _  Preferences standardButtonFont.
		self addMorph: (proceedButton _ SimpleButtonMorph new borderWidth: 0;
				label: proceedLabel font: aFont; color: Color transparent;
				actionSelector: #proceed; target: self).
		proceedButton setBalloonText: 'continue execution'.
		self addMorph: (debugButton _ SimpleButtonMorph new borderWidth: 0;
				label: debugLabel font: aFont; color: Color transparent;
				actionSelector: #debug; target: self).
		debugButton setBalloonText: 'bring up a debugger'.

		proceedButton submorphs first color: Color blue.
		debugButton submorphs first color: Color red].

	self adjustBookControls! !


!Text methodsFor: 'accessing' stamp: 'sw 12/7/1999 12:25'!
rangeOf: attribute startingAt: index forStyle: aStyle
	"This is stupid, slow code, but it works"
	| start stop |
	start _ index.
	[start > 1 and: [(self attributesAt: (start - 1) forStyle: aStyle) includes: attribute]]
		whileTrue: [start _ start - 1].
	stop _ index-1.
	[stop < self size and: [(self attributesAt: (stop + 1) forStyle: aStyle) includes: attribute]]
		whileTrue: [stop _ stop + 1].
	^ start to: stop! !

!Text methodsFor: 'emphasis' stamp: 'sw 12/7/1999 12:30'!
attributesAt: characterIndex 
	"Answer the code for characters in the run beginning at characterIndex."
	"NB: no senders any more (supplanted by #attributesAt:forStyle: but retained for the moment in order not to break user code that may exist somewhere that still calls this"
	| attributes |
	self size = 0
		ifTrue: [^ Array with: (TextFontChange new fontNumber: 1)].  "null text tolerates access"
	attributes _ runs at: characterIndex.
	^ attributes! !

!Text methodsFor: 'emphasis' stamp: 'sw 12/7/1999 11:32'!
attributesAt: characterIndex forStyle: aTextStyle
	"Answer the code for characters in the run beginning at characterIndex."
	| attributes |
	self size = 0
		ifTrue: [^ Array with: (TextFontChange new fontNumber: aTextStyle defaultFontIndex)].  "null text tolerates access"
	attributes _ runs at: characterIndex.
	^ attributes! !

!Text methodsFor: 'emphasis' stamp: 'sw 12/7/1999 10:58'!
fontAt: characterIndex withStyle: aTextStyle
	"Answer the fontfor characters in the run beginning at characterIndex."
	| attributes font |
	self size = 0 ifTrue: [^ aTextStyle defaultFont].	"null text tolerates access"
	attributes _ runs at: characterIndex.
	font _ aTextStyle defaultFont.  "default"
	attributes do: 
		[:att | att forFontInStyle: aTextStyle do: [:f | font _ f]].
	^ font! !


!Text class methodsFor: 'instance creation' stamp: 'sw 12/6/1999 14:14'!
fromString: aString 
	"Answer an instance of me whose characters are those of the argument, aString."

	^ self string: aString attribute: (TextFontChange fontNumber: TextStyle default defaultFontIndex)! !


!TextFontChange class methodsFor: 'as yet unclassified' stamp: 'sw 12/6/1999 17:52'!
defaultFontChange
	"Answer a TextFontChange that represents the default font"

	^ self new fontNumber: TextStyle default defaultFontIndex! !


!TextMorphEditor methodsFor: 'mvc compatibility' stamp: 'sw 12/7/1999 12:27'!
zapSelectionWith: aText
	"**overridden to inhibit old-style display"
	| start stop |
	self deselect.
	start _ startBlock stringIndex.
	stop _ stopBlock stringIndex.
	(aText isEmpty and: [stop > start]) ifTrue:
		["If deleting, then set emphasisHere from 1st character of the deletion"
		emphasisHere _ (paragraph text attributesAt: start forStyle: paragraph textStyle)
					select: [:att | att mayBeExtended]].
	(start = stop and: [aText size = 0]) ifFalse:
		[paragraph replaceFrom: start to: stop - 1
			with: aText displaying: false.  "** was true in super"
		self computeIntervalFrom: start to: start + aText size - 1.
		UndoInterval _ otherInterval _ self selectionInterval].

	self userHasEdited  " -- note text now dirty"! !


!TextStyle methodsFor: 'default font' stamp: 'sw 12/6/1999 13:50'!
defaultFontIndex: anIndex
	defaultFontIndex _ anIndex! !

!TextStyle methodsFor: 'accessing' stamp: 'sw 12/6/1999 12:31'!
defaultFont
	^ fontArray at: self defaultFontIndex! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: 'sw 12/6/1999 13:54'!
fontIndexOf: aFont
	^ fontArray indexOf: aFont ifAbsent: [nil]! !

!TextStyle methodsFor: 'object fileIn' stamp: 'sw 12/7/1999 11:58'!
convertfflbafrrtml0: varDict fflbafrrtmld0: smartRefStrm
	"These variables are automatically stored into the new instance ('fontArray' 'fontFamilySize' 'lineGrid' 'baseline' 'alignment' 'firstIndent' 'restIndent' 'rightIndent' 'tabsArray' 'marginTabsArray' 'leading' ).
	This method is for additional changes. Use statements like (foo _ varDict at: 'foo')."

	"New variables: ('defaultFontIndex' )  If a non-nil value is needed, please assign it."
! !

!TextStyle methodsFor: 'object fileIn' stamp: 'sw 12/6/1999 16:23'!
convertfflbafrrtml0: varDict fflbafrrtmldt0: smartRefStrm
	"These variables are automatically stored into the new instance ('fontArray' 'fontFamilySize' 'lineGrid' 'baseline' 'alignment' 'firstIndent' 'restIndent' 'rightIndent' 'tabsArray' 'marginTabsArray' 'leading' ).
	This method is for additional changes. Use statements like (foo _ varDict at: 'foo')."

	"New variables: ('defaultFontIndex' 'titleFontIndex' )  If a non-nil value is needed, please assign it."
! !


!TextStyle class methodsFor: 'constants' stamp: 'sw 12/6/1999 12:32'!
defaultFont
	"Answer the default system font"

	^ DefaultTextStyle defaultFont! !


!UpdatingStringMorph methodsFor: 'editing' stamp: 'sw 12/7/1999 11:45'!
setFontStyle
	| aList reply style |
	aList _ (TextConstants select: [:anItem | anItem isKindOf: TextStyle])
			keys asOrderedCollection.
	reply _ (SelectionMenu labelList: aList selections: aList) startUp.
	reply ~~ nil ifTrue:
		[(style _ TextStyle named: reply) ifNil: [self beep. ^ true].
		self font: (style defaultFont)]! !


!Utilities class methodsFor: 'summer97 additions' stamp: 'sw 11/30/1999 16:43'!
actualTextStyles
	| aDict |
	"Utilities actualTextStyles"

	"Answer dictionary whose keys are the names of styles in the system and whose values are the actual styles"

	aDict _ TextConstants select: [:thang | thang isKindOf: TextStyle].
	aDict removeKey: #DefaultTextStyle.
	^ aDict! !

!Utilities class methodsFor: 'flaps' stamp: 'sw 12/6/1999 18:07'!
menuFlap
	| aFlap aFlapTab aHolder verticalHolder aMenu |
	aFlap _ PasteUpMorph newSticky color: Color transparent; extent: self currentWorld width @ 264; borderWidth: 0; padding: 0.

	aFlapTab _ FlapTab new referent: aFlap.
	aFlapTab color: Color brown lighter.
	aFlapTab assumeString: 'Menus' font: Preferences standardButtonFont orientation: #horizontal color: Color blue muchLighter.
	aFlapTab setToPopOutOnMouseOver: true.
	aFlapTab edgeToAdhereTo: #top; inboard: false.

	aFlapTab position: ((Display width - aFlapTab width) // 2) @ 0.
	aFlap setProperty: #flap toValue: true.
	aFlap color: (Color blue muchLighter alpha: 0.6).
	aFlap extent: self currentWorld width @ 267.
	aHolder _ AlignmentMorph newRow beSticky beTransparent.

	#(openMenu helpMenu windowsMenu (changesMenu debugMenu ) (playfieldMenu scriptingMenu )) do:
		[:elem |
			(elem isKindOf: Array)
				ifTrue:
					[verticalHolder _ AlignmentMorph newColumn beSticky beTransparent.
					verticalHolder hResizing: #shrinkWrap; inset: 0; centering: #center.
					elem do:
						[:aMenuSymbol |
							verticalHolder addMorphBack: ((aMenu _ self currentHand perform: aMenuSymbol) beSticky; stayUp: true).
							aMenu beSticky.
							aMenu borderWidth: 1.
							aMenu submorphs second delete].
					aHolder addMorphBack: verticalHolder]
				ifFalse:
					[aHolder addMorphBack: ((aMenu _ self currentHand perform: elem) beSticky; stayUp: true).
					aMenu submorphs second delete.
					aMenu beSticky.
					aMenu borderWidth: 1]].

	aFlap addMorphBack: aHolder.

	^ aFlapTab! !

!Utilities class methodsFor: 'flaps' stamp: 'sw 12/6/1999 18:07'!
newFlapTitled: aString onEdge: anEdge
	| aFlap aFlapTab  orientation |

	aFlap _ PasteUpMorph newSticky.
	aFlapTab _ FlapTab new referent: aFlap.
	orientation _ (#(left right) includes: anEdge)
		ifTrue:	[#vertical]
		ifFalse:	[#horizontal].
	aFlapTab assumeString: aString font: Preferences standardButtonFont orientation: orientation color: Color veryLightGray.
	aFlapTab edgeToAdhereTo: anEdge; inboard: false.

	anEdge == #left ifTrue:
		[aFlapTab position: (0 @ 0).
		aFlap extent: (200 @ self currentWorld height)].
	anEdge == #right ifTrue:
		[aFlapTab position: ((self currentWorld width - aFlapTab width) @ 0).
		aFlap extent: (200 @ self currentWorld height)].
	anEdge == #top ifTrue:
		[aFlapTab position: (50 @ 0).
		aFlap extent: (self currentWorld width @ 200)].
	anEdge == #bottom ifTrue:
		[aFlapTab position: (50 @ (self currentWorld height - aFlap height)).
		aFlap extent: (self currentWorld width @ 200)].

	aFlap setProperty: #flap toValue: true.
	aFlap color: (Color veryLightGray muchLighter).

	^ aFlapTab! !

!Utilities class methodsFor: 'flaps' stamp: 'sw 12/7/1999 12:51'!
replaceMenuFlap
	"if there is a global menu flap, replace it with an updated one."

	| aFlapTab |
	aFlapTab _ self globalFlapTabsIfAny detect:
		[:aTab | (aTab submorphs size > 0) and:  [(aTab submorphs first isKindOf: TextMorph) and: [(aTab submorphs first contents string copyWithout: $ ) = 'Menus']]] ifNone: [^ self].
	self removeFlapTab: aFlapTab keepInList: false.
	self addGlobalFlap: self menuFlap.
	World ifNotNil: [World addGlobalFlaps]

"Utilities replaceMenuFlap"! !

!Utilities class methodsFor: 'flaps' stamp: 'sw 12/6/1999 18:08'!
standardBottomFlap
	|  aFlapTab aPage |
	aPage _ self newPartsFlapPage.
	aPage setProperty: #maximumThumbnailWidth toValue: 80.
	aFlapTab _ FlapTab new referent: aPage beSticky.
	aFlapTab color: Color red lighter.
	aFlapTab setToPopOutOnDragOver: true.
	aFlapTab setToPopOutOnMouseOver: true.
	aFlapTab assumeString: 'Supplies' font: Preferences standardButtonFont orientation: #horizontal color: Color red lighter.
	aFlapTab edgeToAdhereTo: #bottom; inboard: false.

	aPage extent: self currentWorld width @ 100.
	#(PaintInvokingMorph RectangleMorph EllipseMorph StarMorph  CurveMorph PolygonMorph TextMorph ImageMorph BasicButton
		PasteUpMorph    BookMorph TabbedPalette 
		JoystickMorph  ) do:
		[:sym | aPage addMorphBack: (Smalltalk at: sym) authoringPrototype].

	aPage addMorphBack: RectangleMorph roundRectPrototype.
	aPage addMorphBack: TrashCanMorph new markAsPartsDonor.
	aPage addMorphBack: ScriptingSystem scriptControlButtons markAsPartsDonor.
	aPage addMorphBack: Morph new previousPageButton markAsPartsDonor.
	aPage addMorphBack: Morph new nextPageButton markAsPartsDonor.
	aPage addMorphBack: (ClockMorph authoringPrototype showSeconds: false) step.

	aPage replaceTallSubmorphsByThumbnails.
	aPage fixLayout.

	aFlapTab position: ((Display width - aFlapTab width) // 2 @ (self currentWorld height - aFlapTab height)).
	aPage setProperty: #flap toValue: true.
	aPage color: (Color red muchLighter "alpha: 0.2").
	aPage extent: self currentWorld width @ 100.
	
	^ aFlapTab! !

!Utilities class methodsFor: 'flaps' stamp: 'sw 12/6/1999 18:09'!
standardLeftFlap
	| aFlap aFlapTab aButton aClock buttonColor anOffset |

	aFlap _ PasteUpMorph newSticky borderWidth: 0.
	aFlapTab _ FlapTab new referent: aFlap.
	aFlapTab assumeString: 'Squeak' font: Preferences standardButtonFont orientation: #vertical color: Color brown lighter lighter.
	aFlapTab edgeToAdhereTo: #left; inboard: false.
	aFlapTab setToPopOutOnDragOver: true.
	aFlapTab setToPopOutOnMouseOver: true.

	aFlapTab position: (0 @ ((Display height - aFlapTab height) // 2)).
	aFlap setProperty: #flap toValue: true.
	aFlap color: (Color brown muchLighter lighter "alpha: 0.3").
	aFlap extent: 200 @ self currentWorld height.

	self addProjectNavigationButtonsTo: aFlap.
	anOffset _ 16.

	buttonColor _ Color green muchLighter.
	aButton _ SimpleButtonMorph new target: Smalltalk.
	aButton color: buttonColor.
	aButton actionSelector: #saveSession.
	aButton setBalloonText: 'Make a complete snapshot of the current state of the image onto disk.'.
	aButton label: 'snapshot'.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	aButton _ aButton fullCopy target: Utilities.
	aButton actionSelector: #fileOutChanges.
	aButton label: 'file out changes'.
	aButton setBalloonText: 'File out the current change set to disk.'.
	aFlap addMorph: aButton.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	aButton _ aButton fullCopy target: Utilities.
	aButton actionSelector: #browseRecentSubmissions.
	aButton setBalloonText: 'Open a message-list browser showing the 20 most-recently-submitted methods.'.
	aButton label: 'recent submissions'.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	aClock _ ClockMorph newSticky.
	aClock color: Color red.
	aClock showSeconds: false.
	aClock font: (TextStyle default fontAt: 3).
	aClock step.
	aClock setBalloonText: 'The time of day.  If you prefer to see seconds, check out my menu.'.
	aFlap addCenteredAtBottom: aClock offset: anOffset.

	aButton _ aButton fullCopy target: Preferences.
	aButton actionSelector: #openPreferencesInspector.
	aButton setBalloonText: 'Open a window allowing me to view and change various Preferences.'.
	aButton label: 'preferences...'.
	aButton color: Color cyan muchLighter.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	aButton _ aButton fullCopy target: Utilities.
	aButton actionSelector: #updateFromServer.
	aButton label: 'load code updates'.
	aButton setBalloonText: 'Check the Squeak server for any new code updates, and load any that are found.'.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	self addSystemStatusLinesTo: aFlap.

	aButton _ SimpleButtonMorph new target: self.
	aButton actionSelector: #explainFlaps; color: buttonColor.
	aButton label: 'About flaps...'.
	aButton setBalloonText: 'Click here to get a window of information about flaps.'.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	aButton _ aButton fullCopy target: Preferences; actionSelector: #editAnnotations;
		label: 'Annotations...'.
	aButton setBalloonText: 'Click here to get a little window that will allow you to specify which types of annotations, in which order, you wish to see in the annotation pane of method-list browsers.'.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	aButton _ TrashCanMorph newSticky.
	aFlap addCenteredAtBottom: aButton offset: anOffset.
	aButton startStepping.

	^ aFlapTab! !

!Utilities class methodsFor: 'flaps' stamp: 'sw 12/6/1999 18:09'!
standardRightFlap
	|  aFlapTab aPage |
	aPage _ self newPartsFlapPage.
	aFlapTab _ FlapTab new referent: aPage beSticky.
	aFlapTab color: Color red lighter.
	aFlapTab assumeString: 'Tools' font: Preferences standardButtonFont orientation: #vertical color: Color orange lighter.
	aFlapTab edgeToAdhereTo: #right; inboard: false.
	aFlapTab setToPopOutOnDragOver: true.
	aFlapTab setToPopOutOnMouseOver: true.

	aPage extent: (90 @ self currentWorld height).
	self addSampleWindowsTo: aPage.
	aPage addMorphBack: ScriptingSystem newScriptingSpace.
	aPage addMorphBack: RecordingControlsMorph authoringPrototype.
	aPage replaceTallSubmorphsByThumbnails.
	aPage fixLayout.

	aFlapTab position: (self currentWorld width - aFlapTab width) @ ((Display height - aFlapTab height) // 2).
	aPage setProperty: #flap toValue: true.
	aPage color: (Color brown muchLighter alpha: 0.5).
	aPage extent: (90 @ self currentWorld height).
	
	^ aFlapTab! !

!Utilities class methodsFor: 'scraps' stamp: 'sw 12/6/1999 18:10'!
scrapsBook
	| header aButton label |
	ScrapsBook ifNil:
		[ScrapsBook _ BookMorph new pageSize: 200@300; setNameTo: 'scraps'.
		ScrapsBook color: Color yellow muchLighter.
		ScrapsBook borderColor: Color darkGray; borderWidth: 2.
		ScrapsBook removeEverything; showPageControls; insertPage.
		header _ AlignmentMorph newRow centering: #center.
		header setProperty: #header toValue: true.
		header addMorph: (aButton _ SimpleButtonMorph new label: 'X' font: Preferences standardButtonFont).
		aButton target: ScrapsBook; color:  Color lightRed; actionSelector: #delete;
				setBalloonText: 'Remove
(to view again later, click on any trashcan).'.

		header addMorphBack: AlignmentMorph newVariableTransparentSpacer beSticky.
		header addMorphBack: 	(label _ UpdatingStringMorph new target: self) beSticky.
		label getSelector: #trashTitle; useStringFormat; step.
		header addMorphBack: AlignmentMorph newVariableTransparentSpacer beSticky.
		header addMorphBack: (aButton _ SimpleButtonMorph new label: 'E' font: Preferences standardButtonFont).
		aButton target: Utilities; color:  Color veryLightGray; actionSelector: #maybeEmptyTrash;
				setBalloonText: 'Click here to empty the trash.'.
		ScrapsBook currentPage addMorph: (TextMorph new contents: 'Objects you drag into the trash will automatically be saved here, one object per page, in case you need them later.  To disable this feature set the "preserveTrash" Preference to false.

You can individually expunge objects by hitting the - control, and you can empty out all the objects in the trash can by hitting the "E" button at top right.'
			wrappedTo: 190).

		ScrapsBook addMorphFront: header.
		ScrapsBook setProperty: #scraps toValue: true].
	^ ScrapsBook

	"Utilities emptyScrapsBook"
! !

!Utilities class methodsFor: 'text styles and fonts' stamp: 'sw 12/7/1999 10:32'!
chooseSystemFont
	Smalltalk isMorphic
		ifFalse:
			[self mvcPromptForFont: 'Choose standard text style' andSendTo: Utilities withSelector: #setSystemFontTo:]
		ifTrue:
			[self promptForFont: 'Choose standard font:' andSendTo: self withSelector: #setSystemFontTo:]! !

!Utilities class methodsFor: 'text styles and fonts'!
fontMenuForStyle: styleName target: target selector: selector
        | menu |
        menu _ MenuMorph entitled: styleName.
        (Utilities fontSizesFor: styleName) do: [:size|
                menu 
                        add: (size asString, ' Point') 
                        target: target 
                        selector: selector
                        argument: ((TextStyle named: styleName) fontOfSize: size).
                menu lastItem font: ((TextStyle named: styleName) fontOfSize: size)].
        ^menu! !

!Preferences class methodsFor: 'window colors' stamp: 'sw 12/7/1999 18:22'!
windowSpecificationPanel
	"Preferences windowSpecificationPanel"
	| aPanel buttonRow aButton aRow aSwatch aColor aWindow aMiniWorld |
	aPanel _ AlignmentMorph newColumn hResizing: #shrinkWrap; vResizing: #shrinkWrap.

	aPanel addMorph: (buttonRow _ AlignmentMorph newRow color: (aColor _ Color tan lighter)).
	
	aButton _ SimpleButtonMorph new target: self.
	buttonRow addTransparentSpacerOfSize: 2@0.
	buttonRow addMorphBack: (SimpleButtonMorph new label: '?'; target: self; actionSelector: #windowColorHelp; setBalloonText: 'Click for an explanation of this panel'; color: Color veryVeryLightGray; yourself).
	buttonRow addTransparentSpacerOfSize: 8@0.
	#(	('Bright' 	installBrightWindowColors	yellow
					'Use standard bright colors for all windows.')
		('Pastel'		installPastelWindowColors	paleMagenta
					'Use standard pastel colors for all windows.')
		('White'	installUniformWindowColors		white
					'Use white backgrounds for all standard windows.')) do:

		[:quad |
			aButton _ aButton fullCopy
				label: quad first;
				actionSelector: quad second;
				color: (Color colorFrom: quad third);
				setBalloonText: quad fourth;
				yourself.
			buttonRow addMorphBack: aButton.
			buttonRow addTransparentSpacerOfSize: 10@0].

	self windowColorClasses do:
		[:aClassName | 
			aRow _ AlignmentMorph newRow color: aColor.
			aSwatch _ ColorSwatch new
				target: self;
				getSelector: #windowColorFor:;
				putSelector: #setWindowColorFor:to:;
				argument: aClassName;
				extent: (40 @ 20);
				yourself.
			aRow addMorphFront: aSwatch.
			aRow addTransparentSpacerOfSize: (12 @ 1).
			aRow addMorphBack: (StringMorph contents: aClassName font: TextStyle defaultFont).
			aPanel addMorphBack: aRow].

	 Smalltalk isMorphic
                ifTrue:
                        [buttonRow _ buttonRow fullCopy removeAllMorphs.
					buttonRow addTransparentSpacerOfSize: 25@0.
					aButton _ aButton fullCopy color: Color tan muchLighter.
					aButton label: 'Update Tools Flap'; target: Utilities; actionSelector: #replaceToolsFlap.
					buttonRow addMorphBack: aButton.
					aButton setBalloonText: 'Press here to place tools which use the above window-color choices  into the Tools flap.'.
					aPanel addMorphBack: buttonRow.

					aWindow _ aPanel wrappedInWindowWithTitle: 'Window Colors'.
                        self currentHand attachMorph: aWindow.
					aWindow world startSteppingSubmorphsOf: aPanel.]
                ifFalse:
                        [(aMiniWorld _ MVCWiWPasteUpMorph newWorldForProject: nil)
						addMorph: aPanel.
                           aMiniWorld startSteppingSubmorphsOf: aPanel.
                        MorphWorldView openOn: aMiniWorld
                                label: 'Window Colors'
                                extent: aMiniWorld fullBounds extent]! !

!Utilities class methodsFor: 'text styles and fonts' stamp: 'sw 12/6/1999 23:36'!
mvcPromptForFont: aPrompt andSendTo: aTarget withSelector: aSelector
        | aMenu aChoice aStyle namesAndSizes aFont |
	"Utilities mvcPromptForFont: 'Choose system font style' andSendTo: Utilities withSelector: #setSystemFontTo:"
	aMenu _ CustomMenu new.
	Utilities actualTextStyles keys do:
		[:styleName |
			aMenu add: styleName action: styleName].
	aChoice _ aMenu startUpWithCaption: aPrompt.
	aChoice ifNil: [^ self].
	aMenu _ CustomMenu new.
	aStyle _ TextStyle named: aChoice.
	(namesAndSizes _ aStyle fontNamesAndSizes) do:
		[:aString | aMenu add: aString action: aString].
	aChoice _ aMenu startUpWithCaption: nil.
	aChoice ifNil: [^ self].
	aFont _ aStyle fontAt: (namesAndSizes indexOf: aChoice).
	aTarget perform: aSelector with: aFont! !

!Utilities class methodsFor: 'text styles and fonts' stamp: 'sw 12/6/1999 17:58'!
promptForFont: aPrompt andSendTo: aTarget withSelector: aSelector
        "Utilities promptForFont: 'Choose system font:' andSendTo: Utilities withSelector: #setSystemFontTo:"
        "NOTE: Morphic ONLY!!!!.  Derived from a method written by Robin Gibson"

        | menu subMenu |
        menu _ MenuMorph entitled: aPrompt.
        Utilities actualTextStyles keys do: [:styleName|
                subMenu _ self fontMenuForStyle: styleName target: aTarget selector: aSelector.
                menu add: styleName subMenu: subMenu.
                menu lastItem font: ((TextStyle named: styleName) fontOfSize: 18)].

        menu popUpAt: World activeHand position forHand: World activeHand! !

!Utilities class methodsFor: 'text styles and fonts' stamp: 'sw 12/7/1999 12:52'!
setSystemFontTo: aFont
	| aStyle newDefaultStyle |
	aFont ifNil: [^ self].
	aStyle _ aFont textStyle ifNil: [^ self].
	newDefaultStyle _ aStyle copy.
	newDefaultStyle defaultFontIndex: (aStyle fontIndexOf: aFont).
	TextConstants at: #DefaultTextStyle put: newDefaultStyle.
	PopUpMenu initialize.
	ListParagraph initialize.
	StandardSystemView setLabelStyle.
	self replaceToolsFlap.
	self replaceMenuFlap.
	self currentWorld restoreDisplay! !


!WonderlandWrapperMorph methodsFor: 'handles' stamp: 'sw 12/6/1999 18:10'!
addHandlesTo: aHaloMorph box: box
	| dismissHandle s lab |
	s _ aHaloMorph handleSize.
	myActor getTexturePointer == nil ifFalse:[
		aHaloMorph addHandleAt: box rightCenter color: Color lightGray on: #mouseDown send: #extractTexture: to: aHaloMorph target].

	aHaloMorph addHandleAt: box topLeft color: Color red
		on: #mouseDown send: #doMenu:with: to: aHaloMorph.

	aHaloMorph addHandleAt: (box leftCenter + (0 @ (s+2)) min: box leftCenter + box bottomLeft // 2)
		color: Color lightBrown
		on: #mouseDown send: #yourself "#tearOffTile" to: self.

	dismissHandle _ aHaloMorph addHandleAt: (box topLeft + ((s+2)@0) min: box topLeft + box topCenter // 2)
		color: Color red muchLighter
		on: #mouseDown send: #mouseDownInDimissHandle:with: to: aHaloMorph.
	dismissHandle on: #mouseUp send: #maybeDismiss:with: to: aHaloMorph.
	dismissHandle on: #mouseStillDown send: #setDismissColor:with: to: aHaloMorph.
	dismissHandle addMorphBack: (lab _ StringMorph contents: 'X' font: Preferences standardButtonFont).
	lab position: dismissHandle position + (4 @ 0).
	lab lock.

	aHaloMorph addHandleAt: box leftCenter color: Color cyan
		on: #mouseDown send: #yourself "#openViewerForArgument" to: self.

	aHaloMorph addHandleAt: box topCenter color: Color black
		on: #mouseDown send: #grabFromHalo:with: to: self.

	(aHaloMorph addHandleAt: (box topCenter + ((s+2)@0) min: box topCenter + box topRight // 2)
		color: Color brown
		on: #mouseDown send: #dragStartFromHalo:with: to: self)
		on: #mouseStillDown send: #dragMoveFromHalo:with: to: self;
		on: #mouseUp send: #dragEndFromHalo:with: to: self.

	(aHaloMorph addHandleAt: box topRight color: Color green
		on: #mouseDown send: #dupStartFromHalo:with: to: self)
		on: #mouseStillDown send: #dupMoveFromHalo:with: to: self;
		on: #mouseUp send: #dupEndFromHalo:with: to: self.

	Preferences showDebugHaloHandle ifTrue:
		[aHaloMorph addHandleAt: ((box topRight + box rightCenter) // 2)
			color: Color blue veryMuchLighter
			on: #mouseDown send: #doDebug:with: to: aHaloMorph].

	(aHaloMorph addHandleAt: box bottomLeft color: Color blue
		on: #mouseDown send: #rotateStartFromHalo:with: to: self)
		on: #mouseStillDown send: #rotateMoveFromHalo:with: to: self;
		on: #mouseUp send: #rotateEndFromHalo:with: to: self.

	(aHaloMorph addHandleAt: box bottomRight color: Color yellow
		on: #mouseDown send: #growStartFromHalo:with: to: self)
		on: #mouseStillDown send: #growMoveFromHalo:with: to: self;
		on: #mouseUp send: #growEndFromHalo:with: to: self! !

!HaloMorph methodsFor: 'private' stamp: 'sw 12/7/1999 17:59'!
addHandlesTo: aHaloMorph box: box

	| s dismissHandle lab |
	s _ aHaloMorph handleSize.

	aHaloMorph addHandleAt: box topLeft color: Color red
		on: #mouseDown send: #doMenu:with: to: self.

	"innerTarget colorSettable ifTrue:
		[aHaloMorph addGraphicalHandle: ColorHalo at: (box topLeft + (0@14)) on: #mouseDown send: #doRecolor:with: to: self]."  "possible future"

	aHaloMorph addHandleAt: (box leftCenter + (0 @ (s+2)) min: box leftCenter + box bottomLeft // 2)
		color: Color lightBrown
		on: #mouseDown send: #tearOffTile to: innerTarget.

	dismissHandle _ aHaloMorph addHandleAt: (box topLeft + ((s+2)@0) min: box topLeft + box topCenter // 2)
		color: Color red muchLighter
		on: #mouseDown send: #mouseDownInDimissHandle:with: to: self.
	dismissHandle on: #mouseUp send: #maybeDismiss:with: to: self.
	dismissHandle on: #mouseStillDown send: #setDismissColor:with: to: self.
	dismissHandle addMorphBack: (lab _ StringMorph contents: 'X' font: Preferences standardButtonFont).
	lab position: dismissHandle position + (4 @ 0).
	lab lock.

	aHaloMorph addHandleAt: box leftCenter color: Color cyan
		on: #mouseDown send: #openViewerForArgument to: innerTarget.

	aHaloMorph addHandleAt: box topCenter color: Color black
		on: #mouseDown send: #doGrab:with: to: self.

	(aHaloMorph addHandleAt: (box topCenter + ((s+2)@0) min: box topCenter + box topRight // 2)
		color: Color brown
		on: #mouseDown send: #startDrag:with: to: self)
		on: #mouseStillDown send: #doDrag:with: to: self.

	aHaloMorph addHandleAt: box topRight color: Color green
		on: #mouseDown send: #doDup:with: to: self.

	Preferences showDebugHaloHandle ifTrue:
		[aHaloMorph addHandleAt: ((box topRight + box rightCenter) // 2)
			color: Color blue veryMuchLighter
			on: #mouseDown send: #doDebug:with: to: self].

	target balloonText ifNotNil:
		[(aHaloMorph addHandleAt: box bottomCenter color: Color lightBlue
			on: #mouseDown send: #mouseDownOnHelpHandle: to: innerTarget)
			on: #mouseUp send: #deleteBalloon to: innerTarget].

	(aHaloMorph addHandleAt: box bottomLeft color: Color blue
		on: #mouseDown send: #startRot:with: to: self)
		on: #mouseStillDown send: #doRot:with: to: self.

	target isFlexMorph
		ifTrue: [(aHaloMorph addHandleAt: box bottomRight color: Color lightOrange
				on: #mouseDown send: #startScale:with: to: self)
				on: #mouseStillDown send: #doScale:with: to: self]
		ifFalse: [(aHaloMorph addHandleAt: box bottomRight color: Color yellow
				on: #mouseDown send: #startGrow:with: to: self)
				on: #mouseStillDown send: #doGrow:with: to: self].

	innerTarget addOptionalHandlesTo: aHaloMorph box: box! !

!SimpleButtonMorph methodsFor: 'accessing' stamp: 'sw 12/7/1999 18:11'!
label: aString

	| oldLabel m |
	(oldLabel _ self findA: StringMorph)
		ifNotNil: [oldLabel delete].
	m _ StringMorph contents: aString font: TextStyle defaultFont.
	self extent: m extent + (borderWidth + 6).
	m position: self center - (m extent // 2).
	self addMorph: m.
	m lock! !

!ColorTileMorph methodsFor: 'other' stamp: 'sw 12/7/1999 18:51'!
addColorSwatch

	| m1 m2 desiredW |
	m1 _ StringMorph contents: 'color' font: (StrikeFont familyName: #NewYork size: 12).
	m2 _ Morph new extent: 12@8; color: (Color r: 0.8 g: 0 b: 0).
	desiredW _ m1 width + 6.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).
	m2 position: (bounds center x - (m2 width // 2)) @ (m1 bottom - 1).
	self addMorph: m1; addMorph: m2.
	colorSwatch _ m2! !

!Text methodsFor: 'attributes' stamp: 'sw 12/7/1999 12:31'!
unembellished 
	"Return true if the only emphases are the default font and bold"
	| font1 bold |
	font1 _ TextFontChange defaultFontChange.
	bold _ TextEmphasis bold.
	Preferences ignoreStyleIfOnlyBold ifFalse:
		["Ignore font1 only or font1-bold followed by font1-plain"
		^ (runs values = (Array with: (Array with: font1)))
		or: [runs values = (Array with: (Array with: font1 with: bold)
 								with: (Array with: font1))]].

	"If preference is set, then ignore any combo of font1 and bold"
	runs withStartStopAndValueDo:
		[:start :stop :emphArray |
		emphArray do:
			[:emph | (font1 = emph or: [bold = emph]) ifFalse: [^ false]]].
	^ true! !


ListParagraph initialize!
PopUpMenu initialize!
Preferences class removeSelector: #fontForScriptorButtons!
StandardScriptingSystem removeSelector: #fontForScriptorButtons!
StandardSystemView setLabelStyle!
Text removeSelector: #rangeOf:startingAt:!