'From Squeak 2.1 of June 30, 1998 on 16 August 1998 at 12:23:55 pm'!
"Change Set:		SundryFixes8-16-di
Date:			16 August 1998
Author:			Dan Ingalls

A number of small improvements...
1.  A quick check eliminates the need for morphic to repaint objects under outlying scrollbars during, eg, continuous scroll operations.  This can greatly improve response time.
2.  FileLists check for and prevent attempts to save contents that are abbreviated, in hex, or are otherwise unsaveable.
3.  Fixes the hidden test in Morph>>fullBounds so that it doesn't break curvy text as in Play With Me 5.
4.  Fixes the initialization of the extended field visible to default to true, anticipating future uses.
"!


!FileList methodsFor: 'file list menu' stamp: 'di 8/16/1998 12:22'!
deleteFile
	"Delete the currently selected file"
	listIndex = 0 ifTrue: [^ self].
	(self confirm: 'Really delete ' , fileName , '?') ifFalse: [^ self].
	directory deleteFileNamed: fileName.
	self updateFileList.
	brevityState _ #FileList.
	self get! !

!FileList methodsFor: 'private' stamp: 'di 8/16/1998 09:26'!
contents
	"Answer the contents of the file, reading it first if needed."
	"Possible brevityState values:
		FileList,
		fullFile, briefFile, needToGetFull, needToGetBrief,
		fullHex, briefHex, needToGetFullHex, needToGetBriefHex"

	(listIndex = 0) | (brevityState == #FileList) ifTrue: [^ self defaultContents].  "no file selected"
	brevityState == #fullFile ifTrue: [^ contents].
	brevityState == #fullHex ifTrue: [^ contents].
	brevityState == #briefFile ifTrue: [^ contents].
	brevityState == #briefHex ifTrue: [^ contents].

	brevityState == #needToGetFullHex ifTrue: [^ self readContentsHex: false].
	brevityState == #needToGetBriefHex ifTrue: [^ self readContentsHex: true].

	brevityState == #needToGetFull ifTrue: [^ self readContentsBrief: false].
	brevityState == #needToGetBrief ifTrue: [^ self readContentsBrief: true].  "default"

	self halt: 'unknown state ' , brevityState printString! !

!FileList methodsFor: 'private' stamp: 'di 8/16/1998 09:25'!
defaultContents
	contents _ list == nil
		ifTrue: [String new]
		ifFalse: [String streamContents:
					[:s | s nextPutAll: 'NO FILE SELECTED'; cr.
					s nextPutAll: '  -- Folder Summary --'; cr.
					list do: [:item | s nextPutAll: item; cr]]].
	brevityState _ #FileList.
	^ contents! !

!FileList methodsFor: 'private' stamp: 'di 8/16/1998 09:46'!
put: aText
	| ff type |
	brevityState == #fullFile ifTrue:
		[ff _ directory newFileNamed: self fullName.
		Cursor write showWhile: [ff nextPutAll: aText asString; close].
		fileName = ff localName 
			ifTrue: [contents _ aText asString]
			ifFalse: [self updateFileList].		"user renamed the file"
		^ true  "accepted"].

	listIndex = 0 ifTrue:
		[PopUpMenu notify: 'No fileName is selected'.
		^ false  "failed"].
	type _ 'These'.
	brevityState = #briefFile ifTrue: [type _ 'Abbreviated'].
	brevityState = #briefHex ifTrue: [type _ 'Abbreviated'].
	brevityState = #fullHex ifTrue: [type _ 'Hexadecimal'].
	brevityState = #FileList ifTrue: [type _ 'Directory'].
	PopUpMenu notify: type , ' contents cannot
meaningfully be saved at present.'.
	^ false  "failed"
! !

!FileList methodsFor: 'private' stamp: 'di 8/16/1998 09:22'!
readContentsBrief: brevityFlag
	"Read the contents of the receiver's selected file, unless it is too long, in which case show just the first 5000 characters. Don't create a file if it doesn't already exist."
	| f fileSize first5000 |
	f _ directory oldFileOrNoneNamed: self fullName.
	f ifNil: [^ 'For some reason, this file cannot be read'].
	(brevityFlag not or: [(fileSize _ f size) <= 30000]) ifTrue:
		[contents _ f contentsOfEntireFile.
		brevityState _ #fullFile.   "don't change till actually read"
		^ contents].

	"if brevityFlag is true, don't display long files when first selected"
	first5000 _ f next: 5000.
	f close.
	contents _ 'File ''', fileName, ''' is ', fileSize printString, ' bytes long.
You may use the ''get'' command to read the entire file.

Here are the first 5000 characters...
------------------------------------------
', first5000 , '
------------------------------------------
... end of the first 5000 characters.'.
	brevityState _ #briefFile.   "don't change till actually read"
	^ contents.
! !

!FileList methodsFor: 'private' stamp: 'di 8/16/1998 09:20'!
readContentsHex: brevity
	"retrieve the contents from the external file unless it is too long.
	  Don't create a file here.  Check if exists."
	| f size data hexData s |

	f _ directory oldFileOrNoneNamed: self fullName. 
	f == nil ifTrue: [^ 'For some reason, this file cannot be read'].
	((size _ f size)) > 5000 & brevity
		ifTrue: [data _ f next: 10000. f close. brevityState _ #briefHex]
		ifFalse: [data _ f contentsOfEntireFile. brevityState _ #fullHex].

	s _ WriteStream on: (String new: data size*4).
	0 to: data size-1 by: 16 do:
		[:loc | s nextPutAll: loc hex; space;
			nextPut: $(; print: loc; nextPut: $); space; tab.
		loc+1 to: (loc+16 min: data size) do: [:i | s nextPutAll: (data at: i) hex; space].
		s cr].
	hexData _ s contents.

	^ contents _ ((size > 5000) & brevity
		ifTrue: ['File ''', fileName, ''' is ', size printString, ' bytes long.
You may use the ''get'' command to read the entire file.

Here are the first 5000 characters...
------------------------------------------
', hexData , '
------------------------------------------
... end of the first 5000 characters.']
		ifFalse: [hexData]).
! !


!Morph methodsFor: 'access properties' stamp: 'di 8/16/1998 11:13'!
removeProperty: propName
	extension == nil ifTrue: [^ self].
	extension removeProperty: propName! !

!Morph methodsFor: 'drawing' stamp: 'di 8/16/1998 11:19'!
fullBounds

	fullBounds ifNil: [
		fullBounds _ self bounds.
		submorphs size > 0 ifTrue: [
			submorphs do: [:m | (m left < 50000 or: [(self hasProperty: #hidden) not])
					ifTrue: [fullBounds _ fullBounds quickMerge: m fullBounds]]]].
	^ fullBounds
! !


!MorphExtension methodsFor: 'initialization' stamp: 'di 8/16/1998 12:02'!
initialize
	"Init all booleans to default values"
	locked _ false.
	visible _ true.
	sticky _ false.
	isPartsDonor _ false.
! !


!ScrollPane methodsFor: 'geometry' stamp: 'di 8/16/1998 01:09'!
scrollBarFills: aRectangle
	"Return true if a flop-out scrollbar fills the rectangle"
	^ (retractableScrollBar and: [submorphs includes: scrollBar]) and:
		[scrollBar bounds containsRect: aRectangle]! !


!SystemWindow methodsFor: 'drawing' stamp: 'di 8/16/1998 01:14'!
areasRemainingToFill: aRectangle
	| areas |
	(areas _ super areasRemainingToFill: aRectangle) isEmpty
		ifTrue: [^ areas "good news -- complete occlusion"].
	"Check for special case that this is scrollbar damage"
	((bounds topLeft - (14@0) corner: bounds bottomRight) containsRect: aRectangle) ifTrue:
		[paneMorphs do: [:p | ((p isKindOf: ScrollPane) and: [p scrollBarFills: aRectangle])
							ifTrue: [^ Array new]]].
	^ areas! !


"Postscript:
Fix default value of visible in extisting extensions.
Executable statements after this comment quote..."
Morph allSubInstancesDo:
	[:m | m extension == nil ifFalse: [m extension visible: true]]!

