"Change Set:		FileOutSlipsFix
Date:			28 August 2000
Author:			Jesse Welton

Fixes ChangeSet>>fileOut and ChangeSet>>mailOut to use the checkForSlips preference rather than suppressCheckForSlips, which no longer exists."!


!ChangeSet methodsFor: 'fileIn/Out' stamp: 'JW 8/28/2000 07:38'!
fileOut
	"File out the receiver, to a file whose name is a function of the change-set name and either of the date & time or chosen to have a unique numeric tag, depending on the preference 'sequentialChangeSetRevertableFileNames'"

	| file slips nameToUse |
	self checkForConversionMethods.
	nameToUse _ Preferences changeSetVersionNumbers
		ifTrue:
			[FileDirectory default nextNameFor: self name extension: 'cs']
		ifFalse:
			[(self name, FileDirectory dot, Utilities dateTimeSuffix, 
				FileDirectory dot, 'cs') asFileName].
	Cursor write showWhile:
		[file _ FileStream newFileNamed: nameToUse.
		file header; timeStamp.
		self fileOutPreambleOn: file.
		self fileOutOn: file.
		self fileOutPostscriptOn: file.
		file trailer; close].

	Preferences checkForSlips ifFalse: [^ self].

	slips _ self checkForSlips.
	(slips size > 0 and: [self confirm: 'Methods in this fileOut have halts
or references to the Transcript
or other ''slips'' in them.
Would you like to browse them?'])
		ifTrue: [Smalltalk browseMessageList: slips
							name: 'Possible slips in ', name]! !

!ChangeSet methodsFor: 'fileIn/Out' stamp: 'JW 8/28/2000 08:01'!
mailOut
	"File out the receiver, to a file whose name is a function of the           
	change-set name and either of the date & time or chosen to have a
	unique numeric tag, depending on the preference
	'sequentialChangeSetRevertableFileNames'."

	| slips messageStrm message compressBuffer compressStream data compressedStream compressTarget |
	(Smalltalk includesKey: #Celeste)
		ifFalse: [^ self notify: 'no mail reader present'].

	self checkForConversionMethods.
	Cursor write
		showWhile: 
			["Prepare the message"
			messageStrm _ WriteStream on: (String new: 300).
			messageStrm nextPutAll: 'From: ';
			 nextPutAll: Celeste userName;
			 cr;
			 nextPutAll: 'To: squeak@cs.uiuc.edu';
			 cr;
			 nextPutAll: 'Subject: [CS] ';
			 nextPutAll: name;
			 cr;
			 nextPutAll: 'from preamble:';
			 cr;
			 cr.
			self fileOutPreambleOn: messageStrm.
			"Prepare the gzipped data"
			message _ MailMessage from: messageStrm contents.
			message _ MailMessage from: message asMultipartText.
			data _ WriteStream on: String new.
			data header.
			self fileOutPreambleOn: data.
			self fileOutOn: data.
			self fileOutPostscriptOn: data.
			data trailer.
			data _ ReadStream on: data contents.
			compressBuffer _ ByteArray new: 1000.
			compressStream _ GZipWriteStream on: (compressTarget _ WriteStream on: (ByteArray new: 1000)).
			[data atEnd]
				whileFalse: [compressStream nextPutAll: (data nextInto: compressBuffer)].
			compressStream close.
			compressedStream _ ReadStream on: compressTarget contents asString.
			CelesteComposition
				openForCeleste: Celeste current 
				initialText: (message asTextEncodingNewPart: compressedStream named: name , '.cs.gz')].
	Preferences checkForSlips ifFalse: [^ self].
	slips _ self checkForSlips.
	(slips size > 0 and: [self confirm: 'Methods in this fileOut have halts
or references to the Transcript
or other ''slips'' in them.
Would you like to browse them?'])
		ifTrue: [Smalltalk browseMessageList: slips name: 'Possible slips in ' , name]! !

