'From Squeak2.9alpha of 4 August 2000 [latest update: #2524] on 31 August 2000 at 12:09:46 pm'!
"Change Set:		CelesteTweaksFixes
Date:			16 August 2000
Author:			Mike Rutenberg (mdr@scn.org)

(1) emptyTrash will no longer remove messages from database if they are also in another category.  This avoids surprises when you have filed a message in two categories, delete one, and then empty the trash.

(2) Fixes a recently introduced bug in Celeste moveAgain where the message was removed from the current category but never added to the proper destination category.

(3) Small cleanups
	Removes two unused methods (Celeste hasUsername: and MailDB delete:)
	Renames a recently included method (updateToc) that differed only in capitalization from an existing method

(4) Clear the password when the POP username is changed
"!


!Celeste methodsFor: 'categories pane' stamp: 'mdr 8/17/2000 13:58'!
cacheTOC
	"Caches a version of the TOC"

	| s |
	currentTOC _ OrderedCollection new: currentMessages size.
	1 to: currentMessages size do: 
		[:i | 
		s _ WriteStream on: (String new: 100).
		s nextPutAll: i printString;
		 space.
		[s position < 4]
			whileTrue: [s space].
		s nextPutAll: (mailDB getTOCstring: (currentMessages at: i)).
		currentTOC add: s contents].
	currentTOC _ currentTOC asArray.
	(currentMessages includes: currentMsgID)
		ifFalse: [currentMsgID _ nil]! !

!Celeste methodsFor: 'categories pane' stamp: 'mdr 8/17/2000 17:38'!
emptyTrash
	"Delete all messages in the '.trash.' category.
	WARNING: The messages will be totally removed from the Celeste index, and the .messages file will be marked so that the message contents are removed when it is next compressed."

	| msgList |

	self requiredCategory: '.trash.'.

	msgList _ mailDB messagesIn: '.trash.'.			"Look at ALL messages in the trash"
	"Remove from the list messages which are also in other categories"
	msgList _ msgList select: [ :msgID | (mailDB categoriesThatInclude: msgID) size = 1].

	mailDB deleteAll: msgList.
	mailDB cleanUpCategories.
	self updateTOC.

	(mailDB messagesIn: '.trash.') isEmpty ifFalse:
		[self inform: 'Some messages were not removed because they are also filed in other categories'].! !

!Celeste methodsFor: 'categories pane' stamp: 'mdr 8/17/2000 17:36'!
setCategory: newCategory 
	"Change the currently selected category. We must also compute the table  
	of contents and message list for the new category."

	| messageCount |
	currentCategory _ newCategory.
	newCategory isNil
		ifTrue: 
			[self status: nil.
			currentMessages _ currentTOC _ currentMsgID _ nil]
		ifFalse: 
			[currentMessages _ self filteredMessagesIn: newCategory.
			messageCount _ currentMessages size.
			messageCount > self maxMessageCount
				ifTrue: 
					[self messages: self maxMessageCount from: messageCount.
					currentMessages _ currentMessages copyLast: self maxMessageCount]
				ifFalse: [self messages: messageCount from: messageCount].
			self cacheTOC].
	self changed: #category.
	self changed: #tocEntryList.
	self changed: #tocEntry.
	self changed: #messageText.
	self changed: #status
! !

!Celeste methodsFor: 'categories pane' stamp: 'mdr 7/31/2000 19:03'!
setPopUserName
	userPassword _ nil.   "Clear the password when a new username is set"
	^self class setPopUserName! !

!Celeste methodsFor: 'categories pane' stamp: 'mdr 8/17/2000 13:59'!
viewAllMessages
	currentMessages _ self filteredMessagesIn: self category.
	self messages: currentMessages size from:  currentMessages size .
	self cacheTOC.
	self changed: #tocEntryList.
	self changed: #tocEntry.
	self changed: #messageText.! !

!Celeste methodsFor: 'table of contents pane' stamp: 'mdr 8/31/2000 11:47'!
moveAgain
	"Move the current message to the same category as last time."

	| newCatName |

	currentMsgID ifNil: [ ^self ].
	(lastCategory isEmpty not)
		ifTrue: [newCatName _ lastCategory]
		ifFalse: [newCatName _ self getCategoryNameIfNone: [^self]].
	newCatName = currentCategory ifTrue: [ ^self ].

	mailDB file: currentMsgID inCategory: newCatName.
	self removeMessage.! !

MailDB removeSelector: #delete:!
Celeste removeSelector: #hasUsername:!
Celeste removeSelector: #updateToc!


