'From Squeak2.9alpha of 5 August 2000 [latest update: #3296] on 24 January 2001 at 10:38:42 pm'!
"Change Set:		expungeAnonymous-sw
Date:			24 January 2001
Author:			Scott Wallace

Removes ancient support for 'anonymous scripts' in the etoy system, and streamlines related code accordingly."!


!Morph methodsFor: 'player' stamp: 'sw 1/22/2001 14:25'!
okayToDuplicate
	"Formerly this protocol was used to guard against awkward situations when there were anonymous scripts in the etoy system.  Nowadays we just always allow duplication"

	^ true! !


!ScriptEditorMorph methodsFor: 'initialization' stamp: 'sw 1/23/2001 11:27'!
bringUpToDate
	"Make certain that the script name and the names of players within are up to date"

	| currentName titleMorph |
	playerScripted ifNil: ["likely a naked test/yes/no fragment!!" ^ self].

	currentName _ playerScripted externalName.
	submorphs size = 0 ifTrue: [^ self].
	self firstSubmorph "align" submorphsDo: [:m | "try quick way"
				m knownName = 'title' ifTrue: [titleMorph _ m]].
	titleMorph ifNil: [
		titleMorph _ self firstSubmorph findDeepSubmorphThat: [:m | m knownName = 'title'] 
								ifAbsent: [nil]].
	titleMorph ifNotNil:
		[titleMorph label: currentName, ' ', self scriptName font: ScriptingSystem fontForTiles]! !

!ScriptEditorMorph methodsFor: 'buttons' stamp: 'sw 1/23/2001 11:27'!
buttonRowForEditor
	"Answer a row of buttons that comprise the header at the top of the Scriptor"

	| aRow aString aButtonMorph buttonFont aStatusMorph aButton aColumn |
	buttonFont _ Preferences standardButtonFont.
	aRow _ AlignmentMorph newRow color: Color transparent; layoutInset: 0.
	aRow hResizing: #shrinkWrap.
	aRow vResizing: #shrinkWrap.

	aRow addMorphFront:
		(SimpleButtonMorph new
			label: '!!' font: (StrikeFont familyName: #ComicBold size: 16);
			target: self;
			color: Color yellow;
			borderWidth: 0;
			actWhen: #whilePressed;
			actionSelector: #tryMe;
			balloonTextSelector: #tryMe).
	aRow addTransparentSpacerOfSize: 6@10.
	self addDismissButtonTo: aRow.
	aRow addTransparentSpacerOfSize: 6@1.
	aColumn _ AlignmentMorph newColumn beTransparent.
	aColumn addTransparentSpacerOfSize: 0@4.
	aButton _ UpdatingThreePhaseButtonMorph checkBox.
	aButton
		target: self;
		actionSelector: #toggleWhetherShowingTiles;
		getSelector: #showingMethodPane.
	aButton setBalloonText: 'toggle between showing tiles and showing textual code'.
	aColumn addMorphBack: aButton.
	aRow addMorphBack: aColumn.

	aRow addTransparentSpacerOfSize: 6@10.

	aString _ playerScripted externalName, ' ', self scriptTitle.
	aRow addMorphBack:
		(aButtonMorph _ SimpleButtonMorph new useSquareCorners label: aString font: buttonFont; target: self; setNameTo: 'title').
	aButtonMorph actWhen: #buttonDown; actionSelector: #offerScriptorMenu.
	aButtonMorph borderColor: (Color fromRgbTriplet: #(0.065 0.258 1.0)).
	aButtonMorph color: ScriptingSystem uniformTileInteriorColor.
	aButtonMorph balloonTextSelector: #offerScriptorMenu.
	aRow addMorphBack: (aStatusMorph _ self scriptInstantiation statusControlMorph).

	aRow addTransparentSpacerOfSize: 6@1.

	aRow addMorphBack:
		(IconicButton new borderWidth: 0;
			labelGraphic: (ScriptingSystem formAtKey: 'AddTest'); color: Color transparent; 
			actWhen: #buttonDown;
			target: self;
			actionSelector: #addYesNoToHand;
			shedSelvedge;
			balloonTextSelector: #addYesNoToHand).
	aRow addTransparentSpacerOfSize: 12@10.
	self addDestroyButtonTo: aRow.
	self scriptInstantiation updateStatusMorph: aStatusMorph.
	^ aRow! !

!ScriptEditorMorph methodsFor: 'buttons' stamp: 'sw 1/24/2001 22:11'!
destroyScript
	"At user request, and only after confirmation, destroy the script, thus removing it from the uniclass's method dictionary and removing its instantiations from all instances of uniclass, etc."

	self confirm: 'Caution -- this destroys this script
permanently; are you sure you want to do this?' orCancel: [^ self].
	true ifTrue: [^ playerScripted removeScript: scriptName fromWorld: self world].

	self flag: #deferred.  "revisit"
	(playerScripted okayToDestroyScriptNamed: scriptName)
		ifFalse:
			[^ self inform: 'Sorry, this script is being called
from another script.'].

	self actuallyDestroyScript! !


!UserScript methodsFor: 'script editor' stamp: 'sw 1/23/2001 11:28'!
instantiatedScriptEditor
	"Return the current script editor, creating it if necessary"

	self isTextuallyCoded ifTrue:
			[^ (player costume pasteUpMorph ifNil: [player costume "the world, backstop"]) scriptorForTextualScript: selector ofPlayer: player].

	currentScriptEditor ifNil:
		[currentScriptEditor _ (player class includesSelector: selector) 
			ifTrue: [ScriptEditorMorph new 
				fromExistingMethod: selector 
				forPlayer: player]
			ifFalse: [ScriptEditorMorph new
				setMorph: player costume
				scriptName: selector].
		status == #ticking ifTrue: [player costume arrangeToStartStepping]].
	
	^ currentScriptEditor! !

StandardScriptingSystem removeSelector: #makeAnonymousScriptsPermanent!
ScriptInstantiation removeSelector: #isAnonymous!
ScriptEditorMorph removeSelector: #abandon!
ScriptEditorMorph removeSelector: #isAnonymous!
