'From Squeak 2.3 of January 14, 1999 on 16 February 1999 at 11:45:06 am'!
"Change Set:		prefs-sw
Date:			16 February 1999
Author:			Scott Wallace

Shifts a number of preference-like options to class Preferences, for more uniform access.

Several options formerly lodged in Presenter or World now go to Preferences, where they can be manipulated in the usual way via the preferences editor.  Many inst vars now removed from Presenter.

Adds a mechanism allowing the setting of a Preference value (in the preferences editor) to trigger some code rather than just smashing a new value into the prefs dictionary.  See noteThatFlag:justChangedTo:.

Greatly reduces the processing load on the Prefs inspector by making it step much less frequently.  (Required addition of a new mechanism whereby UpdatingStringMorphs can hold on to their own steptimes rather than having it hard-coded.

Removes the former Control Panel from various menus and structures, since its work is now subsumed by Preferences.  This does not preclude the construction of ad-hoc control panels."!

Object subclass: #Presenter
	instanceVariableNames: 'associatedMorph stopButton stepButton goButton standardPlayer standardPlayfield standardPalette playerList '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Scripting'!
StringMorph subclass: #UpdatingStringMorph
	instanceVariableNames: 'format target lastValue getSelector putSelector floatPrecision growable stepFrequency '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Widgets'!

!Morph methodsFor: 'dropping/grabbing' stamp: 'sw 2/15/1999 19:18'!
vanishAfterSlidingTo: aPosition event: evt

	| aForm aWorld origin startPoint endPoint |
	aForm _ self imageForm offset: 0@0.
	aWorld _ self world.
	origin _ aWorld viewBox origin.
	startPoint _ evt hand fullBounds origin + origin.
	self delete.
	aWorld displayWorld.
	endPoint _ aPosition + origin.
	aForm slideFrom: startPoint  to: endPoint nSteps: 12 delay: 15.
	Preferences soundsEnabled ifTrue: [TrashCanMorph playDeleteSound].
! !

!Morph methodsFor: 'player commands' stamp: 'sw 2/16/1999 11:33'!
makeFenceSound
	Preferences soundsEnabled ifTrue:
		[self playSoundNamed: 'scratch'].
! !

!Morph methodsFor: 'e-toy support' stamp: 'sw 2/15/1999 19:38'!
enforceTileColorPolicy
	Preferences coloredTilesEnabled
		ifTrue:
			[self makeAllTilesColored]
		ifFalse:
			[self makeAllTilesGreen]! !


!BooklikeMorph methodsFor: 'misc' stamp: 'sw 2/15/1999 19:17'!
playPageFlipSound: soundName
	self presenter ifNil: [^ self].  "Avoid failures when called too early"
	(Preferences soundsEnabled "user-controllable" and:
		[PageFlipSoundOn])  "mechanism to suppress sounds at init time"
			ifTrue: [self playSoundNamed: soundName].
! !


!HandMorph methodsFor: 'event dispatching' stamp: 'sw 2/15/1999 19:14'!
handleMouseOver: evt
	| mList allMouseOvers leftMorphs enteredMorphs now t oldHalo balloonHelpEnabled mouseOverHalosEnabled |
	owner ifNil: [^ self].  "this hand is not in a world"
	balloonHelpEnabled _ Preferences balloonHelpEnabled & evt anyButtonPressed not.
	mouseOverHalosEnabled _ Preferences mouseOverHalosEnabled & evt anyButtonPressed not.

	"Start with a list consisting of the topmost unlocked morph in the
	innermost frame (pasteUp), and all of its containers in that frame."
	mList _ self mouseOverList: evt.

	now _ Time millisecondClockValue.

	"Make a list of all potential mouse-overs..."
	allMouseOvers _ mList select:
		[:m | ((mouseOverHalosEnabled and: [m wantsHalo])
			or: [balloonHelpEnabled and: [m wantsBalloon]])  "To start a timer"
			or: [m handlesMouseOver: (evt transformedBy: (m transformFrom: self))]  "To send mouseEnter:"].
	leftMorphs _ mouseOverMorphs select: [:m | (allMouseOvers includes: m) not].
	enteredMorphs _ allMouseOvers select: [:m | (mouseOverMorphs includes: m) not].

	"Notify and remove any mouse-overs that have just been left..."
	leftMorphs do: [:m |
		mouseOverMorphs remove: m.
		m wantsBalloon ifTrue: [m deleteBalloon].
		m mouseLeave: (evt transformedBy: (m transformFrom: self)).
		mouseOverTimes ifNotNil: [mouseOverTimes removeKey: m ifAbsent: [] ]].

	"Add any new mouse-overs and send mouseEnter: and/or start timers..."
	enteredMorphs do: [:m |
		mouseOverMorphs add: m.
		dragOverMorphs remove: m ifAbsent: [].  "Cant be in two places at once"
		(m handlesMouseOver: evt) ifTrue:
			[m mouseEnter: (evt transformedBy: (m transformFrom: self))].
		(m wantsHalo or: [m wantsBalloon]) ifTrue:
			[mouseOverTimes ifNotNil: [mouseOverTimes at: m put: now]]].

mouseOverTimes ifNotNil:
	[mouseOverTimes keys do:
		[:m | "Check pending timers for lingering"
		t _ mouseOverTimes at: m.
		(now < t "rollover" or: [now > (t+800)]) ifTrue:
			["Yes we have lingered for 0.8 seconds..."
			mouseOverTimes removeKey: m.
			m owner ifNotNil:  "Not deleted during linger (--it happens ;--)"
				[(mouseOverHalosEnabled and: [m wantsHalo])
					ifTrue: [oldHalo _ m world haloMorphOrNil.
							(oldHalo == nil or: [oldHalo target ~~ m])
								ifTrue: ["Put up halo for m"
										self popUpHalo: evt.
										(balloonHelpEnabled and: [m wantsBalloon]) ifTrue:
											["...and reschedule balloon after longer linger"
											mouseOverTimes at: m put: now]]
								ifFalse: ["Halo for m is already up, so show balloon"
										(balloonHelpEnabled and: [m wantsBalloon])
											ifTrue: [m showBalloon: m balloonText]]]
					ifFalse:
						[(balloonHelpEnabled and: [m wantsBalloon])
							ifTrue: [m showBalloon: m balloonText]]]]]]! !

!HandMorph methodsFor: 'world menu' stamp: 'sw 2/15/1999 19:27'!
scriptingMenu
	"Build the scripting menu for the world."
	| menu |
	menu _ (MenuMorph entitled: 'Authoring Tools') defaultTarget: self.
	menu addStayUpItem.
	menu add: 'standard parts bin' target: self presenter action: #createStandardPartsBin.
	menu balloonTextForLastItem: 'A bin of standard parts, from which you can drag out useful morphs.'.
	menu add: 'custom parts bin' target: self presenter action: #launchCustomPartsBin.
	menu balloonTextForLastItem: 'A customized bin of parts.  To define what the custom parts bin is, edit any existing parts bin and tell it to be saved as the custom parts bin.'.
	menu add: 'new scripting area' target: self action: #detachableScriptingSpace.
	menu balloonTextForLastItem: 'A window set up for simple scripting.'.
	menu addLine.
	menu add: 'unlock locked objects' action: #unlockWorldContents.
	menu balloonTextForLastItem: 'If any items on the world desktop are currently locked, unlock them.'.
	menu add: 'unhide hidden objects' action: #showHiders.
	menu balloonTextForLastItem: 'If any items on the world desktop are currently hidden, make them visible.'.
	menu add: 'round up stray objects' action: #roundUpStrayObjects.
	menu balloonTextForLastItem: 'If any items on the desktop are currently off-screen (because their coordinates are outside the bounds of the desktop), bring them back within view.'.
	^ menu! !


!PasteUpMorph methodsFor: 'misc' stamp: 'sw 2/15/1999 19:32'!
impartPrivatePresenter
	presenter ifNil:
		[presenter _ Presenter new associatedMorph: self.
		presenter standardPlayer]! !


!Player methodsFor: 'scripts-standard' stamp: 'sw 2/15/1999 19:18'!
beep: soundName
	Preferences soundsEnabled
		ifTrue: [self playSoundNamed: soundName]
! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 2/15/1999 19:24'!
forward: dist 

	| rho radians delta didStray p fractionalP newP aCostume |
	(aCostume _ self costume) isInWorld ifFalse: [^ self].
	aCostume isWorldOrHandMorph ifTrue: [^ self].

	rho _ (aCostume asNumber: dist) asFloat.
	radians _ (self getHeadingUnrounded asFloat - 90.0) degreesToRadians.
	delta _ (radians cos @ radians sin) * rho.

	((aCostume owner isHandMorph not) and:
	 [Preferences fenceEnabled]) ifTrue:
		[(aCostume owner bounds containsRect: aCostume bounds) ifFalse:
			["If I stray out of the bounds of my owner, pull me back, but
			 without changing my heading as bounce would. Do nothing if
			 bounce has already corrected the direction."
			didStray _ false.
			((aCostume left < aCostume owner left and: [delta x < 0]) or:
			 [aCostume right > aCostume owner right and: [delta x > 0]]) ifTrue: [
				delta _ delta x negated @ delta y.
				didStray _ true].
			((aCostume top < aCostume owner top and: [delta y < 0]) or:
			 [aCostume bottom > aCostume owner bottom and: [delta y > 0]]) ifTrue: [
				delta _ delta x @ delta y negated.
				didStray _ true].
			didStray ifTrue: [aCostume makeFenceSound]]].

	"use and record the fractional position"
	p _ aCostume referencePosition.
	fractionalP _ aCostume actorState fractionalPosition.
	(fractionalP == nil or: [fractionalP asIntegerPoint ~= p])
		ifTrue: [newP _ p asFloatPoint + delta]
		ifFalse: [newP _ fractionalP + delta].
"
Transcript cr; print: p; space; print: fractionalP;
cr; cr; print: newP asIntegerPoint; space; print: newP; show: ''.
"
	aCostume actorState fractionalPosition: newP.
	aCostume referencePosition: newP asIntegerPoint.
! !

!Player methodsFor: 'misc' stamp: 'sw 2/15/1999 19:18'!
makeBounceSound: soundName

	Preferences soundsEnabled
		ifTrue: [self playSoundNamed: soundName].
! !

!Player methodsFor: 'misc' stamp: 'sw 2/15/1999 19:38'!
tileReferringToSelf
	| aTile |
	aTile _ TileMorph new
		setObjectRef: nil "disused parm" actualObject: self;
		typeColor: (ScriptingSystem colorForType: #player).
	aTile enforceTileColorPolicy.
	^ aTile! !


!Preferences class methodsFor: 'initialization' stamp: 'sw 2/15/1999 19:11'!
chooseInitialSettings
	"Restore the default choices for Preferences."
	"Preferences chooseInitialSettings"
	#(	(allowSoundQuickStart				false)
		(allowSysWindowEmbedding			false)
		(automaticViewerPlacement			true)
		(balloonHelpEnabled					true)
		(browseWithPrettyPrint				false)
		(cmdDotEnabled						true)
		(confirmFirstUseOfStyle				true)
		(disableSounds						false)
		(fastDragWindowForMorphic			false)
		(ignoreStyleIfOnlyBold				true)
		(inboardScrollbars					false)
		(logDebuggerStackToFile				true)
		(mouseOverHaloseEnabled			false)
		(noviceMode							false)
		(printAlternateSyntax				false)
		(reverseWindowStagger				true)
		(showDebugHaloHandle				true)
		(showDiffsInChangeList				true)
		(showTimeStampsInMenuTitles		false)
		(showProjectZoom					false)
		(suppressCheckForSlips				false)
		(suppressUpdateServerPrompt		false)
		(thoroughSenders					true)
		(unlimitedPaintArea					false)
		(updateSavesFile						false)
		(useAnnotationPanes				false)
		(useFlaps							true)
		(warnIfNoChangesFile				true)
		(warnIfNoSourcesFile				true))
	do:
		[:aPair |
			aPair last == #true
				ifTrue:
					[self enable: aPair first]
				ifFalse:
					[self disable: aPair first]]

! !

!Preferences class methodsFor: 'initialization' stamp: 'sw 2/15/1999 19:24'!
initializeHelpMessages
	"Preferences initializeHelpMessages"
  	HelpDictionary _ Dictionary new.
	#(

(allowSoundQuickStart
'If true, attempt to start playing sounds using optional "quick start"')

(allowSysWindowEmbedding
'Determines whether, in Morphic, SystemWindows should automatically be droppable into willing receptors')

(automaticViewerPlacement
'If true, new viewers are automatically positioned near the objects they view; if false, new viewers are attached to the hand, from whence you much choose a destination for them')

(balloonHelpEnabled
'Whether balloon help should be offered when the cursor lingers over certain objects.')

(browseWithPrettyPrint
'If true, browsers will automatically format their contents')

(cautionBeforeClosing 
'If true, Morphic windows seen in an mvc project will put up a warning before allowing themselves to be dismissed')

(cmdDotEnabled
'If true, cmd-dot brings up a debugger;
if false, the cmd-dot interrupt is disabled')

(confirmFirstUseOfStyle
'If true, the first attempt to submit a method with non-standard style will bring up a confirmation dialog')

(disableSounds
'If true, all sound playing is disabled')
	
(editPlayerScriptsInPlace 
'If true, textual player scripts are edited in place in Scriptors (still imperfectly implemented)')

(eToyScheme
'If true, new scripting spaces place the Playfield to the left and the the palette to the right of the window; if false, the opposite is true.')

(fastDragWindowForMorphic
'If true, morphic window drag will be done by dragging an outline of the window.')

(fenceEnabled
'Whether an object obeying motion scripts should stop moving when it reaches the edge of its container.')

(ignoreStyleIfOnlyBold
'If true, then any method submission in which the only style change is for bolding will be treated as a method with no style specifications')

(inboardScrollbars
'If true, then ScrollPane will place scrollbars inside on the right and will not hide them on exit')

(logDebuggerStackToFile
'If true, whenever you fall into a debugger a summary of its stack will be written to a file named
''SqueakDebug.log''')

(mouseOverHalosEnabled
'If false, halos will not be put up on mouseovers even if they otherwise might be.')

(noviceMode 
'If true, certain novice-mode accommodations are made.')

(printAlternateSyntax
'If true, then
prettyPrint using experimental syntax.
Otherwise use normal ST-80 syntax.')

(reverseWindowStagger
'If true, a reverse-stagger strategy  is used for determining where newly launched windows will be placed; if false, a direct- stagger strategy is used.')

(showDebugHaloHandle 
'If true, a special debugging halo handle is displayed at the right of the halo; if false, no such handle is shown.')

(showDiffsInChangeList
'If true, changeList browsers and Versions browsers reveal the differences between successive versions or between the in-memory code and the code on disk')

(showPlayerSource
'If true, then all Player methods with fewer than 2 arguments are included in Viewers, whether or not they are intended for end-user use.  This can be dangerous')

(showProjectZoom
'If true, then show a zoom effect when entering or leaving projects.  This can be costly of memory (at least an extra screen buffer) so dont use it in low space situations.  But it is cool.')

(showScriptSource
'If true, then the actual Smalltalk source code for methods is shown in the detail panes for scripts in a viewer; if false, then a help message for scripts is shown instead.')

(showTimeStampsInMenuTitles
'If true, then the author''s timestamp is displayed as the menu title of any message list; if false, no author''s timestamps are shown')

(suppressCheckForSlips 
'If false, then whenever you file out a change set, it is checked for ''slips'' and if any are found, you are so informed and given a chance to open a browser on them')

(suppressUpdateServerPrompt
'If true, the prompt for server choice when updating code from the server is suppressed.  Set this to true to leave the server choice unchanged from update to update.')

(thoroughSenders
'If true, then ''senders'' browsers will dive inside structured literals in their search')

(uniformWindowColors
'If true, then all standard windows are given the same color rather than their customized window-type-specific colors')

(unlimitedPaintArea
'If true, the painting area for a new drawing will not be limited in size; if false, a reasonable
limit will be applied, in an attempt to hold down memory and time price.')

(updateRemoveSequenceNum
'If true, then remove the leading sequence number from the filename before automatically saving a local copy of any update loaded.')

(updateSavesFile
'If true, then when an update is loaded from the server, a copy of it will automatically be saved on a local file as well.')

(useAnnotationPanes
'If true, a thin horizontal annotation pane is used in message-list browsers.')

(useDetailPanesInViewers
'If true, then Viewers will have an extra "" control at the left of each row, the hitting of which toggles the appearance of a textual detail pane.')

(useFlaps
'If true, then flaps are shown along the edges of Morphic projects.')

(useNewViewers
'If true, then the new kinds of viewers introduced in Squeak 2.3 are used; if false, then the old style, from earlier releases, are still used.  Old viewers will hopefully soon be removed from the system.')

(warnIfNoChangesFile
'If true, then you will be warned, whenever you start up, if no changes file
can be found')

(warnIfNoSourcesFile 
'If true, then you will be warned, whenever you start up, if no sources file can be found')) do:
		[:pair | HelpDictionary at: pair first put: 
			(pair first, ':
', pair last)]
! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 2/15/1999 19:44'!
borderColorWhenRunning
	^ Color green! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 2/15/1999 19:45'!
borderColorWhenStopped
	^ Color red! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 2/8/1999 09:37'!
viewerEditingPaneHeight
	^ 60! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 2/8/1999 09:38'!
viewerEditingPaneWidth
	^ 400! !

!Preferences class methodsFor: 'preferences dictionary' stamp: 'sw 2/15/1999 11:35'!
noteThatFlag: prefSymbol justChangedTo: aBoolean
	"Provides a hook so that a user's toggling of a preference might precipitate some immediate action"
	((prefSymbol == #useFlaps) and: [Smalltalk isMorphic]) ifTrue:
		[aBoolean
			ifTrue:
				[self currentWorld addFlaps]
			ifFalse:
				[self currentWorld deleteAllFlapArtifacts]]! !

!Preferences class methodsFor: 'preferences dictionary' stamp: 'sw 2/15/1999 11:36'!
setPreference: prefSymbol toValue: aBoolean
	(FlagDictionary at: prefSymbol ifAbsent: [nil]) == aBoolean
		ifFalse:
			[FlagDictionary at: prefSymbol put: aBoolean.
			self noteThatFlag: prefSymbol justChangedTo: aBoolean]
! !

!Preferences class methodsFor: 'preferences panel' stamp: 'sw 2/15/1999 16:49'!
openPreferencesControlPanel
	"Preferences openPreferencesControlPanel"
	| aPanel aWindow aRow wrapper but aList odd aColor w width1 width2 spacer |
	Smalltalk verifyMorphicAvailability ifFalse: [^ self beep].
	aPanel _ AlignmentMorph newColumn.
	aPanel beSticky.
	aList _ OrderedCollection new.
	FlagDictionary associationsDo: [:assoc | aList add: (Array
				with: assoc key
				with: assoc value
				with: (self helpMessageForPreference: assoc key))].
	odd _ false.
	width1 _ 172.
	spacer _ 4.
	width2 _ 14.
	(aList asSortedCollection: [:a :b | a first < b first])
		do: 
			[:triplet | 
			aPanel addMorphBack: (aRow _ AlignmentMorph newRow).
			aRow color: (aColor _ odd
							ifTrue: [Color green muchLighter]
							ifFalse: [Color red veryMuchLighter]).
			odd _ odd not.
			aRow addMorph: (wrapper _ Morph new color: aColor).
			wrapper setBalloonText: triplet third.
			wrapper extent: width1 @ 15.
			wrapper addMorph: (StringMorph new contents: triplet first).
			aRow addMorphBack: (Morph new color: aColor; extent: (spacer @ 15)).
			aRow addMorphBack: (wrapper _ Morph new color: aColor).
			wrapper extent: width2 @ 15.
			wrapper addMorphBack: (but _ UpdatingBooleanStringMorph new contents: triplet second printString).
			but getSelector: triplet first;
			putSelector: #setPreference:toValue:;
			stepFrequency: 1800;
			 target: self].
	Smalltalk isMorphic
		ifTrue:
			[aWindow _ SystemWindow new model: self.
			aWindow addMorph: aPanel frame: (0 @ 0 extent: 1 @ 1).
			aWindow setLabel: 'Preferences'.
			aWindow openInWorld]
		ifFalse:
			[w _ WorldMorph new addMorph: aPanel.
			w startSteppingSubmorphsOf: aPanel.
			MorphWorldView openOn: w
				label: 'Preferences'
				extent: w fullBounds extent]
! !

!Preferences class methodsFor: 'preferences panel' stamp: 'sw 2/16/1999 11:41'!
soundsEnabled
	"This protocol still used in a number of places, so this simply defers to the Prefs setting"
	^ self disableSounds not! !

!Preferences class methodsFor: 'preferences panel' stamp: 'sw 2/15/1999 20:34'!
toggleButtons
	| aButton  aList |

	self flag: #deferred.  "moved to Preferences but not presently used and not readily usable in this format without some work"

	aList _ #(
	('Balloons'	'BalloonsOn'		'BalloonsOff' 	toggleShowBalloons	balloonHelpEnabled)
	('Sounds'	'SoundOn'		'SoundOff'  		toggleSoundsEnabled	soundsEnabled)
	('Fence'		'FenceOn'		'FenceOff'  		toggleFence			fenceEnabled)) with:

	#(
'Balloon Help: If green, then when the cursor pauses over an object that has balloon help, that help balloon is shown'

'Sounds: If green, sounds will be  heard when appropriate; if red, sounds are suppressed.'

'Fence: If green, an invisible "fence" keeps your objects from straying outside their containers when their scripts move them.')

	collect:
			[:q :helpString |
			aButton _ ToggleButtonMorph new setNameTo: q first.
			aButton onImage: (ScriptingSystem formAtKey: q second);
				offImage: (ScriptingSystem formAtKey: q third);
				pressedImage: nil;
				actionSelector: q fourth;
				stateSelector: q last;
				actWhen: #buttonDown;
				target: self;
				setInitialState;  "Obtains it from target"
				setBalloonText: helpString;
				extent: (ScriptingSystem formAtKey: q second) extent.
			aButton].

	^ aList! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 2/15/1999 19:36'!
colorTilesEnabled
	^ false! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 2/16/1999 11:24'!
showScriptSource
	"Change the definition here to re-hard-code this to your preferred setting"
	"Temporarily hard-coded pending viewer work underway"
	^ false! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 2/16/1999 11:24'!
useCategoryListsInViewers
	"Temporarily hard-coded pending viewer work underway"
	^ false! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 2/16/1999 11:40'!
useDetailPanesInViewers
	"Change the definition here to re-hard-code this to your preferred setting"
	^ false! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 2/16/1999 11:24'!
useScrollingInPartsLists
	"Temporarily hard-coded pending viewer work underway"
	^ false "true"! !


!Presenter methodsFor: 'palette & parts bin' stamp: 'sw 2/15/1999 19:39'!
tilesPagesForPartsBin
	| aPage bools aTile aPhrase |
	aPage _ self newPageForStandardPartsBin padding: 30.
	bools _ self booleanTiles.
	aPage addMorphBack: bools first markAsPartsDonor.
	aPage addMorphBack: bools last markAsPartsDonor.
	aPage addMorphBack: self arithmeticTiles first markAsPartsDonor.
	aPage addMorphBack: RandomNumberTile new markAsPartsDonor.

	#(('(Sensor anyButtonPressed)' 'button down?')
	('(Sensor noButtonPressed)' 'button up?')
	"('(Sensor keyboardPressed)' 'key hit?')   sucker doesn't work for some reason") do:
		[:pair |
			aPhrase _ SystemQueryPhrase new.
			aTile _ BooleanTile new.
			aTile setExpression: pair first label: pair second.
			aPhrase addMorph: aTile.
			aPage addMorphBack: aPhrase].

	aPage enforceTileColorPolicy.
	aPage fixLayout.
	aPage replaceTallSubmorphsByThumbnails.
	^ OrderedCollection with: aPage  "room to grow"! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 2/15/1999 19:46'!
startRunningScripts

	self stopButtonState: false.
	self stepButtonState: false.
	self goButtonState: true.
	associatedMorph startRunningAll.
	associatedMorph borderColor: Preferences borderColorWhenRunning.

	ThumbnailMorph recursionReset.  "needs to be done once in a while"! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 2/15/1999 19:46'!
startRunningScriptsFrom: aGoButton
	(goButton == nil or: [goButton isInWorld not]) ifTrue: [goButton _ aGoButton].

	self stopButtonState: false.
	self stepButtonState: false.
	self goButtonState: true.
	associatedMorph startRunningAll.
	associatedMorph borderColor: Preferences borderColorWhenRunning.

	ThumbnailMorph recursionReset.  "needs to be done once in a while"! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 2/15/1999 19:46'!
stopRunningScripts
	self stopButtonState: true.
	self stepButtonState: false.
	self goButtonState: false.
	associatedMorph stopRunningAll.
	associatedMorph borderColor: Preferences borderColorWhenStopped! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 2/15/1999 19:46'!
stopRunningScriptsFrom: aStopButton
	(stopButton == nil or: [stopButton isInWorld not]) ifTrue: [stopButton _ aStopButton].
	self stopButtonState: true.
	self stepButtonState: false.
	self goButtonState: false.
	associatedMorph stopRunningAll.
	associatedMorph borderColor: Preferences borderColorWhenStopped! !

!Presenter methodsFor: 'viewer' stamp: 'sw 2/15/1999 19:39'!
updatePartsViewer: aPartsViewer
	| aPlayer aPosition aNewPartsViewer oldOwner wasSticky |
	
	"obsolete branch, only for old PartsViewer objects"
	aPlayer _ aPartsViewer scriptedPlayer.
	aPosition _ aPartsViewer position.
	wasSticky _ aPartsViewer isSticky.
	aNewPartsViewer _ aPartsViewer species new visible: false.

	wasSticky ifTrue: [aNewPartsViewer beSticky].
	oldOwner _ aPartsViewer owner.
	aPartsViewer delete.
	oldOwner ifNotNil: [oldOwner addMorphBack: aNewPartsViewer].
	aNewPartsViewer obtainBankInfoFrom: aPartsViewer.
	aNewPartsViewer setPlayer: aPlayer.
	aNewPartsViewer position: aPosition.
	aNewPartsViewer enforceTileColorPolicy.
	aNewPartsViewer visible: true.
	aNewPartsViewer layoutChanged! !

!Presenter methodsFor: 'viewer' stamp: 'sw 2/15/1999 19:39'!
updateViewer: aViewer forceToShow: aCategory
	| aPlayer aPosition newViewer oldOwner wasSticky barHeight |
	aPlayer _ aViewer scriptedPlayer.
	aPosition _ aViewer position.
	wasSticky _ aViewer isSticky.
	newViewer _ aViewer species new visible: false.
	barHeight _ aViewer submorphs first orientation == #vertical
		ifTrue:
			[aViewer submorphs first submorphs first height]
		ifFalse:
			[0].
	newViewer initializeFor: aPlayer barHeight: barHeight.
	wasSticky ifTrue: [newViewer beSticky].
	oldOwner _ aViewer owner.
	oldOwner ifNotNil:
		[oldOwner replaceSubmorph: aViewer by: newViewer].
	
	"It has happened that old readouts are still on steplist.  We may see again!!"

	newViewer obtainBankInfoFrom: aViewer forceToShow: aCategory.
	newViewer position: aPosition.
	newViewer enforceTileColorPolicy.
	newViewer visible: true.
	newViewer world startSteppingSubmorphsOf: newViewer.
	newViewer layoutChanged! !

!Presenter methodsFor: 'viewer' stamp: 'sw 2/15/1999 19:39'!
viewMorph: aMorph
	| aPlayer aViewer aPalette aRect aPoint nominalHeight |
	Sensor leftShiftDown ifFalse:
		[((aPalette _ aMorph standardPalette) ~~ nil and: [aPalette isInWorld])
			ifTrue:	[^ aPalette viewMorph: aMorph]].

	aPlayer _ aMorph assuredPlayer.
	associatedMorph addMorph: (aViewer _ self nascentPartsViewer).
	aViewer initializeFor: aPlayer barHeight: 6.
	aViewer enforceTileColorPolicy.
	Preferences automaticViewerPlacement ifTrue:
		[aPoint _ aMorph bounds right @ 
			(aMorph center y - ((nominalHeight _ aViewer initialHeightToAllow) // 2)).
		aRect _ (aPoint extent: (aViewer width @ nominalHeight)) translatedToBeWithin: associatedMorph world bounds.
		aViewer position: aRect topLeft.
		aViewer visible: true.
		associatedMorph world startSteppingSubmorphsOf: aViewer.
		"it's already in the world, somewhat coincidentally"
		^ self].
	aMorph primaryHand attachMorph: (aViewer visible: true)! !


!TabbedPalette methodsFor: 'parts & controls tabs' stamp: 'sw 2/15/1999 20:33'!
addControlsTab
	| controlWrapper controlPage |
	controlPage _ AlignmentMorph newColumn beSticky color: Color white.
	controlWrapper _ AlignmentMorph newRow beSticky vResizing: #spaceFill; color: Color white.
	controlWrapper centering: #center.
	controlWrapper addMorphBack: AlignmentMorph newColumn beTransparent.
	controlWrapper addMorphBack: AlignmentMorph newColumn beTransparent.
	controlPage addMorphBack: controlWrapper.
	controlPage addMorphBack: (Morph new beTransparent extent: (1 @ 10)).
	controlPage addMorphBack: RecordingControlsMorph newSticky.
	controlPage setNameTo: 'Controls'.
	self addTabForBook: controlPage  withBalloonText:
'recording controls, and
controls to turn sound,
balloon help, etc., on and off'! !

!TabbedPalette methodsFor: 'viewer tab' stamp: 'sw 2/15/1999 19:49'!
viewMorph: aMorph
	"The receiver is expected to have a viewer tab; select it, and target it to aMorph"
	| aPlayer aViewer |
	((currentPage isKindOf: Viewer) and: [currentPage scriptedPlayer == aMorph player])
		ifTrue:
			[^ self].
	self visible: false.
	aPlayer _ aMorph assuredPlayer.

	aViewer _  StandardViewer new initializeFor: aPlayer barHeight: 0.
	aViewer enforceTileColorPolicy.
	self showNoPalette.
	currentPage ifNotNil: [currentPage delete].
	self addMorphBack: (currentPage _ aViewer beSticky).
	self snapToEdgeIfAppropriate..
	self world startSteppingSubmorphsOf: aViewer.
	self visible: true! !


!TileMorph methodsFor: 'arrows' stamp: 'sw 2/15/1999 19:40'!
showSuffixChoices
	| plus phrase pad outer num |
	(phrase _ self ownerThatIsA: PhraseTileMorph) ifNil: [^ self].

	(type == #literal) & (literal isNumber) ifTrue: ["Tile is a constant number"
		phrase lastSubmorph == owner "pad"
			ifTrue: ["we are adding the first time (at end of our phrase)"
				plus _ self presenter phraseForReceiver: literal 
						op: #+ arg: 1 resultType: #number.
				owner acceptDroppingMorph: plus event: self primaryHand lastEvent.
				num _ plus firstSubmorph firstSubmorph.
				num deleteSuffixArrow]].

	type == #operator ifTrue: ["Tile is accessor of an expression"
		phrase resultType == #number ifTrue:
			[outer _ phrase ownerThatIsA: PhraseTileMorph.
			pad _ self ownerThatIsA: TilePadMorph.
			outer ifNotNil:
				[outer lastSubmorph == pad ifTrue: [ "first time"
					plus _ self presenter phraseForReceiver: 1 
							op: #+ arg: 1 resultType: #number.
					pad acceptDroppingMorph: plus event: self primaryHand lastEvent.
					(plus firstSubmorph) removeAllMorphs.
					(plus firstSubmorph) addMorph: phrase.	"car's heading"
					self deleteSuffixArrow]]]].

	(phrase topEditor ifNil: [phrase]) enforceTileColorPolicy! !


!TrashCanMorph methodsFor: 'event handling' stamp: 'sw 2/15/1999 19:20'!
mouseEnter: event
	"Present feedback for potential deletion."
	| hand |
	hand _ event hand.
	((hand submorphCount > 0) and:
	 [hand submorphs first ~~ self])
		ifTrue: [
			Preferences soundsEnabled ifTrue: [self class playMouseEnterSound].
			hand startDisplaySuppression.
			self world abandonAllHalos.
			self state: #pressed]
		ifFalse: [
			self showStampIn: hand].
! !

!TrashCanMorph methodsFor: 'event handling' stamp: 'sw 2/15/1999 19:20'!
mouseLeave: event
	"Present feedback for aborted deletion."
	| hand |
	hand _ event hand.
	((hand submorphCount > 0) and:
	 [hand submorphs first ~~ self])
		ifTrue:
			[Preferences soundsEnabled ifTrue: [self class playMouseLeaveSound].
			hand endDisplaySuppression.
			self state: #off]
		ifFalse:
			[self stopShowingStampIn: hand].
! !

!TrashCanMorph methodsFor: 'dropping' stamp: 'sw 2/15/1999 19:20'!
acceptDroppingMorph: aMorph event: evt

	| palette |
	Preferences soundsEnabled ifTrue: [self class playDeleteSound].
	evt hand endDisplaySuppression.
	self state: #off.
	aMorph delete.
	palette _ self standardPalette.
	(palette notNil and: [palette isInWorld]) ifTrue: [palette addToTrash: aMorph].! !


!UpdatingStringMorph methodsFor: 'stepping' stamp: 'sw 2/15/1999 16:46'!
stepFrequency: aNumber
	stepFrequency _ aNumber! !

!UpdatingStringMorph methodsFor: 'stepping' stamp: 'sw 2/15/1999 16:45'!
stepTime
	^ stepFrequency ifNil: [50]! !

!UpdatingStringMorph methodsFor: 'copying' stamp: 'sw 2/15/1999 16:57'!
veryDeepInner: deepCopier
	"Copy all of my instance variables.  Some need to be not copied at all, but shared.  	Warning!!!!  Every instance variable defined in this class must be handled.  We must also implement veryDeepFixupWith:.  See DeepCopier class comment."

	super veryDeepInner: deepCopier.
	format _ format.				"Weakly copied"
	target _ target.					"Weakly copied"
	lastValue _ lastValue veryDeepCopyWith: deepCopier.
	getSelector _ getSelector.			"Weakly copied"
	putSelector _ putSelector.		"Weakly copied"
	floatPrecision _ floatPrecision.	"Weakly copied"
	growable _ growable.			"Weakly copied"
	stepFrequency _ stepFrequency.	"Weakly copied"! !


!WorldMorph methodsFor: 'initialization' stamp: 'sw 2/15/1999 19:26'!
configureForConstruction
	| aCan |

	self addMorph: (self presenter standardPartsWindow position: 10@10).

	presenter standardPlayer.

	aCan _ self presenter addTrashCan.
	aCan position: (Display boundingBox bottomRight - aCan extent).  "This oddity necessary to avoid the trashcan's showing up at top-left until an ownerChanged is sent to it; the problem is that addTrashCan: is called too early otherwise."! !

!WorldMorph methodsFor: 'etoy support' stamp: 'sw 2/15/1999 19:32'!
presenter
	^ presenter ifNil:
		[presenter _ Presenter new associatedMorph: self]! !


!WorldMorph class methodsFor: 'all' stamp: 'sw 2/15/1999 19:26'!
openWithStandardPartsBinShowing
	"WorldMorph openWithStandardPartsBinShowing"
	| aWorld anExtent aWindow |
	anExtent _  700 @ 500.
	aWorld _ self new setProperty: #initialExtent toValue: anExtent.
	aWorld extent: anExtent.
	aWorld presenter addTrashCan.
	aWorld addMorph: (aWindow _ aWorld presenter standardPartsWindow position: 10@10).
	aWindow activate.
	MorphWorldView openOn: aWorld label: 'Construction' extent: anExtent! !


HandMorph removeSelector: #toggleColoredTiles!
PartsViewer removeSelector: #coloredTilesEnabled!
Preferences class removeSelector: #useFlaps!
Presenter removeSelector: #toggleSoundsEnabled!
Presenter removeSelector: #createControlPanel!
Presenter removeSelector: #toggleFence!
Presenter removeSelector: #borderColorWhenStopped!
Presenter removeSelector: #mouseOverHalosEnabled!
Presenter removeSelector: #controlPanel!
Presenter removeSelector: #harmonizeTileColorPolicyFor:!
Presenter removeSelector: #soundsEnabled!
Presenter removeSelector: #coloredTilesEnabled!
Presenter removeSelector: #toggleShowHalos!
Presenter removeSelector: #toggleButtons!
Presenter removeSelector: #fenceEnabled!
Presenter removeSelector: #toggleColoredTiles!
Presenter removeSelector: #borderColorWhenRunning!
Presenter removeSelector: #initializeToggles!
Presenter removeSelector: #harmonizeTilesWithColorSetting!
StandardScriptingSystem removeSelector: #frozenPlayfieldBorderColor!
WorldMorph removeSelector: #mouseOverHalosEnabled!
WorldMorph removeSelector: #positionControlPanel:!
WorldMorph removeSelector: #soundsEnabled!
"Postscript:
"
Preferences initializeHelpMessages.
Preferences enable: #balloonHelpEnabled.
Preferences disable: #mouseOverHalosEnabled.
Preferences enable: #fenceEnabled.
ScriptingSystem resetStandardPartsBin.!

