'From Squeak 2.3 of January 14, 1999 on 25 February 1999 at 5:15:56 pm'!
"Change Set:		miscFebFixes-sw
Date:			25 February 1999
Author:			Scott Wallace

Fixes bug reported by Kim whereby if you change a costume of a never-named, never-viewed object, its name got set to nil.

Provides bulletproofing against failures in next-Costume and prev-costume requests for morphs not currently in a world.

When you drop a simple-button-morph which you'd been dragging, you no longer get the button's action performed as an unwanted concomitant."!


!Object methodsFor: 'macpal' stamp: 'sw 2/25/1999 17:02'!
scriptPerformer
	^ self! !


!Morph methodsFor: 'player' stamp: 'sw 2/25/1999 17:15'!
assureExternalName
	self knownName ifNil:
		[self setNameTo: self externalName]! !

!Morph methodsFor: 'player' stamp: 'sw 2/19/1999 09:06'!
assuredPlayer
	"Answer the receiver's player, creating a new one if none currently exists"

	| aPlayer |
	(aPlayer _ self player) ifNil:
		[self assureExternalName.  "a default may be given if not named yet"
		self player: (aPlayer _ self newPlayerInstance).  
			"Different morphs may demand different player types"
		aPlayer costume: self.
		self presenter ifNotNil: [self presenter flushPlayerListCache]].
	^ aPlayer! !


!PartsViewer methodsFor: 'private' stamp: 'sw 2/25/1999 16:57'!
rowBeforeParts
	| r nail aButton aTextMorph aFont |

	r _ AlignmentMorph newRow color: self color; centering: #center.
	aFont _ ScriptingSystem fontForScriptorButtons.
	r addMorph: (aButton _ SimpleButtonMorph new label: 'X' font: aFont).
	aButton target: self;
			color:  Color lightRed;
			actionSelector: #delete;
			setBalloonText: 'Hit the X to remove this Viewer'.
	r addTransparentSpacerOfSize: 5@5.

	aButton _ SimpleButtonMorph new target: self; actionSelector: #previousPartsBank; label: '<' font: aFont; color: self controlButtonColor; borderWidth: 0; actWhen: #buttonUp.
	r addMorphBack: aButton.
	aButton setBalloonText: 'Show previous bank of parts'.	
	r addTransparentSpacerOfSize: 5@5.

	aButton _ SimpleButtonMorph new target: self; actionSelector: #nextPartsBank; label: '>' font: aFont; color: self controlButtonColor; borderWidth: 0; actWhen: #buttonUp.
	aButton setBalloonText: 'Show next bank of parts'.	
	r addMorphBack: aButton.
	r addTransparentSpacerOfSize: 7@5.

	scriptedPlayer costume assureExternalName.
	aTextMorph _ UpdatingStringMorph new
		useStringFormat;
		target:  scriptedPlayer;
		getSelector: #getName;
		putSelector: #setName:;
		setNameTo: 'name';
		font: ScriptingSystem fontForNameEditingInScriptor.
	aTextMorph setProperty: #okToTextEdit toValue: true.
	aTextMorph step.
	r  addMorphBack: aTextMorph.
	aTextMorph setBalloonText: 'Click here to edit the player''s name
use backspace to delete unwanted characters'.	
	r addTransparentSpacerOfSize: 5@5.

	aButton _ SimpleButtonMorph new target: self; actionSelector: #previousCostume; label: '<' font: aFont; color: Color veryLightGray; borderWidth: 0; actWhen: #buttonUp.
	r addMorphBack: aButton.
	aButton setBalloonText: 'switch to previous costume'.	
	r addTransparentSpacerOfSize: 5@5.

	aButton _ SimpleButtonMorph new target: self; actionSelector: #nextCostume; label: '>' font: aFont; color: Color veryLightGray; borderWidth: 0; actWhen: #buttonUp.
	r addMorphBack: aButton.
	aButton setBalloonText: 'switch to next costume'.
	r addTransparentSpacerOfSize: 10@5.

	nail _ ThumbnailMorph new objectToView: scriptedPlayer viewSelector: #costume.
	nail on: #mouseDown send: #thumbnailMenuEvt:forMorph: to: scriptedPlayer.
	r addMorphBack: nail.
	nail setBalloonText: 'click here to get a menu
to specify new costume,
tear off a tile, etc..'.	
	^ r
! !


!Player methodsFor: 'costume' stamp: 'sw 2/19/1999 09:19'!
renderedCostume: aMorph
	"Make aMorph be the receiver's rendered costume; if flexing is currently in effect, make the new morph be flexed correspondingly"

	| renderedMorph known |
	renderedMorph _ costume renderedMorph.
	renderedMorph == aMorph ifTrue: [^ self].
	self rememberCostume: renderedMorph.
	costume isFlexMorph
		ifTrue:
			[costume adjustAfter:
				[costume replaceSubmorph: renderedMorph by: aMorph]]
		ifFalse:
			[costume owner replaceSubmorph: costume by: aMorph.
			aMorph player: self.
			aMorph actorState: costume actorState.
			(known _ costume knownName) ifNotNil:
				[aMorph setNameTo: known].
			costume _ aMorph]! !


!SimpleButtonMorph methodsFor: 'events' stamp: 'sw 2/16/1999 15:08'!
mouseUp: evt
	oldColor ifNotNil:  "if oldColor nil, it signals that mouse had not gone DOWN inside me, e.g. because of a cmd-drag; in this case we want to avoid triggering the action!!"

		[self color: oldColor.
		(actWhen == #buttonUp and: [self containsPoint: evt cursorPoint])
			ifTrue: [self doButtonAction]]
! !


!Viewer methodsFor: 'all' stamp: 'sw 2/17/1999 13:42'!
nextCostume
	| aList aPlayer itsCurrent anIndex newIndex |
	aList _ (aPlayer _ scriptedPlayer) availableCostumesForArrows.
	aList size == 0 ifTrue: [^ self beep].
	itsCurrent _ aPlayer costume renderedMorph.
	anIndex _ aList indexOf: itsCurrent ifAbsent: [nil].
	newIndex _ anIndex
		ifNil:		[1]
		ifNotNil:	[anIndex + 1].
	newIndex > aList size ifTrue: [newIndex _ 1].
	aPlayer renderedCostume: (aList at: newIndex).
	self presenter ifNotNil: [self presenter updateViewer: self]! !

!Viewer methodsFor: 'all' stamp: 'sw 2/17/1999 14:22'!
previousCostume
	| aList aPlayer itsCurrent anIndex newIndex |
	aList _ (aPlayer _ scriptedPlayer) availableCostumesForArrows.
	aList size == 0 ifTrue: [^ self beep].
	itsCurrent _ aPlayer costume renderedMorph.
	anIndex _ aList indexOf: itsCurrent ifAbsent: [nil].
	newIndex _ anIndex
		ifNil:		[aList size]
		ifNotNil:	[anIndex - 1].
	newIndex < 1 ifTrue: [newIndex _ aList size].
	aPlayer renderedCostume: (aList at: newIndex).
	self presenter ifNotNil: [self presenter updateViewer: self]! !


!StandardViewer methodsFor: 'all' stamp: 'sw 2/25/1999 16:57'!
addHeaderMorphWithBarHeight: anInteger
	| header aFont aButton aTextMorph nail wrpr costs headWrapper |
	header _ AlignmentMorph newRow color: self color muchLighter; centering: #center.
	aFont _ ScriptingSystem fontForScriptorButtons.
	header addMorph: (aButton _ SimpleButtonMorph new label: 'X' font: aFont).
	aButton target: self;
			color:  Color lightRed;
			actionSelector: #dismiss;
			setBalloonText: 'Delete'.
	header addTransparentSpacerOfSize: 4@1.

	header addMorphBack: (aButton _ SimpleButtonMorph new label: '+' font: aFont).
	aButton target: self; beTransparent; borderWidth: 0;
			actionSelector: #addCategoryViewer;
			setBalloonText: 'click here to add
another viewer'.
	header addTransparentSpacerOfSize: 4@1.

	(((costs _ scriptedPlayer costumes) size > 1) or: [costs size == 1 and: [costs first ~~ scriptedPlayer costume]]) ifTrue:
		[header addUpDownArrowsFor: self.
		(wrpr _ header submorphs last) submorphs second setBalloonText: 'switch to previous costume'.	
		wrpr submorphs first  setBalloonText: 'switch to next costume'].	

	nail _ ThumbnailMorph new objectToView: scriptedPlayer viewSelector: #costume.
	nail on: #mouseDown send: #thumbnailMenuEvt:forMorph: to: scriptedPlayer.
	header addMorphBack: nail.
	nail setBalloonText: 'click here to get a menu
to specify new costume,
tear off a tile, etc..'.	

	header addTransparentSpacerOfSize: 5@5.

"	aButton _ SimpleButtonMorph new target: self; actionSelector: #newEmptyScript; label: 'S' font: (aFont _ StrikeFont familyName: #ComicBold size: 16);  color: Color transparent; borderWidth: 0; actWhen: #buttonDown.
	aButton setBalloonText: 'drag from here to
create a new script
for this object'.	
	header addMorphBack: aButton.

	header addTransparentSpacerOfSize: 8@5.

	aButton _ SimpleButtonMorph new target: scriptedPlayer; actionSelector: #addInstanceVariable; label: 'I' font: aFont;  color: Color transparent; borderWidth: 0; actWhen: #buttonUp.
	aButton setBalloonText: 'click here to add
an instance variable
to this object.'.
	header addMorphBack: aButton.

	header addTransparentSpacerOfSize: 5@5."

	scriptedPlayer costume assureExternalName.
	aTextMorph _ UpdatingStringMorph new
		useStringFormat;
		target:  scriptedPlayer;
		getSelector: #getName;
		putSelector: #setName:;
		setNameTo: 'name';
		font: ScriptingSystem fontForNameEditingInScriptor.
	aTextMorph setProperty: #okToTextEdit toValue: true.
	aTextMorph step.
	header  addMorphBack: aTextMorph.
	aTextMorph setBalloonText: 'Click here to edit the player''s name
use backspace to delete unwanted characters'.	

	header beSticky.
	anInteger > 0
		ifTrue:
			[headWrapper _ AlignmentMorph newColumn color: self color.
			headWrapper addTransparentSpacerOfSize: (0 @ anInteger).
			headWrapper addMorphBack: header.
			self addMorph: headWrapper]
		ifFalse:
			[self addMorph: header]! !


Player removeSelector: #assureExternalName!
