'From Squeak 2.3 of January 14, 1999 on 16 March 1999 at 9:00:49 pm'!
"Change Set:		AcornFixes
Date:			19 February 1999
Author:			Tim Rowledge

A recent change from Bill Dargel broke the sh-cmd-keys on Acorn since it removed the entries for the plain ascii values of the keys and replaced them with the shifted (ascii - 32) values. This change simply adds the plain values back in. It avoids a VM change and re-release.

Also defines AcornFileDirectory>>fullNameFor:
"!


!AcornFileDirectory methodsFor: 'file name utilities' stamp: 'TPR 2/1/1999 12:22'!
fullNameFor: fileName
	"Return a corrected, fully-qualified name for the given file name. If the given name is already a full path (i.e., it contains a $ or other root character), assume it is already a fully-qualified name. Otherwise, prefix it with the path to this directory. In either case, correct the local part of the file name."
	"Details: Note that path relative to a directory, such as '../../foo' are disallowed by this algorithm."

	| correctedLocalName prefix |
	FileDirectory splitName: fileName to:
		[:filePath :localName |
			correctedLocalName _ self checkName: localName fixErrors: true.
			filePath isEmpty
				ifTrue: [prefix _ pathName]  "path to this directory"
				ifFalse: ["filePath might be rooted or might be relative to my pathName"
					(filePath includes: $$ )
						ifTrue:[prefix _ filePath]"path supplied with fileName"

						ifFalse:[prefix _ pathName, self pathNameDelimiter asString, filePath]"path to this directory plus path given in file name"]].  
	prefix isEmpty
		ifTrue: [^ correctedLocalName]
		ifFalse: [^ prefix, self pathNameDelimiter asString, correctedLocalName].
! !


!ParagraphEditor class methodsFor: 'keyboard shortcut tables' stamp: 'TPR 2/19/1999 13:38'!
initializeShiftCmdKeyShortcuts
	"Initialize the shift-command-key (or control-key) shortcut table."
	"NOTE: if you don't know what your keyboard generates, use Sensor kbdTest"
	"wod 11/3/1998: Fix setting of cmdMap for shifted keys to actually use the 
	capitalized versions of the letters.
	TPR 2/18/99: add the plain ascii values back in for those VMs that don't return the shifted values."

	| cmdMap cmds |
	"shift-command and control shortcuts"
	cmdMap _ Array new: 256.  "use temp in case of a crash"
	cmdMap atAllPut: #noop:.
	cmdMap at: ( 1 + 1) put: #cursorHome:.			"home key"
	cmdMap at: ( 4 + 1) put: #cursorEnd:.			"end key"
	cmdMap at: ( 8 + 1) put: #forwardDelete:.			"ctrl-H or delete key"
	cmdMap at: (13 + 1) put: #crWithIndent:.			"ctrl-Return"
	cmdMap at: (27 + 1) put: #selectCurrentTypeIn:.	"escape key"
	cmdMap at: (28 + 1) put: #cursorLeft:.			"left arrow key"
	cmdMap at: (29 + 1) put: #cursorRight:.			"right arrow key"
	cmdMap at: (30 + 1) put: #cursorUp:.				"up arrow key"
	cmdMap at: (31 + 1) put: #cursorDown:.			"down arrow key"
	cmdMap at: (45 + 1) put: #changeEmphasis:.		"cmd-sh-minus"
	cmdMap at: (61 + 1) put: #changeEmphasis:.		"cmd-sh-plus"
	cmdMap at: (127 + 1) put: #forwardDelete:.		"del key"

	"Note: Command key overrides shift key, so, for example, cmd-shift-9 produces $9 not $("
	'9[,''' do: [ :char | cmdMap at: (char asciiValue + 1) put: #shiftEnclose: ].	"({< and double-quote"
	"Note: Must use cmd-9 or ctrl-9 to get '()' since cmd-shift-9 is a Mac FKey command."
	cmdMap at: (27 + 1) put: #shiftEnclose:.	"ctrl-["

	cmds _ #(
		$a	argAdvance:
		$b	browseItHere:
		$c	compareToClipboard:
		$d	duplicate:
		$e	methodStringsContainingIt:
		$f	displayIfFalse:
		$j	doAgainMany:
		$k	changeStyle:
		$n	referencesToIt:
		$r	indent:
		$l	outdent:
		$s	search:
		$t	displayIfTrue:
		$u	changeLfToCr:
		$v	pasteInitials:
		$w	methodNamesContainingIt:
		$x	makeLowercase:
		$y	makeUppercase:
		$z	makeCapitalized:
	).
	1 to: cmds size by: 2 do: [ :i |
		cmdMap at: ((cmds at: i) asciiValue + 1) put: (cmds at: i + 1).		"plain keys"
		cmdMap at: ((cmds at: i) asciiValue - 32 + 1) put: (cmds at: i + 1).		"shifted keys"
		cmdMap at: ((cmds at: i) asciiValue - 96 + 1) put: (cmds at: i + 1).		"ctrl keys"
	].
	ShiftCmdActions _ cmdMap.! !


"Postscript:
reinitialize the shift cmd key table"
ParagraphEditor initializeShiftCmdKeyShortcuts.
!

