'From Squeak3.1alpha of 28 February 2001 [latest update: #4303] on 4 September 2001 at 8:28:49 am'!
"Change Set:		startAllEtc-sw
Date:			4 September 2001
Author:			Scott Wallace

Adds etoy scripting commands startAll:, pauseAll: and stopAll:, which serve to change the status of a given script in all sibling instances of an object by a single gesture."!


!Morph class methodsFor: 'scripting' stamp: 'sw 9/4/2001 06:42'!
additionsToViewerCategoryMiscellaneous

	^#(
		miscellaneous 
		(
			(command doMenuItem: 'do the menu item' menu)
			(command show 'make the object visible')
			(command hide 'make the object invisible')
			(command wearCostumeOf: 'wear the costume of...' player)
			(command doScript: 'run the given script once, on the next tick' string)
			(command startScript: 'start the given script ticking' string)
			(command stopScript: 'make the given script be "normal"' string)
			(command pauseScript: 'make the given script be "paused"' string)


			(command startAll: 'start the given script ticking in the object and all of its siblings.' string)
			(command stopAll: 'make the given script be "normal" in the object and all of its siblings' string)
			(command pauseAll: 'make the given script be "paused" in the object and all of its siblings' string)

			(command tellAllSiblings: 'send a message to all siblings' string)
			(command fire 'trigger any and all of this object''s button actions')
			(slot copy 'returns a copy of this object' player readOnly player getNewClone	 unused unused)
			(slot elementNumber 'my index in my container' number readWrite player getIndexInOwner player setIndexInOwner:)
			(slot holder 'the object''s container' player readOnly player getHolder player setHolder:)
			(command stampAndErase 'add my image to the pen trails and go away')
		)
	)

! !


!Player methodsFor: 'slots-kernel' stamp: 'sw 9/4/2001 07:26'!
usableMethodInterfacesIn: methodInterfaceList
	"Filter the list given by methodInterfaceList, to remove items inappropriate to the receiver"

	self hasCostumeThatIsAWorld ifTrue:
		[^ methodInterfaceList select: [:anInterface |
			#(append: beep: clearTurtleTrails doScript: getColor "color" getCursor "cursor" deleteCard doMenuItem emptyScript firstPage goToFirstCardInBackground goToFirstCardOfStack goToLastCardInBackground goToLastCardOfStack goToNextCardInStack goToPreviousCardInStack initiatePainting insertCard  liftAllPens lowerAllPens getMouseX getMouseY "mouseX mouseY" pauseScript: reverse roundUpStrays shuffleContents startScript: stopScript: unhideHiddenObjects getValueAtCursor "valueAtCursor"
startAll: pauseAll: stopAll:)
includes: anInterface selector]].

	self hasAnyBorderedCostumes ifTrue: [^ methodInterfaceList].

	^ self hasOnlySketchCostumes
		ifTrue:
			[methodInterfaceList select: [:anInterface | (#(getColor getBorderColor getBorderWidth) includes: anInterface selector) not]]
		ifFalse:
			[methodInterfaceList select: [:anInterface | (#(getBorderColor getBorderWidth) includes: anInterface selector) not]]! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 9/4/2001 07:26'!
usablePhraseSpecsIn: aListOfTuples
	"Filter the list given by aListOfTuples, to remove items inappropriate to the receiver"

	self hasCostumeThatIsAWorld ifTrue:
		[^ aListOfTuples select: [:tuple |
			#(beep doMenuItem color doScript: startScript: stopScript: pauseScript: liftAllPens lowerAllPens clearTurtleTrails initiatePainting cursor valueAtCursor mouseX mouseY roundUpStrays unhideHiddenObjects startAll: pauseAll: stopAll:) includes: tuple second]].

	self hasAnyBorderedCostumes ifTrue: [^ aListOfTuples].

	^ self hasOnlySketchCostumes
		ifTrue:
			[aListOfTuples select: [:tuple | (#(color borderColor borderWidth) includes: tuple second) not]]
		ifFalse:
			[aListOfTuples select: [:tuple | (#(borderColor borderWidth) includes: tuple second) not]]
! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/4/2001 06:56'!
existingScriptInstantiationForSelector: scriptName
	"Answer the existing script instantiation for the given selector, or nil if none"

	scriptName ifNil: [^ nil].
	Symbol hasInterned: scriptName ifTrue:
		[:sym |
			^ self actorState instantiatedUserScriptsDictionary at: sym ifAbsent: [nil]].
	^ nil! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/4/2001 07:42'!
assignStatus: newStatus toAllFor: scriptName
	"Change the status of my script of the given name to be as specified in me and all of my siblings."

	| aWorld |
	(self existingScriptInstantiationForSelector: scriptName) ifNotNilDo:
		[:scriptInstantiation |
				scriptInstantiation status: newStatus.
				scriptInstantiation assignStatusToAllSiblings.
				^ (aWorld _ self costume world) ifNotNil:
					[aWorld updateStatusForAllScriptEditors]]! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/4/2001 07:23'!
pauseAll: scriptName
	"Change the status of my script of the given name to be #paused in me and all of my siblings."

	self assignStatus: #paused toAllFor: scriptName! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/4/2001 06:59'!
startAll: scriptName
	"Change the status of my script of the given name to be #ticking in me and all of my siblings."

	self assignStatus: #ticking toAllFor: scriptName! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/4/2001 07:24'!
stopAll: scriptName
	"Change the status of my script of the given name to be #normal in me and all of my siblings."

		self assignStatus: #normal toAllFor: scriptName! !


!Vocabulary class methodsFor: 'class initialization' stamp: 'sw 9/4/2001 07:30'!
addKiswahiliVocabulary
	"Add a Kiswahili vocabulary"
	"Vocabulary addKiswahiliVocabulary"

	| voc |
	self removeVocabularyNamed: 'Kiswahili'.
	self removeVocabularyNamed: 'Kiswahili - EToy'.
	voc _ EToyVocabulary new vocabularyName: 'Kiswahili - EToy'.
	self addVocabulary: voc.
	voc translateMethodInterfaceWordings: #(

(append:					'tia mwishoni'				'weka kitu hicho mwishoni')
(beep:						'fanya kelele'  				'piga kelele fulani')
(bounce:					'ruka duta'					'ruka duta kama mpira')
(cameraPoint 				'penye kamera'				'mahali penya kamera') 
(clear 						'kumba'						'ondoa vilivyokwemo')
(clearOwnersPenTrails		'ondoa nyayo'				'ondoa nyayo za wino')
(clearTurtleTrails 			'ondoa nyayo ndani' 		'ondoa nyayo za wino zilzo ndani')
(color:sees: 					'rangi yaona rangi'			'kama rangi fulana yaona rangi nyingine')
(deleteCard 					'tupa karata'				'tupa karata hii')
(doMenuItem: 				'fanya uchaguzi'			'fanya uchaguzi fulani')
(doScript:					'piga script'					'piga script ya jina fulani mara moja')
(emptyScript 				'script tupu'				'tengeneza script mpya tupu')
(fire						'waka'						'waka script, yaani kuianzisha')
(firstPage 					'nenda mwanzoni'			'nenda penye ukurasa wa kwanza')
(followPath					'fuata njia'					'fuata njia iliyofanywa kabla')
(forward: 					'nenda mbele'				'sogea mbela kwa kiasi fulani')
(goToFirstCardInBackground	'endea kwanza ya nyuma'	'endea karata kwanza ya nyuma')
(goToFirstCardOfStack		'endea kwanza ya stack'	'endea karata iliyo ya kwanza ya stack')
(goToLastCardInBackground	'endea mwisho ya nyuma'	'endea karata ya mwisho ya nyuma')
(goToLastCardOfStack		'endea mwisho ya stack'		'endea karata ya mwisho ya stack')
(goToNextCardInStack		'endea karata ifuatayo'		'endea karata itakayofuata penye stack')
(goToPreviousCardInStack	'endea karata itanguliayo'	'endea karata kliyonitangulia penye stack')
(goToRightOf:				'endea karibu ya kulia'		'sogea hata nipo upande wa kulia kuhusu kitu fulani')
(goto:						'endea mahali fulani'		'endea mahali fulani')
(hide					'ficha'							'nifanywe ili nisionekane')
(initiatePainting			'anza kupiga picha'				'anza kupiga picha mpya')
(insertCard				'weka karata mpya'				'weka karata mpya ndani ya stack')
(lastPage				'ukurasa wa mwisho'			'endea ukurasa ya mwisho')
(liftAllPens				'inua kalamu zote'				'inua kalamu zote zilizomo ndani, ili zisipige rangi')
(loadSineWave			'pakia wimbi la sine'			'pakia wimibi (la kitrigonometry) la sine')
(loadSound:				'pakia kelele'					'pakia kelele fulani')
(lowerAllPens			'telemsha kalamu zote'			'telemsha kalamu zote ya vitu vyote vilivyomo ndani')
(makeNewDrawingIn:	'anza kupiga picha kiwanjani'	'anza kupaga picha mpya ndani ya kiwanja')
(moveToward:			'nenda upande wa'				'nenda upande wa kitu fulani')
(nextPage				'endea ukurasa ufuatao'		'nenda ukurasani unaofuata')
(pauseScript:			'pumzisha script'				'pumzisha script fulani')
(pauseAll:				'pumzisha script zote'			'pumzisha script fulani katika mwenyewe na ndugu zangu wote')
(play					'cheza'							'cheza, basi!!')
(previousPage			'endea ukurasa uliotangulia'	'enda ukurasa uliotangulia ukurusa huu')
(removeAll				'ondoa vyote vilivyokuwemo'	'ondoa vitu vyote vilvyomo dani')
(reverse				'kinyume'						'kinyume cha upande')
(roundUpStrays			'kusanya'						'sanya vitu vilovyopotoleka')
(seesColor:				'yaona rangi'					'kama naona rangi fulani')
(show					'onyesha'						'fanya hata naonekana')
(shuffleContents			'changanya'					'changanya orodha ya ndani')
(stampAndErase			'piga chapa na kufuta'			'piga chapa, halafu kufuta')
(startScript:				'anzisha script'					'anzisha script ya jina fulani')
(startAll:				'anzisha script zote'				'anzisha script fulani penye mwenyewe na ndugu zangu wote')
(stopAll:					'simamisha script zote'			'simamisha script fulani penye mwenyewe na ndugu zangu wote')
(stopScript:				'simamisa skriptu'				'simamisha script ya jana fulani')
(tellAllSiblings:			'watangazie ndugu'				'tangaza habari kwa ndugu zangu wote')
(touchesA:				'yagusa'						'kama nagusa kitu cha aina fulani')
(turn:					'geuka'							'geuka kwa pembe fulani')
(unhideHiddenObjects	'onyesha vilivyofichwa' 		'onyesha vitu ndani vilivyofichwa')
(wearCostumeOf: 		'vaa nguo za'					'vaa nguo za mtu mwingine')
(wrap					'zunguka'						'baada ya kutoka, ingia n''gambo')

(getActWhen 			'waka kama'			'lini ya waka')
(getAllButFirstCharacter	'herufi ila ya kwanza' 'herufi zote isipokuwa ile ya kwanza tu')
(getAmount				'kiasi'					'kiasi gani')
(getAngle				'pembe'					'pembe iliyopo (degree)')
(getBorderColor			'rangi ya mpaka'		'rangi ya mpaka wangu')
(getBorderWidth			'upana wa mpaka'		'upana wa mpaka wangu')
(getBottom 				'chini'					'chini yangu')
(getBrightnessUnder		'mng''aro chini'			'mwangaza chini yangu')
(getCharacters			'herufi'					'herufi zangu')
(getColor				'rangi' 					'rangi yangu')
(getColorUnder 			'rangi chini'			'rangi chini yangu')
(getConePosition			'penye cone'			'mahali penye cone')
(getCursor				'kidole'					'namba ya kitu ndani kilichagulwa')
(getDescending			'kama yaenda chini'	'kama naonyesha vitu chini')
(getDistance				'urefu'					'urefu kutoka asili')
(getFirstCharacter		'herufi ya kwanza'		'herufi yangu ya kwanza')
(getFirstElement			'kitu cha kwanza'		'kitu changu cha ndani cha kwanza')
(getFogColor				'rangi ya ukungu'		'rangi ya ukungu wangu')
(getFogDensity			'nguvu wa ukungu'	'nguvu ya ukungu wangu')
(getFogRangeEnd		'mwisho wa ukungu'	'mwisho wa upana wa ukungu wangu')
(getFogRangeStart		'mwanzo wa ukungu'	'mwanzo wa upana wa ukungu wangu')
(getFogType				'aina ya ukungu'		'aina ya ukungu wangu')
(getGraphic				'picha'					'picha ninayonyesha')
(getGraphicAtCursor		'picha penye kidolee'	'picha iliyopo penye kidole changu')
(getHeading				'upande'				'upande gani ninayoelekea')
(getHeight				'urefu'					'urefu wangu')
(getHolder				'mshikoi'				'kitu niliomo ndani yake')
(getIndexInOwner		'namba kataki mwenyeji' 'namba niliyo nayo katika mwenyeji')
(getIsUnderMouse		'chini kipanya'			'kama nipo chini ya kipanya')
(getKnobColor			'rangi ya ndani'		'rangi ya sehemu yangu ya ndani')
(getLabel				'tangazo'				'iliyoandishwa juu yangu')
(getLastValue			'mapimo'				'iliyokuwemo ndani')
(getLeft					'kushoto'				'mpaka wa kushoto')
(getLeftRight			'kiasi cha sawasawa'	'kiasi cha kushoto ama kulia')
(getLuminanceUnder	'uNg''aa chini'			'uNg''aa ya sehemu chini yangu')
(getMaxVal				'kiasi cha juu'			'kiasi cha juu humu ndani')
(getMinVal				'kiasi cha chini'		'kiasi cha chini humu ndani')
(getMouseX				'x ya kipanya'			'mahali pa x pa kipanya')
(getMouseY				'y ya kipanya'			'mahali pa y pa kipanya')
(getNewClone			'nakala'				'fanya nakala yangu')
(getNumberAtCursor		'namba kidoleni'		'namba iliyopo kidoleni')
(getNumericValue		'namba humu'			'namba iliyopo katika kituc hicho')
(getObtrudes				'jiingiliza'				'kama kitu hicho hujiingiliza')
(getPenColor				'rangi ya kalamu'		'rangi ninayotumia kwa kalamu')
(getPenDown			'kalamu chini'			'kama kalamu hukaa chini')
(getPenSize				'upana wa kalamu'		'urefu wa kalamu ninayotumia')
(getRight				'kulia'					'mpaka wa kulia')
(getRoundedCorners		'viringisha'				'tumia pembe zilizoviringishwa')
(getSampleAtCursor		'kiasi kidoleni'			'kiasi kilichopo kidoleni')
(getSaturationUnder		'kunyewesha chini'		'kiasi cha kunyewesha chini ya kati yangu')
(getScaleFactor			'kuzidisha kwa'			'kiasi ninachozidishwa nacho')
(getTheta				'theta'					'pemba kwa x-axis')
(getTop					'juu'					'mpaka wa juu')
(getTruncate			'kata'					'kama kukata ama sivyo')
(getUpDown				'juu/chini'				'kiasi cha juu ama cha chini')
(getValueAtCursor		'mchezaji kidoleni'		'mchechazji aliyepo kidoneni')
(getViewingByIcon 		'angalia kwa picha'		'kama vitu vilivyomo ndani huanagaliwa kwa picha ama sivyo')
(getX					'x'						'mahali pa x')
(getY					'y'						'mahali ya y')
(getWidth				'upana'					'upana wangu')).

	voc addToTranslationTableFrom: #(
(:						'_'						'tia ndani')
(Incr:					'pamoja na'				'tia thamani + fulani')
(Decr:					'toa'					'tia thamani - fulani')
(Mult:					'mara'					'tia thamani * fulani')).

	voc translateCategories: #(
(basic					muhimu				'mambo muhimu muhimu')
(#'book navigation'		#'kuongoza vitabu'		'kuhusu kuongozea vitabu')
(button					kifungo					'mambo kuhusu vifungo')
(collections				mikusanyo				'kuhusu mikusanyo ya vitu')
(fog					ukungu					'kuhusu ukungu (3D)')
(geometry				kupimia					'urefu na kadhaliki')
(#'color & border'		#'rangi & mpaka'		'kuhusu rangi na mpaka')
(graphics				picha					'mambo kuhusu picha')
(#'instance variables'	badiliko					'data zilizoundwa na yule atumiaye')
(joystick				#'fimbo la furaha'		'kuhusu fimbo la furaha, yaani "joystick"')
(miscellaneous			mbalimbali				'mambo mbalimbali')
(motion					kusogea					'kwenda, kuegeuka, etc.')
(paintbox				#'kupiga rangi'			'vitu kuhusu kupigia rangi')
(#'pen trails'			#'nyayo za kalamu'		'kuhusu nyay za kalamu')
(#'pen use'				#'kalamu'				'kuhusu kalamu')
(playfield				kiwanja					'vitu kuhusu kiwanjani')
(sampling				kuchagua				'mambo kuhusu kuchagua')
(scripts					taratibu				'taratibu zilizoundwa na atumiaye')
(slider					telezo					'kitu kionyeshacho kiasi cha namba fulani')
(speaker				spika					'kuhusu spika za kelele')
(#'stack navigation'		#'kuongoza chungu'	'kuhuso kuongozea chungu')
(storyboard				kusimulia				'kusimilia hadithi')
(tests					kama					'amua kama hali fulani i kweli ama sivyo')
(text					maneno					'maandiko ya maneno')
(viewing				kuangaliwa				'kuhusu kuangalia vitu')
).
! !

"Postscript:"
StandardScriptingSystem initialize.
Vocabulary initialize.!

