'From Squeak3.8alpha of ''17 July 2004'' [latest update: #5976] on 12 August 2004 at 6:49:49 pm'!
"Change Set:		sqRelease
Date:			12 August 2004
Author:			Michael Rueger

Methods for making the squeakland plugin image."!


!StandardScriptingSystem class methodsFor: 'utilities' stamp: 'mir 8/3/2004 22:39'!
applyNewEToyLook
	"Apply the new EToy look based on free fonts, approximating the classic look as closely as possible."

	"StandardScriptingSystem applyNewEToyLook"

	| aTextStyle aFont | 
	aTextStyle _ TextStyle named: #BitstreamVeraSansMono.
	aFont _ aTextStyle fontOfSize: 12.
	aFont _ aFont emphasis: 1.
	Preferences setEToysFontTo: aFont.
	Preferences setButtonFontTo: aFont.

	aTextStyle _ TextStyle named: #Accushi.
	aFont _ aTextStyle fontOfSize: 12.
	Preferences setFlapsFontTo: aFont.

	(aTextStyle _ TextStyle named: #Accuny)
		ifNotNil:
			[Preferences setSystemFontTo: (aTextStyle fontOfSize: 12)]! !


!SystemDictionary methodsFor: 'squeakland' stamp: 'mir 5/7/2002 13:59'!
makeSqueaklandRelease
	"Smalltalk makeSqueaklandRelease"

	Smalltalk
		makeSqueaklandReleasePhasePrepare;
		makeSqueaklandReleasePhaseStripping;
		makeSqueaklandReleasePhaseFinalSettings;
		makeSqueaklandReleasePhaseCleanup! !

!SystemDictionary methodsFor: 'squeakland' stamp: 'mir 8/3/2004 14:01'!
makeSqueaklandReleasePhaseCleanup
	"Smalltalk makeSqueaklandReleasePhaseCleanup"

	Browser initialize.
	ChangeSorter removeChangeSetsNamedSuchThat:
		[:cs| cs name ~= ChangeSet current name].
	ChangeSet current clear.
	ChangeSet current name: 'Unnamed1'.
	Smalltalk garbageCollect.
	"Reinitialize DataStream; it may hold on to some zapped entitities"
	DataStream initialize.
	"Remove existing player references"
	References keys do:[:k| References removeKey: k].

	Smalltalk garbageCollect.
	ScheduledControllers _ nil.
	Behavior flushObsoleteSubclasses.
	Smalltalk garbageCollect; garbageCollect.
	Smalltalk obsoleteBehaviors isEmpty ifFalse:[self error:'Still have obsolete behaviors'].

	"Reinitialize DataStream; it may hold on to some zapped entitities"
	DataStream initialize.
	Smalltalk fixObsoleteReferences.
	Smalltalk abandonTempNames.
	Smalltalk zapAllOtherProjects.
	Smalltalk forgetDoIts.
	Smalltalk flushClassNameCache.
	3 timesRepeat: [
		Smalltalk garbageCollect.
		Symbol compactSymbolTable.
	].
! !

!SystemDictionary methodsFor: 'squeakland' stamp: 'mir 8/3/2004 22:34'!
makeSqueaklandReleasePhaseFinalSettings
	"Smalltalk makeSqueaklandReleasePhaseFinalSettings"

	| serverName serverURL serverDir updateServer highestUpdate newVersion |

	ProjectLauncher splashMorph: (FileDirectory default readOnlyFileNamed: 'scripts\SqueaklandSplash.morph') fileInObjectAndCode.

	"Dump all morphs so we don't hold onto anything"
	World submorphsDo:[:m| m delete].

	#(
		(honorDesktopCmdKeys false)
		(warnIfNoChangesFile false)
		(warnIfNoSourcesFile false)
		(showDirectionForSketches true)
		(menuColorFromWorld false)
		(unlimitedPaintArea true)
		(useGlobalFlaps false)
		(mvcProjectsAllowed false)
		(projectViewsInWindows false)
		(automaticKeyGeneration true)
		(securityChecksEnabled true)
		(showSecurityStatus false)
		(startInUntrustedDirectory true)
		(warnAboutInsecureContent false)
		(promptForUpdateServer false)
		(fastDragWindowForMorphic false)

		(externalServerDefsOnly true)
		(allowCelesteTell false)
		(eToyFriendly true)
		(eToyLoginEnabled true)
		(magicHalos true)
		(mouseOverHalos true)
		(includeSoundControlInNavigator true)
		(readDocumentAtStartup true)
		(preserveTrash true)

	) do:[:spec|
		Preferences setPreference: spec first toValue: spec last].
	"Workaround for bug"
	Preferences enable: #readDocumentAtStartup.

	World color: (Color r: 0.9 g: 0.9 b: 1.0).

	"Clear all server entries"
	ServerDirectory serverNames do: [:each | ServerDirectory removeServerNamed: each].
	SystemVersion current resetHighestUpdate.

	"Add the squeakalpha update stream"
	serverName _ 'Squeakalpha'.
	serverURL _ 'squeakalpha.org'.
	serverDir _ serverURL , '/'.

	updateServer _ ServerDirectory new.
	updateServer
		server: serverURL;
		directory: 'updates/';
		altUrl: serverDir;
		user: 'sqland';
		password: nil.
	Utilities updateUrlLists addFirst: {serverName. {serverDir. }.}.

	"Add the squeakland update stream"
	serverName _ 'Squeakland'.
	serverURL _ 'squeakland.org'.
	serverDir _ serverURL , '/'.

	updateServer _ ServerDirectory new.
	updateServer
		server: serverURL;
		directory: 'public_html/updates/';
		altUrl: serverDir.
	Utilities updateUrlLists addFirst: {serverName. {serverDir. }.}.

	highestUpdate _ SystemVersion current highestUpdate.
	SystemVersion current highestUpdate: 0.

	newVersion _ FillInTheBlank request: 'New version designation:' initialAnswer: 'Squeakland 3.8.' , highestUpdate printString. 
	SystemVersion newVersion: newVersion.
	(self confirm: self version , '
Is this the correct version designation?
If not, choose no, and fix it.') ifFalse: [^ self].
! !

!SystemDictionary methodsFor: 'squeakland' stamp: 'mir 8/3/2004 13:46'!
makeSqueaklandReleasePhasePrepare
	"Smalltalk makeSqueaklandReleasePhasePrepare"

	Undeclared removeUnreferencedKeys.
	StandardScriptingSystem initialize.
	Preferences initialize.
	(Object classPool at: #DependentsFields) size > 1 ifTrue: [self error:'Still have dependents'].
	Undeclared isEmpty ifFalse: [self error:'Please clean out Undeclared'].

	"Dump all projects"
	Project allSubInstancesDo:[:prj| prj == Project current ifFalse:[Project deletingProject: prj]].

	Browser initialize.
	ScriptingSystem deletePrivateGraphics.
	ChangeSorter removeChangeSetsNamedSuchThat:
		[:cs| cs name ~= ChangeSet current name].
	ChangeSet current clear.
	ChangeSet current name: 'Unnamed1'.
	Smalltalk garbageCollect.
	"Reinitialize DataStream; it may hold on to some zapped entitities"
	DataStream initialize.
	"Remove existing player references"
	References keys do:[:k| References removeKey: k].

	Smalltalk garbageCollect.
	ScheduledControllers _ nil.
	Smalltalk garbageCollect.
! !

!SystemDictionary methodsFor: 'squeakland' stamp: 'mir 8/3/2004 14:01'!
makeSqueaklandReleasePhaseStripping
	"Smalltalk makeSqueaklandReleasePhaseStripping"

	#(Helvetica Palatino Courier) do:
		[:n | TextConstants removeKey: n ifAbsent: []].
	Smalltalk at: #Player ifPresent:[:superCls|
		superCls allSubclassesDo:[:cls|
			cls isSystemDefined ifFalse:[cls removeFromSystem].
			cls _ nil]].
	Smalltalk garbageCollect.
	Smalltalk
		discardFFI;
"		discardMVC;"
		discardSUnit;
		discardSpeech;
		yourself.

	SystemOrganization removeEmptyCategories.

	Smalltalk garbageCollect.
	ScheduledControllers _ nil.
	Behavior flushObsoleteSubclasses.
	Smalltalk garbageCollect; garbageCollect.
	DataStream initialize.
	Smalltalk fixObsoleteReferences! !

"Postscript:
Apply the changed etoy look.
"

StandardScriptingSystem applyNewEToyLook.
!